# http://morfeo-project.org
#
# Component: EzForge
# Programming Language: Python
#
# File: EzForgeLayer/utils.py
# Version: 1.0
# Date: 17-10-2007
# Last modified by: Luis de la Fuente Dominguez
#
# (C) Copyright 2007 Telefonica Investigacion y Desarrollo
# S.A.Unipersonal (Telefonica I+D)
#
# See CREDITS file for info about members and contributors.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# If you want to use this software an plan to distribute a
# proprietary application in any way, and you are not licensing and
# distributing your source code under GPL, you probably need to
# purchase a commercial license of the product. More info about
# licensing options is available at:
#
# http://morfeo-project.org/EzForge/Licensing
from django.http import HttpResponse
from django.template import Context, loader

from config.properties import FORMAT, XML_FORMAT, JSON_FORMAT, HTML_FORMAT
from commons.utils import json_encode
from config import properties

import sys

#Constants declaration
JSON_MIMETYPE = "application/json; charset=UTF-8"
XML_MIMETYPE = "text/xml"
HTML_MIMETYPE = "text/html"

# Since render_to_response shortcut in Django 0.96 doesn't accept the argument mimetype, we use an own shortcut that allows it.
# When we change to Django SVN version, we just need to delete this method in this file, and uncomment the line 
# 'from django.shortcuts import render_to_response' in the rest of files
def render_to_response (template, context, mimetype):
    t = loader.get_template(template)
    c = Context(context)
    return HttpResponse(t.render(c), mimetype)


#returns the url base for EzForge forge
#parameters are obtained from the config file (config/properties.py) 
def getEzForgeBaseURI():
    #build EzForge base url from config file
    protocol = properties.protocol
    host = properties.host
    port = properties.port
    context = properties.context
    version = properties.version
    ezforge_base_url = protocol + '://' + host + ':' + port + '/' + context + '/' + version
    return ezforge_base_url


#returns a string indicating if the xml received as parameter
#contains any error
def getErrorMessage(xml_doc):
    #test if xml contains an error element    
    errors = xml_doc.getElementsByTagName('error')        
    
    if len(errors)>0:#backend system reports an error situation. An string with the reason is returned
        sys.__stdout__.write('Backend system has reported an error\n')
        msg = 'ERROR REPORTED BY BACKEND SYSTEM: '
        msg += xml_doc.getElementsByTagName('message')[0].firstChild.data
        return msg
    else:#backend system reports an ok situation. An empty string is returned
        return ''                        
            

#returns format due to the 'format' param extracted from the request
def getResponseFormat(request):
    try:
        format = request.GET [FORMAT]           
        if format == JSON_FORMAT:
            return JSON_FORMAT                                   
        else:# XML format. v 1.0: only two formats are possible [JSON | XML]                        
            return XML_FORMAT
    except:#if format param is missing then default format (XML)is used          
        return XML_FORMAT


#render the appropriate response  
def generateResponse (format, data_hash, template):            
    if format == JSON_FORMAT:
        return HttpResponse(json_encode(data_hash), JSON_MIMETYPE)                                   
    else:# XML format. v 1.0: only two formats are possible [JSON | XML]. Special case is HTML format  
        t = loader.get_template(template)
        c = Context(data_hash)
        mime_type = XML_MIMETYPE
        if (format == HTML_FORMAT):
            mime_type = HTML_MIMETYPE                    
        return HttpResponse(t.render(c), mime_type)
              
            
              
            
            
            
            
            
            
            
    
