#
#  Copyright (C) 2009 GSyC/LibreSoft
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#    Author : Jose Antonio Santos Cadenas <jcaden __at__ gsyc __dot__ es>
#



# This file defines the api that must be used instead of accessing 
#  directly to the data models.
import sys

from django.contrib.auth.models import User
from django.contrib.gis.geos import Point 
from django.contrib.gis.measure import D 

from datetime import datetime

from models import *
from utils import *

from PIL import Image
import StringIO

def get_all_nodes():
    """
        Returns an array with all the Social_nodes
    """
    try:
        nodes_mod=Social_node.objects.all()
        nodes=[]
        for node in nodes_mod:
            n=change_node(node)
            nodes+=[n]
        return nodes
    except:
        return []

def get_node_data(node_id):
    """
        Returns the details for a node
    """
    try:
        n=Social_node.objects.get(id=node_id)
        node=change_node(n)
        return node
    except:
        return None
    
def get_all_notes():
    """
        Returns an array with all the Notes
    """
    try:
        notes_mod=Note.objects.all()
        notes=[]
        for note in notes_mod:
            n=change_note(note)
            notes+=[n]
        return notes
    except:
        return []

def get_note_data(note_id):
    """
        Returns the details for a note
    """
    try:
        n=Note.objects.get(id=note_id)
        note=change_note(n)
        return note
    except:
        return None

def create_or_modify_user(person):
    """
        Receives a types.Person object and adds it to the database, if the
        user is already created, modifies it
    """
    try:        
        p=None
        point = Point(float(person.longitude), float(person.latitude), srid=4326)
        try:
            p=Person.objects.get(username=person.username)
            p.first_name=person.first_name
            p.last_name=person.last_name
            p.email=person.email
            p.is_active=True
            p.position = point
            p.radius = person.radius
        except:
            p=Person(username=person.username, first_name=person.first_name,
                   last_name=person.last_name, password=person.password,
                   email=person.email, is_active=True, position=point, radius=person.radius)
        
        p.set_password( person.password )
        p.pos_time = datetime.now()
        p.save()
        return True, "No error"
    except:
        return False, "Unknown error"

def create_note(title, text, latitude, longitude, owner=None):
    try:
        p=None
        point = Point(float(longitude), float(latitude), srid=4326)
        try:
            p=Person.objects.get(pk=owner)
        except:
            pass
        try:
            note=Note(title = title, text = text, position=point);
            note.save()
        except:
            return False, "Note can't be created"
        if p != None:
            try:
                w=Writer(node1=p, node2=note)
                w.save()
            except:
                return False, "Can't associate user with new note"

        return True, "No error"
    except:
        return False, "Unknown error"

def set_user_status(userid, status):
    """
        Changes the user status
    """
    try:
        p=Person.objects.get(pk=userid)
        p.status=status
        p.save()
        return True, "No error"
    except:
        return False, "The user doesn't exist"

def set_coordinates(nodeid, latitude, longitude, radius=0.0):
    """
        Sets the position of any type of social_node
    """
    try:
        n=Social_node.objects.get(pk=nodeid)
        n.position = Point(float(longitude), float(latitude), srid=4326)
        n.radius = float(radius)
        n.pos_time = datetime.now()
        n.save()
        return True, "No error"
    except:
        return False, "The node doesn't exist"

def get_all_friends(userid):
    """
        Returns all friends for the user indicated
    """
    try:
        p=Person.objects.get(pk=userid)
        friends = Person.objects.distance(p.position).order_by("distance").filter(
                  pk__in=Friendship.objects.get_relations_for(p))
        persons=[]
        for person in friends:
            try:
                p=change_person(person)
                persons+=[p]
            except:
                pass
        return persons
    except:
        return []

def get_nearby_friends(userid, radius):
    """
        Finds friends in a radius (in km) from your position
    """
    try:
        p=Person.objects.get(pk=userid)
        friends = Person.objects.distance(p.position).order_by("distance").filter(
                  position__distance_lte=(p.position, D(km=float(radius))),
                  pk__in=Friendship.objects.get_relations_for(p))                  
        persons=[]
        for person in friends:
            try:
                p=change_person(person)
                persons+=[p]
            except:
                pass
        return persons
    except:
        return []

def get_nearby_people(userid, radius):
    """
        Finds people in a radius (in km) from your position
    """
    try:
        p=Person.objects.get(pk=userid)
        people = Person.objects.distance(p.position).order_by("distance").filter(position__distance_lte=(p.position, D(km=float(radius))))
        persons=[]
        for person in people:
            try:
                p=change_person(person)
                persons+=[p]
            except:
                pass
        return persons
    except:
        return []

def get_nearby_node(nodeid, radius):
    """
        Finds people in a radius (in km) from your position
    """
    try:
        n=Social_node.objects.get(pk=nodeid)
        nodes = Social_node.objects.distance(n.position).order_by("distance").filter(position__distance_lte=(n.position, D(km=float(radius))))
        nds=[]
        for node in nodes:
            try:
                n=change_node(node)
                nds+=[n]
            except:
                pass
        return nds
    except:
        return []

def delete_user(userid):
    """
        Deletes the person with this username
    """
    try:
        p=Person.objects.get(pk=userid)
        p.delete()
        u.delete()
        return True, "No error"
    except:
        return False, "This user doesn't exist"
    
def create_friendship(person_id, friend_id):
    """
        Creates a relation between the users whose names are indicated
    """
    try:
        p1=None
        p2=None
        try:
            p1=Person.objects.get(pk=person_id)
        except:
            return False, "Person id is incorrect"
        try:
            p2=Person.objects.get(pk=friend_id)
        except:
            return False, "Friend id is incorrect"
        relation=None
        try:
            relation=Frienship.objects.get_relation(node1=p1, node2=p2)
        except:
            relation=Friendship(node1=p1, node2=p2)
        #Save more data...
        relation.save()
        return True, "No error"
    except:
        return False, "Unknown error"

def delete_friendship(person_id, friend_id):
    """
        Deletes the relationship between the users
    """
    try:
        p1=None
        p2=None
        try:
            p1=Person.objects.get(pk=person_id)
        except:
            return False, "Person doesn't exist"
        try:
            p2=Person.objects.get(pk=friend_id)
        except:
            return False, "Friend doesn't exist"
        try:
            relation=Friendship.objects.get_relation(node1=p1, node2=p2)
            relation.delete()
            return True, "No error"
        except:
            return False, "Relation doesn't exist"
    except:
        return False, "Unknown error"

def get_user_data(userid):
    """
        Returns an object with the user information
    """
    try:
        p=Person.objects.get(pk=userid)
        person=change_person(p)
        return person
    except:
        return None    

def get_all_users():
    """
        Returns an array with all the Persons
    """
    try:
        people=Person.objects.all()
        persons=[]
        for person in people:
            p=change_person(person)
            persons+=[p]
        return persons
    except:
        return []
    
def create_or_modify_group(group):
    """
        Receives a types.Group object and adds it to the database, if the
        group is already created, modifies it
    """
    try:
        g=None
        try:
            g=Group.objects.get(name=group.name)
        except:
            point = Point(float(group.longitude), float(group.latitude), srid=4326)
            g=Group(name=group.name, position=point, radius=group.radius)
        g.save()
        for person in group.people:
            try:
                p=Person.objects.get(pk=person.id)
                add_person_to_group(p, g)
            except:
                return False, "The person couldn't be joined to the group"
        return True, "No error"
    except:
        return False, "Unknown error"
    
def get_all_groups():
    """
        Returns all the groups in the database
    """
    try:
        groups=[]
        groups_obj=Group.objects.all()
        for group in groups_obj:
            groups+=[change_group(group)]
        return groups
    except:
        return []
    
def get_group_data(gid):
    """
        Returns the data for the group
    """
    try:
        g=Group.objects.get(pk=gid)
        return change_group(g)
    except:
        return None
    
def delete_group(gid):
    """
        Deletes the group indicated
    """
    try:
        g=Group.objects.get(pk=gid)
        g.delete()
        return True, "No error"
    except:
        return False, "This group doesn't exist"

def group_join(group, person):
    """
        Joins the person to the group
    """
    try:
        g=Group.objects.get(pk=group)
        try:
            p=Person.objects.get(pk=person)
            add_person_to_group(p, g)
            return True, "No error"
        except:
            return False, "The user doesn't exist"
    except:
        return False, "The group doesn't exist"

def group_unjoin (group, person):
    """
        Deletes the person from the group
    """
    try:
        g=Group.objects.get(pk=group)
        try:
            p=Person.objects.get(pk=person)
            try:
                m=Membership.objects.get_relation(node1=p, node2=g)
                m.delete()
                return True, "No error"
            except:
                return False, "This user doesn't belong to this group"
        except:
            return False, "The user doesn't exist"
    except:
        return False, "The group doesn't exist"
    
def upload_photo (userid, photo_name, photo, latitude, longitude, description):
    """
        Uploads a photo to the database accessible by a URL or PATH
    """
    try:
        try:
            p=Person.objects.get(pk=userid)
        except:
            return False, "The user doesn't exist"
        
        try:
            i=Image.open(Image.open(StringIO.StringIO(photo)))
            if not (i.format in SUPPORTED_TYPES):
                return False, "The image type is not supported"
        except IOError:
            return False, "The image type is no supported"

        
        point = Point(float(longitude), float(latitude), srid=4326)
        
        p=Photo(name=photo_name, uploader=p, position=point, description=description)
        #photo name must be random because of security issue
        p.photo.save(rand_name(), photo)
        p.save()
        
        return True, "No error"
    except:
        return False, sys.exc_value

def get_all_photos ():
    """
        Shows all photos in the database
    """
    try:
        photos_mod=Photo.objects.all()
        photos=[]
        for photo in photos_mod:
            try:
                p=change_photo(photo)
                photos+=[p]
            except:
                pass
        return photos
    except:
        return []
    
def get_photo_data (photo_id):
    """
        Returns an object with the photo data
    """
    try:
        photo=Photo.objects.get(id=photo_id)
        p=change_photo(photo)
        return p
    except:
        return None
    
def get_photo_image_path (photo_id):
    """
        Returns the path to the image
    """
    try:
        photo=Photo.objects.get(id=photo_id)
        return photo.photo.path
    except:
        return None