#
#  Copyright (C) 2009 GSyC/LibreSoft
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#    Author : Jose Antonio Santos Cadenas <jcaden __at__ gsyc __dot__ es>
#


#from django.db import models
from django.contrib.auth.models import User
from django.contrib.gis.db import models
from django.core.files.storage import FileSystemStorage


from datetime import datetime

class Tag(models.Model):
    """
        Model for tag social objects
    """
    tag = models.CharField(max_length=200, unique=True)
    
    objects = models.GeoManager()
    

class Social_node(models.Model):
    """
        General social object, can be related with any other 
        social object
    """
    position = models.PointField(srid=4326)
    radius = models.FloatField(default=0.0)
    pos_time = models.DateTimeField(default=datetime.now)
    since = models.DateTimeField(default=datetime.now)
    tags = models.ManyToManyField(Tag)
    
    objects = models.GeoManager()
    
    def __unicode__(self):
        return u"social node %s" % self.pk

class Person(Social_node, User):
    """
        Represents a person in social networks issues
    """
    status = models.CharField(max_length=500)
    objects = models.GeoManager()

    def is_friend(self, friend):
        if Friendship.objects.get_relation(self, friend).count() > 0:
            return True
        else:
            return False

    def __unicode__(self):
        return self.username

class Group(Social_node):
    """
        A group of people
    """
    name = models.CharField(max_length=200, unique=True)
    
    objects = models.GeoManager()
    
class Note(Social_node):
    """
       A note 
    """
    title = models.CharField(max_length=200)
    text = models.TextField()

    objects = models.GeoManager()
    
class Photo(Social_node):
    """
        A photo
    """
    name = models.CharField(max_length=200, unique=True)
    photo = models.ImageField(upload_to="images/photos/", blank=False,
                              null=False, max_length=200, 
                              storage=FileSystemStorage())
    description = models.TextField()
    uploader = models.ForeignKey(Person)

    objects = models.GeoManager()


class RelationManager(models.GeoManager):  
    def get_relation(self, node1, node2):
        try:
            relation = self.get(node1=node1, node2=node2)
            return relation
        except:
            try:
                relation = self.get(node1=node2, node2=node1)
                return relation
            except:
                return None
        return None
    
    def get_relations_for (self, node):
        relations = []
        try:
            for rel in self.filter(node1=node):
                relations += [rel.node2]
        except:
            pass
        try:
            for rel in self.filter(node2=node):
                relations += [rel.node1]
        except:
            pass
        return relations

class Relation(models.Model):
    """
        Represents the relation between two social_nodes
    """
    node1 = models.ForeignKey(Social_node, related_name="related")
    node2 = models.ForeignKey(Social_node, related_name="more_related")
    since = models.DateTimeField(default=datetime.now)
    
    objects = RelationManager()
    
    def __unicode__(self):
        return self.node1 + "<-->" + self.node2
      
class Friendship(Relation):
    """
        Specific relation between persons 
    """
    objects = RelationManager()
    
    
class Membership(Relation):
    """
        Specific relation between persons and groups
    """
    objects = RelationManager()
     
class Writer(Relation):
    """
        Relation between person and note
    """
    objects = RelationManager()
