#
#  Copyright (C) 2009 GSyC/LibreSoft
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#    Author : Jose Antonio Santos Cadenas <jcaden __at__ gsyc __dot__ es>
#


from datetime import datetime

class Social_node:
    """
        Represents the general social node
    """
    latitude=0.0
    longitude=0.0
    raidus=0.0
    pos_time=datetime.now
    since=datetime.now
    tags=[]
    id=-1
    
    def __init__ (self, latitude=0.0, longitude=0.0, radius=0.0, id=-1, tags=[],
                  since=datetime.now(), pos_time=datetime.now()):
        self.latitude=latitude
        self.longitude=longitude
        self.radius=radius
        self.pos_time=pos_time
        self.since=since
        self.tags=tags
        self.id = id

class Person(Social_node):
    """
        Represent a person in social networks issues
    """
    def __init__(self, username, password, first_name="", last_name="",
                email="", friends=[], groups=[], latitude=0.0,
                longitude=0.0, radius=0.0, status="", id=-1, tags=[]):
            self.username = username
            self.first_name = first_name
            self.last_name = last_name
            self.email = email
            self.password = password
            self.friends = friends
            self.groups = groups
            self.since = datetime.now
            self.latitude = latitude
            self.longitude = longitude
            self.radius = radius
            self.pos_time = datetime.now
            self.status = status
            self.tags = tags
            self.id = id
    
    def get_full_name(self):
        full_name = u'%s %s' % (self.first_name, self.last_name)
        return full_name.strip()
    
    def __unicode__(self):
        return self.username
    
class Group(Social_node):
    """
        A group of people
    """
    def __init__(self, name, people=[], longitude=0.0, latitude=0.0, radius=0.0, id=-1, tags=[]):
        self.name = name
        self.people = people
        self.since = datetime.now
        self.latitude=latitude
        self.longitude=longitude
        self.radius=radius
        self.tags=tags
        self.id = id
    
    def __unicode__(self):
        return self.name
    
class Note (Social_node):
    """
        A note
    """
    def __init__ (self, title, text, latitude, longitude, radius=0.0, id=-1, tags=[],
                  since=datetime.now, writers=[]):
        self.title = title
        self.text = text
        self.latitude = latitude
        self.longitude = longitude
        self.radius = radius
        self.since = since
        self.tags = tags
        self.id = id
        self.pos_time=datetime.now
        self.writers=writers
        
    def __unicode__(self):
        return u"note: %s", self.title
    
class Photo (Social_node):
    """
        A photo
    """
    def __init__ (self, name, url, file, uploader, latitude, longitude, description,
                  radius=0.0, id=-1, tags=[], since=datetime.now):
        self.name = name
        self.description = description
        self.url=url
        self.file=file
        self.uploader=uploader
        self.latitude=latitude
        self.longitude=longitude
        self.radius=radius
        self.id=id
        self.tags=tags
        self.since=since
        
    def __unicode__(self):
        return u"photo: %s", self.name
    