#
#  Copyright (C) 2009 GSyC/LibreSoft
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#    Author : Jose Antonio Santos Cadenas <jcaden __at__ gsyc __dot__ es>
#


import types
from models import Relation, Membership, Friendship, Writer


import hashlib
import random

def rand_name():
    m = hashlib.md5()
    m.update(str(random.random()))
    return m.hexdigest()

def change_person(person):
    """
        Changes from the model Person to the type Person
    """
    tags_objs = person.tags.all();
    tags=[]
    for tag in tags_objs:
        tags+=[tag.tag]
    p=types.Person(username=person.username,password=person.password, tags=tags)
    p.id = person.pk
    p.first_name = person.first_name
    p.last_name = person.last_name
    p.email = person.email
    p.since = person.since
    p.longitude = person.position.get_x()
    p.latitude = person.position.get_y()
    p.radius = person.radius
    p.status = person.status
    try:
        p.distance = round(person.distance.m, 2)
    except:
        pass
    #This is no more served to avoid unnecessary database load
    #friends = Friendship.objects.get_relations_for(person)
    #p.friends=[]
    #for f in friends:
        #p.friends+=[{"id":f.person.pk, "username":f.person.username}]
    #membership=Membership.objects.get_relations_for(person)
    #p.groups=[]
    #for m in membership:
        #p.groups+=[{"id":m.group.pk, "name":m.group.name}]
    return p

def change_group(group):
    """
        Changes from the model Group to the type Group
    """
    tags_objs = group.tags.all();
    tags=[]
    for tag in tags_objs:
        tags+=[tag.tag]
    g=types.Group(name=group.name, tags=tags, 
                  latitude=group.position.get_y(), longitude=group.position.get_x())
    g.since=group.since
    g.id=group.pk
    try:
        g.distance = round (group.distance.m, 2)
    except:
        pass
    #This is no more served to avoid unnecessary database load
    #people=Membership.objects.get_relations_for(group)
    #g.people=[]
    #for p in people:
    #    g.people+=[{"id":p.person.pk, "username":p.person.username}]
    return g

def change_node(node):
    """
        Changes from the model Social_node to the type Social_node
    """
    tags_objs = node.tags.all();
    tags=[]
    for tag in tags_objs:
        tags+=[tag.tag]
    n=types.Social_node(latitude=node.position.get_y(), longitude=node.position.get_x(),
                        radius=node.radius, id=node.pk, tags=tags, since=node.since)
    try:
        n.distance = round (node.distance.m, 2)
    except:
        pass
    n.type=[]
    
    try:
        if node.person.id == node.id:
            n.type+=["person"]
    except:
        pass
    try:  
        if node.group.id == node.id:
            n.type+=["group"]
    except:
        pass
    try:
        if node.note.id == node.id:
            n.type+=["note"]
    except:
        pass
    try:
        if node.photo.id == node.id:
            n.type+=["photo"]
    except:
        pass
    return n

def change_note(note):
    """
        Changes from the model Note to the type note
    """
    tags_objs = note.tags.all();
    tags=[]
    for tag in tags_objs:
        tags+=[tag.tag]
    writers_objs = Writer.objects.get_relations_for(note)
    writers=[]
    for writer in writers_objs:
        try:
            writers+=[change_person(writer.person)]
        except:
            pass
    n=types.Note(latitude=note.position.get_y(), longitude=note.position.get_x(),
                 radius=note.radius, id=note.pk, tags=tags, text=note.text, 
                 title= note.title,since=note.since, writers=writers)
    try:
        n.distance = round (note.distance.m, 2)
    except:
        pass
        
    return n

def change_photo(photo):
    """
        Changes from model Photo su type Photo
    """
    tags_objs = photo.tags.all();
    tags=[]
    for tag in tags_objs:
        tags+=[tag.tag]
    
    p=types.Photo(name=photo.name, url=photo.photo.url, file=photo.photo.path,
                  uploader=change_person(photo.uploader.person), description=photo.description,
                  latitude=photo.position.get_y(), longitude=photo.position.get_x(),
                  radius=photo.radius, id=photo.id, tags=tags, since=photo.since)
    p.url= u"/social/photo/%s/image" % p.id
    try:
        p.distance = round (photo.distance.m, 2)
    except:
        pass
    return p

def add_person_to_group(person, group, data=[]):
    """
        Receives the models Person and Group and create a new Member
    """
    if Membership.objects.get_relation(node1=person, node2=group) == None:
        member=Membership(node1=person, node2=group)
        member.save()
        return True
    else:
        return False
