#
#  Copyright (C) 2009 GSyC/LibreSoft
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#    Author : Jose Antonio Santos Cadenas <jcaden __at__ gsyc __dot__ es>
#


# Create your views here.
import sys

from format.utils import  getResponseFormat, generateResponse
from social.core.api import *
from social.core import api
from social.core.types import Person, Group
from social.rest.forms import LoginForm, PhotoForm

from django.http import HttpResponse

#from PIL import Image
#import StringIO

def error (format, message, code='500'):
    data = {'code'         : code,
            'description'  : message,}
    return generateResponse(format, data, "error.xml")

def forbbiden_photo(request):
    format = getResponseFormat (request)    
    return error(format, "You don't have permission to access here", code='405')

def user_login (request):
    format = getResponseFormat (request)

    if request.method == "POST":
       loginform = LoginForm(request.POST)
       if loginform.login(request):
           data = {'code'         : '200',
                   'description'  : 'User logged in',}
           return generateResponse(format, data, "ok.xml")
       else:
           return error(format, "User and password do not match") 
    else:
        return error(format, "Need a post petition")

def user_create (request):
    format = getResponseFormat (request)
    if request.method == "POST":
       if ("password" in request.POST) and ("username" in request.POST):
           try: 
               passwd=request.POST['password']
               user=request.POST['username']
               p=Person(username=user, password=passwd)
               #Fixing optional fields
               if ("first_name" in request.POST):
                   p.first_name=request.POST["first_name"]
               if ("last_name" in request.POST):
                   p.last_name=request.POST["last_name"]
               if ("email" in request.POST):
                   p.email=request.POST["email"]
               if ("latitude" in request.POST):
                   p.latitude=request.POST["latitude"]
               if ("longitude" in request.POST):
                   p.longitude=request.POST["longitude"]
               if ("radius" in request.POST):
                   p.radius=request.POST["radius"]  
               correct, message=create_or_modify_user(p)
               if correct:
                   data = {'code'         : '200',
                           'description'  : 'User created correctly',}
                   return generateResponse(format, data, "ok.xml")
               else:
                   return error(format, message)
           except:
               return error(format ,'Unknown error occurred')               
       else:
          return error (format, 'Missing parameters')
    else:
        return error (format, 'Need a POST petition')
    
def user_set_status(request, user):
    format = getResponseFormat (request)
    if request.method != "POST":
        return error (format, "Need a POST petition")
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    try:
        status = request.POST["status"]
    except:
        return error(format, "Needs a POST parameter 'status' with the new status")
    correct, message = set_user_status(user, status)
    if correct:
        data = {'code'         : '200',
                'description'  : 'Position updated'}
        return generateResponse(format, data, "ok.xml")
    else:
        return error(format, message)
    
def user_position (request, user):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    if request.method == "POST":
        if ("latitude" in request.POST) and ("longitude" in request.POST):
            latitude=request.POST["latitude"]
            longitude=request.POST["longitude"]
            if ("radius" in request.POST):
                radius=request.POST["radius"]
                correct, message = set_coordinates(user, latitude, longitude, radius)
            else:
                correct, message = set_coordinates(user, latitude, longitude)
            if correct:
                data = { 'code'         : '200',
                         'description'  : 'Position updated'}
                return generateResponse(format, data, "ok.xml")
            else:
                enerateResponse(format, data, "ok.xml")
        else:
          return error (format, 'Missing parameters')
    else:
        return error (format, 'Need a POST petition')
    
def user_data (request, user):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    p=get_user_data(user)
    if p==None:
        return error (format, 'User doesn\'t exist')
    data= {'code'      : '200',
           'id'        : p.id,
           'name'      : p.get_full_name(),
           'username'  : p.username,
           'status'    : p.status, 
           'email'     : p.email,
           'latitude'  : p.latitude,
           'longitude' : p.longitude,
           'radius'    : p.radius,}
    return generateResponse(format, data, "user/data.xml")

def user_my_data (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    return user_data(request, request.user.id)
        
def user_delete (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    if request.method == "POST":
       if ("userid" in request.POST):
           user=request.POST['userid']
           correct, message=delete_user(user)
           if correct:
               data = {'code'         : '200',
                       'description'  : 'User deleted correctly',}
               return generateResponse(format, data, "ok.xml")
           else:
               return error (format, message)               
       else:
          return error (format, 'Missing parameters')
    else:
        return error (format, 'Need a POST petition')

def user_all(request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    people=get_all_users()
    users=[]
    for person in people:
        users+=[{"username" : person.username,
                 "id"       : person.id,
                 "name"     : person.get_full_name(),
                 "latitude" : person.latitude,
                 "longitude": person.longitude,
                 "status"   : person.status,}]
    data = {"code"  : '200',
            "users" : users }
    return generateResponse(format, data, "user/list.xml")

def user_friends(request, user):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    people=get_all_friends(user)
    users=[]
    for person in people:
        users+=[{"username" : person.username,
                 "id"       : person.id,
                 "name"     : person.get_full_name(),
                 "latitude" : person.latitude,
                 "longitude": person.longitude,
                 "status"   : person.status,
                 "distance" : person.distance,}]
    data = {"code"  : '200',
            "users" : users }
    return generateResponse(format, data, "user/dist.xml")

def user_near_friends(request, user):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    try:
        radius=request.REQUEST["r"]
    except:
        return error(format, 'Need a (get or post) "r" parameter with the radius')
    people=get_nearby_friends(user, radius)
    users=[]
    for person in people:
        users+=[{"username" : person.username,
                 "id"       : person.id,
                 "name"     : person.get_full_name(),
                 "latitude" : person.latitude,
                 "longitude": person.longitude,
                 "status"   : person.status,
                 "distance" : person.distance}]
    data = {"code"  : '200',
            "users" : users }
    return generateResponse(format, data, "user/dist.xml")

def user_near(request, user):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    try:
        radius=request.REQUEST["r"]
    except:
        return error(format, 'Need a (get or post) "r" parameter with the radius')
    people=get_nearby_people(user, radius)
    users=[]
    for person in people:
        users+=[{"username" : person.username,
                 "id"       : person.id,
                 "name"     : person.get_full_name(),
                 "latitude" : person.latitude,
                 "longitude": person.longitude,
                 "status"   : person.status,
                 "distance" : person.distance}]
    data = {"code"  : '200',
            "users" : users }
    return generateResponse(format, data, "user/dist.xml")

def user_relation (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    if request.method=='POST':
        if ("userid1" in request.POST) and ("userid2" in request.POST):
            user1=request.POST['userid1']
            user2=request.POST['userid2']
            correct, message=api.create_friendship(user1, user2)
            if correct:
                data = {'code'         : '200',
                       'description'   : 'Relation created correctly',}
                return generateResponse(format, data, "ok.xml")
            else:
                return error (format, message)
        else:
            return error (format, 'Missing parameters')
    else:
        return error (format, 'Need a POST petition')

def user_relation_delete (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    if request.method=='POST':
        if ("userid1" in request.POST) and ("userid2" in request.POST):
            user1=request.POST['userid1']
            user2=request.POST['userid2']
            correct, message=delete_friendship(user1, user2)
            if correct:
                data = {'code'        : '200',
                       'description'  : 'Relation deleted correctly',}
                return generateResponse(format, data, "ok.xml")
            else:
                return error (format,  message)
        else:
            return error (format, 'Missing parameters')
    else:
        return error (format, 'Need a POST petition')
    
def node_near(request, node):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    try:
        radius=request.REQUEST["r"]
    except:
        return error(format, 'Need a (get or post) "r" parameter with the radius')
    set=get_nearby_node(node, radius)
    nodes=[]
    for node in set:
        nodes+=[{"id"        : node.id,
                 "latitude"  : node.latitude,
                 "longitude" : node.longitude,
                 "since"     : node.since,
                 "type"      : node.type,
                 "distance"  : node.distance}]
    data = {"code"  : '200',
            "nodes" : nodes }
    return generateResponse(format, data, "nodes/dist.xml")

def node_list(request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    nodes_obj=get_all_nodes()
    nodes=[]
    for node in nodes_obj:
        nodes+=[{"latitude"  : node.latitude,
                 "longitude" : node.longitude,
                 "id"        : node.id,
                 "since"     : node.since,
                 "type"      : node.type}]
    data = {"code"  : '200',
            "nodes" : nodes }
    return generateResponse(format, data, "nodes/list.xml")

def node_data(request, node_id):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error (format, "The user is not authenticated")
    try:
        node=get_node_data(node_id)
        if node == None:
            return error(format, 'Node doesn\'t exist')
        else:
            data =  {"code"     : '200',
                     "id"       : node.id,
                     "latitude" : node.latitude,
                     "longitude": node.longitude,
                     "since"    : node.since,
                     "type"     : node.type
                    }
            return generateResponse(format, data, "nodes/data.xml")
    except:
        return error(format, 'Some errors occurred')

def node_set_position (request, node_id):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    if request.method == "POST":
        if ("latitude" in request.POST) and ("longitude" in request.POST):
            latitude=request.POST["latitude"]
            longitude=request.POST["longitude"]
            if ("radius" in request.POST):
                radius=request.POST["radius"]
                correct, message = set_coordinates(node_id, latitude, longitude, radius)
            else:
                correct, message = set_coordinates(node_id, latitude, longitude)
            if correct:
                data = { 'code'         : '200',
                         'description'  : 'Position updated'}
                return generateResponse(format, data, "ok.xml")
            else:
                enerateResponse(format, data, "ok.xml")
        else:
          return error (format, 'Missing parameters')
    else:
        return error (format, 'Need a POST petition')

def group_create (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    if request.method == "POST":
       if ("groupname" in request.POST):
           group=Group(request.POST["groupname"])
           correct, message=create_or_modify_group(group)
           if correct:
               data = {'code'         : '200',
                       'description'  : 'Group created correctly',}
               return generateResponse(format, data, "ok.xml")
           else:
               return error(format, message)               
       else:
          return error(format, 'Missing parameters')
    else:
        return error(format, 'Need a POST petition')

def group_list (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    try:
        groups_obj=get_all_groups()
        groups=[]
        for group in groups_obj:
            groups+=[{"id"          : group.id,
                      "name"        : group.name,
                      "latitude"    : group.latitude,
                      "longitude"   : group.longitude,
                      "since"       : group.since,
                     }]
        data = {'code'      : '200',
                'groups'   : groups}
        return generateResponse(format, data, "group/list.xml")
    except:
        return error(format, 'Some errors occurred')

def group_data (request, group):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    try:
        group=get_group_data(group)
        if group == None:
            return error(format, 'Group doesn\'t exist')
        else:
            data = {'code'      : '200',
                    'id'        : group.id,
                    'name'      : group.name,
                    'latitude'  : group.latitude,
                    'longitude' : group.longitude,
                    'since'     : group.since,
                   }
            return generateResponse(format, data, "group/data.xml")
    except:
        return error(format, 'Group doesn\'t exist')

def group_delete (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    if request.method == "POST":
       if ("groupid" in request.POST):
           group=request.POST['groupid']
           correct, message = delete_group(group)
           if correct:
               data = {'code'         : '200',
                       'description'  : 'Group deleted correctly',}
               return generateResponse(format, data, "ok.xml")
           else:
               return error(format, message)               
       else:
          return error(format, 'Missing parameters')
    else:
        return error(format, 'Need a POST petition')

def group_join (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    if request.method == "POST":
       if ("groupid" in request.POST) and ("userid" in request.POST):
           try:
               group=request.POST["groupid"]
               user=request.POST["userid"]
               correct, message = api.group_join(group, user)
               if correct:
                   data={'code'         : '200',
                         'description'  : 'Joined correctly',}
                   return generateResponse(format, data, "ok.xml")
               else:
                   return error (format, message)
           except:
               return error(format, 'Unknown error occurred')               
       else:
          return error(format, 'Missing parameters')
    else:
        return error(format, 'Need a POST petition')

def group_join_delete (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    if request.method == "POST":
       if ("groupid" in request.POST) and ("userid" in request.POST):
           try:
               group=request.POST["groupid"]
               user=request.POST["userid"]
               correct, message = group_unjoin (group, user)
               if correct:
                   data = {'code'         : '200',
                           'description'  : 'Unjoined correctly',}
                   return generateResponse(format, data, "ok.xml")
               else:
                   return error(format, message)
           except:
               return error(format, 'Unknown error occurred')            
       else:
          return error(format, 'Missing parameters')
    else:
        return error(format, 'Need a POST petition')

def create_note (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error(format, "The user is not authenticated")
    if request.method == "POST":
        if request.user.is_authenticated():
            if ("text" in request.POST) and ("latitude" in request.POST) and \
             ("longitude" in request.POST) and ("title" in request.POST):
                title = request.POST['title']
                text = request.POST['text']
                latitude = request.POST['latitude']
                longitude = request.POST['longitude']
                correct, message = api.create_note(text=text, title=title,
                                                   latitude=latitude, longitude=longitude,
                                                   owner=request.user.pk)  
                if correct:
                     data = {'code'         : '200',
                             'description'  : 'Note created correctly',}
                     return generateResponse(format, data, "ok.xml")             
                else:
                    return error(format, message)
            else:
                return error(format, "Some POST parameters missing")
        else:
            return error(format, "The user must be logged in")
    else:
        return error(format, 'Need a POST petition')

def note_list (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error (format, "The user is not authenticated")
    try:
        notes_obj=get_all_notes()
        notes=[]
        for note in notes_obj:
            writers=[]
            for writer in note.writers:
                writers+=[{'id'        : writer.id,
                           'name'      : writer.get_full_name(),
                           'username'  : writer.username,
                           'status'    : writer.status, 
                           'email'     : writer.email,
                           'latitude'  : writer.latitude,
                           'longitude' : writer.longitude,
                           'radius'    : writer.radius,}]
            notes+=[{"id": note.id,
                     "title": note.title,
                     "text":note.text,
                     "latitude":note.latitude,
                     "longitude":note.longitude,
                     "since":note.since,
                     "writers": writers}]
        data = {'code'    : '200',
                'notes'   : notes}
        return generateResponse(format, data, "note/list.xml")
    except:
        return error(format, 'Some errors occurred')
    
def note_data(request, note_id):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error (format, "The user is not authenticated")
    try:
        note=get_note_data(note_id)
        if note == None:
            return error(format, 'Note doesn\'t exist')
        else:
            writers=[]
            for writer in note.writers:
                writers+=[{'id'        : writer.id,
                           'name'      : writer.get_full_name(),
                           'username'  : writer.username,
                           'status'    : writer.status, 
                           'email'     : writer.email,
                           'latitude'  : writer.latitude,
                           'longitude' : writer.longitude,
                           'radius'    : writer.radius,}]
            data =  {"code"     : '200',
                     "id"       : note.id,
                     "title"    : note.title,
                     "text"     : note.text,
                     "latitude" : note.latitude,
                     "longitude": note.longitude,
                     "since"    : note.since,
                     "writers"  : writers}
            return generateResponse(format, data, "note/data.xml")
    except:
        return error(format, 'Some errors occurred')
    
def photo_upload (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error (format, "The user is not authenticated")
    try:
        if request.method == "POST":
            if "latitude" in request.POST:
                latitude = request.POST['latitude']
            else:
                return error (format, "Need a POST param called 'latitude'")
            if "longitude" in request.POST:
                longitude = request.POST['longitude']
            else:
                return error (format, "Need a POST param called 'longitude'")
            if "description" in request.POST:
                description = request.POST['description']
            else:
                return error (format, "Need a POST param called 'description'")
            
            form = PhotoForm(request.POST, request.FILES)
            if form.is_valid():
                try:
                    clean_data = form.cleaned_data
                    photo_data = clean_data['photo']
                    photo_name = clean_data['name']
                    correct, message = upload_photo (request.user.pk, photo_name,
                                                     photo_data, latitude, longitude,
                                                     description)
                    if correct:
                         data = {'code'         : '200',
                                 'description'  : 'Photo uploaded correctly',}
                         return generateResponse(format, data, "ok.xml")             
                    else:
                        return error(format, message)
                except:
                    return error (format, sys.exc_value)
            else:
                return error (format, "Sent data not valid")
        else:
            return error (format, "Need a POST petition")
    except:
        return error (format, sys.exc_value)

def photo_list (request):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error (format, "The user is not authenticated")
    try:
        photos_obj=get_all_photos()
        photos=[]
        for photo in photos_obj:
            uploader={'id'        : photo.uploader.id,
                      'name'      : photo.uploader.get_full_name(),
                      'username'  : photo.uploader.username,
                      'status'    : photo.uploader.status, 
                      'email'     : photo.uploader.email,
                      'latitude'  : photo.uploader.latitude,
                      'longitude' : photo.uploader.longitude,
                      'radius'    : photo.uploader.radius,}
            photos+=[{"id": photo.id,
                     "name": photo.name,
                     "description":photo.description,
                     "url":photo.url,
                     "latitude":photo.latitude,
                     "longitude":photo.longitude,
                     "since":photo.since,
                     "uploader": uploader}]
        data = {'code'    : '200',
                'photos'   : photos}
        return generateResponse(format, data, "photo/list.xml")
    except:
        return error(format, 'Some errors occurred')    

def photo_data(request, photo_id):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error (format, "The user is not authenticated")
    try:
        photo=get_photo_data(photo_id)
        if photo == None:
            return error(format, 'Photo doesn\'t exist')
        else:
            photo=get_photo_data(photo_id)
            uploader={'id'        : photo.uploader.id,
                      'name'      : photo.uploader.get_full_name(),
                      'username'  : photo.uploader.username,
                      'status'    : photo.uploader.status, 
                      'email'     : photo.uploader.email,
                      'latitude'  : photo.uploader.latitude,
                      'longitude' : photo.uploader.longitude,
                      'radius'    : photo.uploader.radius,}
            data =  {"code"         : '200',
                     "id"           : photo.id,
                     "name"         : photo.name,
                     "description"  : photo.description,
                     "url"          : photo.url,
                     "latitude"     : photo.latitude,
                     "longitude"    : photo.longitude,
                     "since"        : photo.since,
                     "uploader"     : uploader}
            return generateResponse(format, data, "photo/data.xml")
    except:
        return error(format, "Some error occurred")
    
def photo_image(request, photo_id):
    format = getResponseFormat (request)
    if not request.user.is_authenticated():
        return error (format, "The user is not authenticated")
    try:
        image_path=get_photo_image_path(photo_id)
        if image_path == None:
            return error(format, 'Photo doesn\'t exist')
        image_data = open(image_path, "rb").read()
        return HttpResponse(image_data, mimetype="image/jpeg")
    except:
        return error(format, "Some error occurred")
