#
#  Copyright (C) 2009 GSyC/LibreSoft
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#    Author : Jose Antonio Santos Cadenas <jcaden __at__ gsyc __dot__ es>
#

from django.contrib.gis.geos import Point
from django.contrib.gis.geos import MultiPoint
import math

class Geo(object):
    @staticmethod
    def dist(elem, elem2=None, centroid=None):
        """
        Returns the distance between elem and elem2 if provided or the cetroid. If
        none of them, the centroid or the elem2 are provided an error is returned
        
        @param elem: An element
        @param elem2: If the distance is calculated between two elements, this 
                    is the second elements
        @param centroid: If the distance is calculated between an element an a 
                    cluster centroid, this is the value of the centroid.
                    Typically the centroid is calculated with the function
                    whith the same name, ex: label_dist will receive the centroid
                    calculated by label_centroid.
                    If the param elem is None, this param must be a tuple with to
                    centroid values, this is the way for comparing two centroids
        """
        if elem == None:
            if centroid != None:
                if len(centroid) == 2:
                    return distance(centroid[0], centroid[1])
                    #return centroid[0].distance(centroid[1])
                else:
                    raise ValueError("centroid value expected to be a tuple with two centroid values")
            else:
                raise ValueError("Expected centroid values")
        elif elem2 != None:
            return distance(elem.position, elem2.position)
            #return elem.position.distance(elem2.position)
        elif centroid != None:
            return distance(centroid, elem.position)
            #return centroid.distance(elem.position)
        else:
            #Raise exception
            raise ValueError("The elem2 or the centroid must be provided")
    
    @staticmethod
    def centroid(cluster):
        points=[e.position for e in cluster.elements]
        centroid= MultiPoint(points).centroid
        centroid.srid=4326
        return centroid
    
    @staticmethod
    def cohesion(cluster, elem, min_dist=10.0):
        n_elems = len(cluster.elements)
        max_dist = min_dist * n_elems
        too_far = 0
        if Geo.dist(elem, centroid=cluster.centroid()) > max_dist:
            return False
        else:
            return True 
        
        
    @staticmethod
    def group_data(cluster, algorithm):
        return {"name":"geo-%s" % (cluster.centroid()), "position": cluster.centroid() }
        

def distance(point1, point2):

    lat1 = point1.get_y()
    long1 = point1.get_x()
    lat2 = point2.get_y()
    long2 = point2.get_x()
    
    mean_earth_radius = 6371.009 #km
    # Convert latitude and longitude to 
    # spherical coordinates in radians.
    degrees_to_radians = math.pi/180.0
        
    # phi = 90 - latitude
    phi1 = (90.0 - lat1)*degrees_to_radians
    phi2 = (90.0 - lat2)*degrees_to_radians
        
    # theta = longitude
    theta1 = long1*degrees_to_radians
    theta2 = long2*degrees_to_radians
        
    # Compute spherical distance from spherical coordinates.
        
    # For two locations in spherical coordinates 
    # (1, theta, phi) and (1, theta, phi)
    # cosine( arc length ) = 
    #    sin phi sin phi' cos(theta-theta') + cos phi cos phi'
    # distance = rho * arc length
    
    cos = (math.sin(phi1)*math.sin(phi2)*math.cos(theta1 - theta2) + 
           math.cos(phi1)*math.cos(phi2))
    try:
        arc = math.acos( cos )
    except:
        arc = 0.0

    # Remember to multiply arc by the radius of the earth 
    # in your favorite set of units to get length.
    return arc * mean_earth_radius