# -*- coding: iso-8859-15 -*- 

from django.db import models

from datetime import datetime

from django.contrib.gis.db import models

from social.core.models import *

from django.core.files.storage import FileSystemStorage

from social.core.models import Social_node

#
#  Copyright (C) 2009 GSyC/LibreSoft
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#    Author : Jorge Fernandez Gonzalez
#

class Manager(models.Model):
    user = models.ForeignKey(Person,null=False)
    events = models.ManyToManyField("Event",null=True)

class TeamMember(models.Model):
    user = models.ForeignKey(Person,null=False)
    team = models.ForeignKey("Team",null=False)
    event = models.ForeignKey("Event",null=False)

class Team(models.Model):
    group = models.ForeignKey(Group,null=False) # groupname unique (social.core.models)
    event = models.ManyToManyField("Event",null=False)
    responses = models.ManyToManyField("Response",null=True)

class Finished(models.Model):
    team = models.ForeignKey("Team",null=False)
    event = models.ForeignKey("Event",null=False)
    is_finished = models.BooleanField(null=False) # Indica si el equipo ha finalizado esta gymkhana

class AcquiredClue(models.Model):
    team = models.ForeignKey("Team",null=False)
    clue = models.ForeignKey("Clue",null=False)

class FirstProof(models.Model):
    event = models.ForeignKey("Event",null=False)
    team = models.ForeignKey("Team",null=False)
    first_proof = models.ForeignKey("Proof",null=True) # Para este equipo en este evento, cual es el primer reto que se le presentara.

class Response(Social_node): # Almacena las respuestas que cada equipo dio a cada prueba
    proof = models.ForeignKey("Proof",null=False)
    response_text = models.CharField(max_length=255) #null=False Si la respuesta es una cadena de texto
    is_correct = models.BooleanField(default=False) # Si la respuesta es in/correcta

    date = models.DateTimeField(default=datetime.now)

    photo = models.ImageField(upload_to="img", storage=FileSystemStorage()) # Si la respuesta al reto es una foto

    distance_difference = models.FloatField(default=-1) # Si es un reto de geolocalizacion, distancia que lo separo del punto marcado como objetivo

class Clue(models.Model):
    proof = models.ForeignKey("Proof",null=False)
    number = models.IntegerField(null=False)
    help = models.CharField(max_length=255,null=False)
    cost = models.IntegerField(default=0,null=False)

class Proof(models.Model):
    challenge = models.TextField(null=False)
    number = models.IntegerField(null=False)
    event = models.ForeignKey("Event",null=False)
    next_proof_id = models.IntegerField(null=False) # Siguiente reto en la lista enlazada de pruebas de la gymkhana
    is_stop = models.BooleanField(default=False,null=False) # Indica si se necesita una respuesta correcta para poder continuar
    can_jump = models.BooleanField(default=False) # Indica si el reto se puede saltar, yendo al siguiente sin responder el actual
    max_score = models.IntegerField(default=100,null=False)
    proof_type = models.IntegerField(default=1,null=False) # 1 -> Textual; 2 -> Photo; 3 -> Geolocation; ...
    augmented_reality = models.BooleanField(default=False,null=True) # Para las pruebas textuales, si hace uso del modulo de realidad aumentada
    target_place = models.PointField(srid=4326, null=True) # Si tipo = 3; punto geografico al que hay que llegar
    distance_to_target_place = models.FloatField(default=0.0,null=True)
    mark_place = models.BooleanField(default=False,null=True) # En un reto de geolocalizacion, indica si queremos indicar al equipo el lugar objetivo en el mapa

class Solution(models.Model):
    proof = models.ForeignKey("Proof",null=False)
    possible_solution = models.CharField(max_length=255,null=False)

class JumpedProof(models.Model):
    proof = models.ForeignKey("Proof",null=False)
    team = models.ForeignKey("Team",null=False)

class Scoreboard(models.Model):
    event = models.ForeignKey("Event",null=False)
    team = models.ForeignKey("Team",null=False)
    score = models.IntegerField(default=0,null=False)
    num_correct_responses = models.IntegerField(default=0,null=False)
    num_incorrect_responses = models.IntegerField(default=0,null=False)

class Event(models.Model):
    title = models.CharField(max_length=255,null=False)
    celebration = models.DateField(default=datetime.now(),null=False)
    place = models.CharField(max_length=255,null=False)
    welcome_text = models.TextField(null=False)
    goodbye_text = models.TextField(null=False)

class Message(Social_node):
    event = models.ForeignKey("Event",null=False)
    from_manager = models.ForeignKey("Manager",blank=True,null=True,related_name="sender_manager")
    from_team = models.ForeignKey("Team",blank=True,null=True,related_name="sender_team")
    to_manager = models.ForeignKey("Manager",blank=True,null=True,related_name="receiver_manager")
    to_team = models.ManyToManyField("Team",blank=True,null=True,related_name="receiver_team")
    text = models.TextField(null=False)

    date = models.DateTimeField(default=datetime.now)
