/*
 * Copyright (c) 2009 GSyC/LibreSoft 
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *    	 
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *  		 Raúl Román <rroman@gsyc.es>
 *  
 */

package com.LibreGeoSocialApp;


import java.util.List;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SubMenu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.User;
import com.LibreGeoSocialApp.Overlay.PositionOverlay;
import com.LibreGeoSocialApp.Resources.ResourceManager;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

public class Home extends MapActivity implements Runnable, OnClickListener  {
	
    private ProgressDialog pd;
    
	LocationManager mLocationManager;
	Location mLocation;
	MyLocationListener mLocationListener;
	
	static Location currentLocation = null;
	
	TextView outlat;
	TextView outlong;
	Button btUpdateLocation;
	MapView mapView;
	ImageView imageUser;
	TextView txtUserName;
	TextView txtState;
	
	private static final int MENU_RESOURCES = Menu.FIRST + 1;
	private static final int MENU_STATE = Menu.FIRST + 2;
	private static final int MENU_LOCATION = Menu.FIRST + 3;
	private static final int MENU_LOCATION_GPS = Menu.FIRST + 4;
	private static final int MENU_LOCATION_OTHERS = Menu.FIRST + 5;
	
	private static final int DIALOG_UPDATE_STATE = 1;
	private String LocationType;
	
	private GeoPoint geoPoint;
	
	private User user;
	
	public static Location getLocation()
	{
		return currentLocation;
	}
		
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
 
        
        setContentView(R.layout.home);
        
        LocationType = LocationManager.GPS_PROVIDER;
        
        btUpdateLocation = (Button)this.findViewById(R.id.btLocation); 
        btUpdateLocation.setOnClickListener(this);
        
        imageUser = (ImageView)this.findViewById(R.id.imgUser);
        imageUser.setImageResource(R.drawable.user_48);

        txtUserName = (TextView) this.findViewById(R.id.lbName);
        txtState = (TextView) this.findViewById(R.id.lbState);
        
        //writeSignalGPS();
                
        user = LibreGeoSocial.getInstance().getInfoUser(true, null);
        
        txtUserName.setText (user.getName());
        txtState.setText(user.getState());             
        
    }
    
	protected void onResume(){
		super.onResume();
		
		user = LibreGeoSocial.getInstance().getInfoUser(true, null);
		
		if(user.getLatitude()==0 || user.getLongitude()==0){
			Toast.makeText(getBaseContext(), 
                    "No location found", 
                    Toast.LENGTH_LONG).show();
			return;
		}
			
		Location loc = new Location("MANUAL_PROVIDER");
		loc.setLatitude(user.getLatitude());
		loc.setLongitude(user.getLongitude());
		
		setCurrentLocation(loc);
		showMap();
	}
	
    private void setCurrentLocation(Location loc) {
    	currentLocation = loc;	
    	LibreGeoSocial.getInstance().setUserPosition(loc.getLatitude(), loc.getLongitude());
    }
    
	public void onClick(View arg0) {
		// TODO Auto-generated method stub
		
		writeSignalGPS(); 
		
	}
	

    
    private void writeSignalGPS() {
    	
    	DialogInterface.OnCancelListener dialogCancel = new DialogInterface.OnCancelListener() {

            public void onCancel(DialogInterface dialog) {

            }
          
        };
    	
		pd = ProgressDialog.show(this, this.getResources().getString(R.string.search), 
				this.getResources().getString(R.string.search_signal_gps), true, true, dialogCancel);
		
		Thread thread = new Thread(this);
		thread.start();

    }
    
    private void showMap() 
    {
    	
        mapView = (MapView) findViewById(R.id.myMapView);
    	 
    	//mapView = new MapView(this, "0Tp7WsIDiyqp_gJLEiO55RXtkz7VsxOjQvL44SA");
        MapController mapControl = mapView.getController();
        
        geoPoint = new GeoPoint(  (int) (currentLocation.getLatitude() * 1000000), 
        						  (int) (currentLocation.getLongitude() * 1000000));
        mapControl.setZoom(20);
        mapControl.animateTo(geoPoint);
        
        PositionOverlay myposOverlay = new PositionOverlay();
        myposOverlay.setUser(user);
        
	    final List<Overlay> overlays = mapView.getOverlays();	 
	    overlays.clear();
	    overlays.add( myposOverlay );
	    
	    mapView.setClickable(true);
        
    }
    
    @Override
    protected Dialog onCreateDialog(int id) {
        
    	switch (id) {
	    	case DIALOG_UPDATE_STATE:
	    		
	    		LayoutInflater factory = LayoutInflater.from(this);
	    		final View textEntryView = factory.inflate(R.layout.state, null);
	    		
	    		final EditText edit = (EditText) textEntryView.findViewById (R.id.txtstate);
	    		edit.setText("");
	    		
		        return new AlertDialog.Builder(this)	        
		        .setTitle("Update state")
		        .setView(textEntryView)
		        .setPositiveButton("update", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		
		            	if ( LibreGeoSocial.getInstance().setStatus(edit.getText().toString()))
		            	{		            		
		            		txtState.setText(edit.getText().toString());
		            	}
		            	else
		            	{
		            		Toast.makeText(getBaseContext(), 
			     					   "Error while the status was setted: " + 
			     					   LibreGeoSocial.getInstance().getLastMsgServer(), 
			     					   Toast.LENGTH_LONG).show();
		            	}
		            	
		                /* User clicked OK so do some stuff */
		            }
		        })
		        .setNegativeButton("cancel", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		
		                /* User clicked cancel so do some stuff */
		            }
		        })
		        .create();
		    
    	}
    	
    	return null;
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	
        super.onCreateOptionsMenu(menu);
        menu.add(0, MENU_RESOURCES, 0, "Around Places")
        		.setIcon(R.drawable.register)
                .setAlphabeticShortcut('A');
        
        menu.add(0, MENU_STATE, 0 , "Update State")
        		.setIcon(R.drawable.refresh_48)
                .setAlphabeticShortcut('B');
        
        SubMenu sub = menu.addSubMenu (0, MENU_LOCATION, 0 , "Location")
        			  .setIcon(R.drawable.mundo);
        
        sub.add(0,MENU_LOCATION_GPS,0, "GPS");
        sub.add(0,MENU_LOCATION_OTHERS,0,"LOCALIZAME");
        
        return true;
    }

    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    	
    	case MENU_RESOURCES:
        	
    		ResourceManager.getInstance().reset();
    		Intent i = new Intent(this,ResourceTab.class);
        	startActivity(i); 
        	
    		break;

    	case MENU_STATE:
    		
    		showDialog(DIALOG_UPDATE_STATE);
    		
    		break;
    		
    	case MENU_LOCATION_GPS:
    
    		LocationType = LocationManager.GPS_PROVIDER;
    		
    		break;
    		
    	case MENU_LOCATION_OTHERS:
    		
    		Intent i1 = new Intent(this,LocationWays.class);
        	startActivity(i1);
    		
    		break;
    	
    		
        }
        return super.onOptionsItemSelected(item);
    }
    
	@Override
	protected boolean isRouteDisplayed() {
		// TODO Auto-generated method stub
		return false;
	} 
	
	public void run() {
    	
		mLocationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
		
		if (mLocationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
			
			Looper.prepare();
			
			mLocationListener = new MyLocationListener();
			
			mLocationManager.requestLocationUpdates(
					LocationType, 0, 0, mLocationListener);
			Looper.loop(); 
			Looper.myLooper().quit(); 
			
		} else {
			
			
            Toast.makeText(getBaseContext(), 
                    getResources().getString(R.string.gps_signal_not_found), 
                    Toast.LENGTH_LONG).show();
            
		}
	}
	
	
    
	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			pd.dismiss();
			mLocationManager.removeUpdates(mLocationListener);
	    	if (currentLocation!=null) {
	    		//outlat.setText("Latitude: " + currentLocation.getLatitude());
	    		//outlong.setText("Longitude: " + currentLocation.getLongitude());
	    	}
		}
	};
	
	
    private class MyLocationListener implements LocationListener 
    {

        public void onLocationChanged(Location loc) {
            if (loc != null) {
                Toast.makeText(getBaseContext(), 
                    getResources().getString(R.string.gps_signal_found), 
                    Toast.LENGTH_LONG).show();
                setCurrentLocation(loc);
                showMap();
                handler.sendEmptyMessage(0);
            }
        }


        public void onProviderDisabled(String provider) {
            // TODO Auto-generated method stub
        }


        public void onProviderEnabled(String provider) {
            // TODO Auto-generated method stub
        }


        public void onStatusChanged(String provider, int status, 
            Bundle extras) {
            // TODO Auto-generated method stub
        }
    }



    
}

