/*
 * Copyright (c) 2009 GSyC/LibreSoft 
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *    	 
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License. 
 *
 *  Author : Raúl Román <rroman@gsyc.es>
 *  		  
 */

package com.LibreGeoSocialApp;

import java.util.List;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.location.Location;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.User;
import com.LibreGeoSocialApp.Overlay.PositionOverlay;
import com.LibreGeoSocialApp.Utils.LocalizameAPI;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.MapView.LayoutParams;

public class LocationWays extends MapActivity {
	private MapView map;
	private MapController controller;
	private User user;
	private String usrpwd, usrtlf, message;
	public Location currentLocation = null;
	
	ProgressDialog pd = null;
	
	private static final int MENU_SATELLITE = Menu.FIRST + 1;
	private static final int MENU_LOCATION_MOVISTAR = Menu.FIRST + 2;
	
	private static final int DIALOG_WELCOME = 1;
	
	private boolean capture = false;
	private GeoPoint map_center;
	
	public void init(){
		LinearLayout zoomLayout = (LinearLayout) findViewById(R.id.location_layout_zoom);
		
		map = (MapView) findViewById(R.id.locationMap);
		
		controller = map.getController();

		//Adding a zoom control
		View zoomView = map.getZoomControls();
		zoomLayout.addView(zoomView, new LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT));
		map.setClickable(true);
		map.setOnTouchListener(touchListener);
		
		map.displayZoomControls(true);
		
		if(currentLocation!=null){
			setPositionInMap(17);
		}
		map.setSatellite(false);
		map.setStreetView(false);
		
		map_center = map.getMapCenter();
		
		showDialog(DIALOG_WELCOME);
	}
	
    /** Called when the activity is first created. */
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.location);
        user = LibreGeoSocial.getInstance().getInfoUser(true, null);
        currentLocation = Home.getLocation();
        init();
        
    }
    
    private OnTouchListener touchListener = new OnTouchListener(){
		@Override
		public boolean onTouch(View v, MotionEvent event) {
			// TODO Auto-generated method stub
			
			switch(event.getAction()){
			
			case MotionEvent.ACTION_DOWN:
				map_center = map.getMapCenter();
				capture = true;
				break;
				
			case MotionEvent.ACTION_MOVE:
				GeoPoint new_map_center = map.getMapCenter();
		    	
		    	double distance = Math.sqrt(Math.pow(new_map_center.getLatitudeE6()-map_center.getLatitudeE6(),2)+
		    			Math.pow(new_map_center.getLongitudeE6()-map_center.getLongitudeE6(),2));
		    	
		    	map_center = new_map_center;
		    	
		    	if(distance > 1)
		    		capture = false;
				break;
			
			case MotionEvent.ACTION_UP:
				if (!capture)
					break;
				capture = false;
				
				Location loc = getLocFromXY(event.getX(), event.getY());
				
				setCurrentLocation(loc);
				setPositionInMap(17);
				
				Toast.makeText(getBaseContext(), 
                        "Location updated", 
                        Toast.LENGTH_LONG).show();
				break;
				
			default:
					break;
			}
			
			return false;
		}
    };
    
    private Location getLocFromXY(float x, float y_inverted){
    	double screenYsize = map.getHeight();
    	double screenXsize = map.getWidth();
    	GeoPoint center = map.getMapCenter();
    	double latitudeYsize = map.getLatitudeSpan();
    	double longitudeXsize = map.getLongitudeSpan();
    	
    	float y = ((float)screenYsize) - y_inverted;
    	//Obtaining how many degrees of Latitude and Longitude has a single pixel
    	double pix_Xlongitude = longitudeXsize/screenXsize;
    	double pix_Ylatitude = latitudeYsize/screenYsize;
    	
    	//Obtaining the distance (in microdegrees) to the center
    	double pixcenter_lat = screenYsize/2;
    	double pixcenter_long = screenXsize/2;
    	
    	double degrees_lat = ((((double)y) - pixcenter_lat)*pix_Ylatitude);
    	double degrees_long = ((((double)x) - pixcenter_long)*pix_Xlongitude);
    	
    	//Obtaining the coordinates the user has tapped
    	double lat = (degrees_lat + (double) center.getLatitudeE6())/1E6;
    	double lng = (degrees_long + (double) center.getLongitudeE6())/1E6;
    	
    	Location loc = new Location("MANUAL_PROVIDER");
    	loc.setLatitude(lat);
    	loc.setLongitude(lng);
    	
    	return loc;
    }
    
	private void setPositionInMap(int zoom){
		
		GeoPoint point = new GeoPoint(  (int) (currentLocation.getLatitude() * 1E6), 
				  (int) (currentLocation.getLongitude() * 1E6));
		
		controller.animateTo(point);
		controller.setZoom(zoom);
		//OverlayItem marker = new OverlayItem(point, "You", "Now");
		PositionOverlay myposOverlay = new PositionOverlay();
        myposOverlay.setUser(user);
        //map.getOverlays().clear();
	    final List<Overlay> overlays = map.getOverlays();	   
	    overlays.add( myposOverlay );
	}
	
	private void setCurrentLocation(Location loc){
		currentLocation = loc;
		user.setLatitude(loc.getLatitude());
		user.setLongitude(loc.getLongitude());
		LibreGeoSocial.getInstance().setUserPosition(loc.getLatitude(), loc.getLongitude());
	}

	public int searchMovistar(String user, String pwd){
		double location[] = new double[3];
		LocalizameAPI movistar = new LocalizameAPI();
				
		if(!movistar.login(user, pwd)){
			message = "Service unavailable";
			return 0;
		}
		
		handlerRefresh.sendEmptyMessage(0);
		if(!movistar.authorize(user)){
			message = "Service unavailable";
			return 0;
		}
		movistar.logout();

		movistar.login(user,pwd);
		handlerRefresh.sendEmptyMessage(1);
		location = movistar.locate(user);
		movistar.logout();

		movistar.login(user, pwd);
		handlerRefresh.sendEmptyMessage(2);
		movistar.forbid(user);
		movistar.logout();
			
		if(location==null){
			message = "User unreachable";
			return 0;
		}
		
		currentLocation = new Location("MOVISTAR_PROVIDER");
		currentLocation.setLatitude(location[0]);
		currentLocation.setLongitude(location[1]);
		
		message = "Location updated";
		
		return ((int)location[2]);
	}
	
    @Override
    protected Dialog onCreateDialog(int id) {
        
    	switch (id) {
	    	case DIALOG_WELCOME:
	    		
	    		LayoutInflater factory1 = LayoutInflater.from(this);
	    		final View textEntryView1 = factory1.inflate(R.layout.localizame_welcome, null);
	    		
	    		ImageView logo = (ImageView)textEntryView1.findViewById(R.id.imgPop_up2Logo);
	    		logo.setImageResource(R.drawable.combo);
	    		
		        return new AlertDialog.Builder(this)	        
		        .setTitle("Welcome!")
		        .setView(textEntryView1)
		        .setPositiveButton("Ok", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		            	
		            }
		        })
		        .create();
    	}
    	
    	return null;
    }
	
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	
        super.onCreateOptionsMenu(menu);
        
        menu.add(0, MENU_SATELLITE, 0, "Satellite")
				.setIcon(R.drawable.register)
				.setAlphabeticShortcut('S');
        
        
        menu.add (0, MENU_LOCATION_MOVISTAR, 0 , "Localizame")
        			  .setIcon(R.drawable.movistar);       
        return true;
    }

    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    		
    	case MENU_SATELLITE:
    		if (map.isSatellite())
	    		map.setSatellite(false);
	    	else
	    		map.setSatellite(true);
	        break;
    		
    	case MENU_LOCATION_MOVISTAR:
    		
    		//showDialog(DIALOG_MOVISTAR_LOCATION);
    		usrtlf = OMFConfig.number;
    		usrpwd = OMFConfig.locpassword;
    		
    		if(usrtlf==null || usrpwd==null){
    			Toast.makeText(getBaseContext(), 
                        "Please, insert data in main Movistar tab.", 
                        Toast.LENGTH_LONG).show();
    			break;
    		}
    		if (usrtlf.equals("") || usrpwd.equals("")){
    			Toast.makeText(getBaseContext(), 
    					"Please, insert data in main Movistar tab", 
    					Toast.LENGTH_LONG).show();
    			break;
    		}
    		
    		runMovistar();
    		break;
    	
        }
        return super.onOptionsItemSelected(item);
    }
	
    private int calculateZoom(double radius){
    	
    	if(radius<200)
    		return 18;
    	if(radius>=200 && radius<400)
    		return 17;
    	if(radius>=400 && radius<700)
    		return 16;
    	if(radius>=700 && radius<1000)
    		return 15;
    	if(radius>=1000 && radius<2000)
    		return 14;
    	if(radius>=2000 && radius<5000)
    		return 13;
    	if(radius>5000 && radius<=15000)
    		return 11;
    	if(radius>15000 && radius<=25000)
    		return 10;
    	if(radius>25000 && radius<=50000)
    		return 9;
    	if(radius>50000 && radius<=75000)
    		return 8;
    	if(radius>75000 && radius<=150000)
    		return 7;
    	if(radius>=150000 && radius<500000)
    		return 6;
    	if(radius>=500000 && radius<1E6)
    		return 5;
    	
    	return 4;
    }
    
	@Override
	protected boolean isRouteDisplayed() {
		// TODO Auto-generated method stub
		return false;
	}
	
	private Handler handlerRefresh = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			switch(msg.what){
			case 0:
				pd.setMessage("Authorizing...");
				break;
			case 1:
				pd.setMessage("Locating...");
				break;
			case 2:
				pd.setMessage("Forbidding...");
				break;
			}
			
		}
	};
	
	private Handler handlerEnd = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			pd.dismiss();
			if(msg.what!=0){
				setCurrentLocation(currentLocation);
				setPositionInMap(calculateZoom((double)msg.what));
			}
			Toast.makeText(getBaseContext(), message,Toast.LENGTH_LONG).show();
		}
	};
	
	private void runMovistar(){
		pd = ProgressDialog.show(LocationWays.this, "Localizame", "Searching...", true);
		
		new Thread(){
			public void run(){
				int radius = searchMovistar(usrtlf, usrpwd);
				handlerEnd.sendEmptyMessage(radius);
				
			}
		}.start();
		
	}
	

	
}