/*
 * Copyright (c) 2009 GSyC/LibreSoft 
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *    	 
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *  
 */

package com.LibreGeoSocialApp.Overlay;

import android.graphics.Canvas;
import android.graphics.Point;
import android.graphics.drawable.Drawable;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.Projection;

public class MapOverlay extends Overlay {
	
	   private Drawable mMarker;
	   private int mMarkerXOffset;
	   private int mMarkerYOffset;
	   
	   private GeoPoint mGeoPoint;
	   
	   public void setGeoPoint (GeoPoint geoPoint)
	   {
		   mGeoPoint = geoPoint;			   
	   }
	   
	   public void setDrawable(Drawable draw)
	   {
		   mMarker = draw;
	   }
	   
	   @Override
	   public void draw(Canvas canvas, MapView mapView, boolean shadow) {
         if (!shadow) {
         	
	     	   //mMarker = getResources().getDrawable(mDrawable);
	   	        
	   	        // Make sure to give mMarker bounds so it will draw in the overlay
	   	       final int intrinsicWidth = mMarker.getIntrinsicWidth();
	   	       final int intrinsicHeight = mMarker.getIntrinsicHeight();
	   	       mMarker.setBounds(0, 0, intrinsicWidth, intrinsicHeight);
	   	        
	   	       mMarkerXOffset = -(intrinsicWidth / 2);
	   	       mMarkerYOffset = -intrinsicHeight;
	       
             Point point2 = new Point();
             Projection p = mapView.getProjection();
             p.toPixels(mGeoPoint, point2);

             super.draw(canvas, mapView, shadow);
             drawAt(canvas, mMarker, point2.x + mMarkerXOffset, point2.y + mMarkerYOffset, shadow);

         }
	   }
 }
