/*
 * Copyright (c) 2009 GSyC/LibreSoft 
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *    	 
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *  
 */

package com.LibreGeoSocialApp.Overlay;

import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.RectF;

import com.LibreGeoSocial.Api.User;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.Projection;

public class PositionOverlay extends Overlay
{
	private final int mRadius = 5;
	
	private User mUser;
	
	public void setUser (User user)
	{
		mUser = user;
	}
	
	@Override
	public void draw(Canvas canvas, MapView mapView, boolean shadow)
	{
	
        GeoPoint geoPoint = new GeoPoint(  (int) (mUser.getLatitude() * 1000000), 
				  						   (int) (mUser.getLongitude() * 1000000));
        
		Point point= new Point();
		Projection projection = mapView.getProjection();
	        
		projection.toPixels(geoPoint, point);
	        
		RectF oval = new RectF (point.x - mRadius, point.y - mRadius,
								point.x + mRadius, point.y + mRadius);
	    
		Paint paint = new Paint();
		paint.setARGB(250,255,0,0);
		paint.setAntiAlias(true);
		paint.setFakeBoldText(true);
		
	        
		Paint backPaint = new Paint();
		backPaint.setARGB(175,0,0,0);
		backPaint.setAntiAlias(true);
	        
		String text = mUser.getUsername();
		RectF backRect = new RectF (point.x + 2 + mRadius, point.y - 3*mRadius,
									point.x + 60, point.y + mRadius);
	        
		canvas.drawOval(oval,paint);
		
		canvas.drawRoundRect(backRect, 5, 5, backPaint);
		
		canvas.drawText (text, point.x + 2*mRadius, point.y, paint);
	}
	
}
