/*
 * Copyright (c) 2009 GSyC/LibreSoft 
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *    	 
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *  	
 *  
 */


package com.LibreGeoSocialApp;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URISyntaxException;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.ListActivity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

import com.LibreGeoSocialApp.Resources.PanoramioItem;
import com.LibreGeoSocialApp.Resources.ResourceItem;
import com.LibreGeoSocialApp.Resources.ResourceManager;


public class PhotoList extends ListActivity {

	private static final String TAG 		= "PhotoList";
	private static final String TAG_ERROR 	= "PhotoList::ERROR";

	PhotoAdapter mAdapter; 
	ResourceManager mPanoramio = new ResourceManager();
	
	public void onCreate(Bundle savedInstanceState) {
		
		super.onCreate(savedInstanceState);
		    

		// Get and parse JSON
		mPanoramio = ResourceManager.getInstance();
		
		getJSON();
		
  		// Adapter for ListView
        mAdapter = new PhotoAdapter(this);
        setListAdapter(mAdapter);

		
	}
	
    protected void onListItemClick(ListView l, View v, int position, long id) {
        
    	ResourceItem resource = ResourceManager.getInstance().getResource(position,
				  ResourceManager.PHOTO_RESOURCE);

    	Intent i = new Intent(this, ViewMap.class);
    	
    	i.putExtra("latitude",resource.getLatitude());
    	i.putExtra("longitude",resource.getLongitude());
    	i.putExtra("url",resource.getUrl());
    	
    	startActivity(i);
    
	}
	
	
	private void getJSON ()
	{
		
		try {
            
			String URL = "http://libregeosocial.libresoft.es/agregador/rest/?tipo=panoramio&latitud=" +
						  Home.getLocation().getLatitude() + "&longitud=" + 
						  Home.getLocation().getLongitude() + "&radio=0.8&texto=&format=JSON";
			
            HttpGet get = new HttpGet(URL);
  
            HttpClient client = new DefaultHttpClient();
            HttpResponse response = client.execute(get);
            HttpEntity entity = response.getEntity();

            String str = convertStreamToString(entity.getContent());
            JSONObject json = new JSONObject(str);

            parseJSON (json);
      
        } catch (Exception e) {
            Log.e(TAG, e.toString());
        }
		
	}
	
    private String convertStreamToString (InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is), 8*1024);
        StringBuilder sb = new StringBuilder();
 
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        
        return sb.toString();

    }

    private void parseJSON (JSONObject json) throws URISyntaxException
    {
    	JSONArray array;
    	
		try {
			array = json.getJSONArray("resources");
			int count = array.length();
			for (int i = 0; i < count; i++) {
				
				JSONObject obj = array.getJSONObject(i);
				String title, owner, url, urlImage, urlThumb, service, summary, date = null;
				Double latitude, longitude;
				
							
				title 		= obj.getString("title");
				summary 	= obj.getString("summary");
				date 		= obj.getString("date");
				url			= obj.getString("url");
				latitude	= obj.getDouble("latitude");
				longitude	= obj.getDouble("longitude");
				service 	= obj.getString("service");
				
				ResourceItem resource = null;

				if (service.equals(ResourceManager.PHOTO_RESOURCE))
				{
					owner 		= obj.getString("credits_owner");
					urlImage	= obj.getString("img_url");
					urlThumb  	= obj.getString("thumb_url");
					
					resource = new PanoramioItem (title, summary, date, 
												  latitude, longitude, url, 
												  service, null, owner, 
												  urlImage, urlThumb);
				}
				else
				{
					resource = new ResourceItem();
					Log.e(TAG_ERROR, "Unknow Resource\n");
				}
				
				ResourceManager.getInstance().addResource(resource,ResourceManager.PHOTO_RESOURCE);
				
			}
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			Log.e(TAG, e.toString());
		}
		
		
	
    }


    
	public static class PhotoAdapter extends BaseAdapter {

		private Context mContext;
		
		public PhotoAdapter(Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return ResourceManager.getInstance().getSize(ResourceManager.PHOTO_RESOURCE);
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            // Make up a new view
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.photolist, null);
	        } else {
	            // Use convertView if it is available
	            view = convertView;
	        }
			
	        
	        ResourceItem resource = 
	        	ResourceManager.getInstance().getResource(position, ResourceManager.PHOTO_RESOURCE);
	        
	        if (resource.getService().equals(ResourceManager.PHOTO_RESOURCE))
	        {
	        	PanoramioItem photo = (PanoramioItem) resource;

		        ImageView img = (ImageView) view.findViewById(R.id.image);
		        
				img.setImageBitmap(photo.getBitmapThumb());
				
				TextView t = (TextView) view.findViewById(R.id.title);
				t.setText(photo.getTitle());
				
				t = (TextView) view.findViewById(R.id.owner);
				t.setText(photo.getOwner());
		        
				return view;
	        }
	        
	        return null;
		}

	}



}
