/*
 * Copyright (c) 2009 GSyC/LibreSoft 
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *    	 
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *  		 Raúl Román <rroman@gsyc.es>
 *  
 */

package com.LibreGeoSocialApp;

import java.util.List;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.User;
import com.LibreGeoSocialApp.Overlay.PositionOverlay;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

public class UserCard extends MapActivity {

	
	private ImageView imageUser;
	private TextView txtUserName;
	private TextView txtState;

	private MapView mapView;
	private User user;
	
	private static final int MENU_CALL = Menu.FIRST + 1;
	private static final int MENU_SMS = Menu.FIRST + 2;
	
	private static final int DIALOG_QUESTION_SMS = 1;
	
	
    public void onCreate(Bundle icicle) {
    	
         super.onCreate(icicle);
        
         setContentView(R.layout.user_card);
         
         imageUser = (ImageView)this.findViewById(R.id.imgUser);
         imageUser.setImageResource(R.drawable.user_48);

         txtUserName = (TextView) this.findViewById(R.id.lbName);
         txtState = (TextView) this.findViewById(R.id.lbState);         
                 
         Intent i = getIntent();
         
         Integer userId  = i.getIntExtra ("userId", Integer.MIN_VALUE);
         
         user = LibreGeoSocial.getInstance().getInfoUser(true, userId);
         
         txtUserName.setText (user.getName());
         txtState.setText(user.getState());  
         
         
         showMap();
        
    }

	@Override
	protected boolean isRouteDisplayed() {
		// TODO Auto-generated method stub
		return false;
	}
	
    private void showMap()
    {
    	
        mapView = (MapView) findViewById(R.id.myMapView);
    	 
    	//mapView = new MapView(this, "0Tp7WsIDiyqp_gJLEiO55RXtkz7VsxOjQvL44SA");
        MapController mapControl = mapView.getController();
        
        
        GeoPoint geoPoint = new GeoPoint(  (int) (user.getLatitude() * 1000000), 
        						  		   (int) (user.getLongitude() * 1000000));
        mapControl.setZoom(20);
        mapControl.setCenter(geoPoint);
        
        PositionOverlay myposOverlay = new PositionOverlay();
        myposOverlay.setUser(user);
        
	    final List<Overlay> overlays = mapView.getOverlays();	   
	    overlays.add( myposOverlay );
	    
	    mapView.setClickable(true);
        
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	
        super.onCreateOptionsMenu(menu);
        menu.add(0, MENU_CALL, 0, "Call friend")
        		.setIcon(R.drawable.phone)
                .setAlphabeticShortcut('C');
        
        menu.add(0, MENU_SMS, 0, "Send SMS")
        		.setIcon(R.drawable.paper_pencil_48)
        		.setAlphabeticShortcut('S');
        
        return true;
    }

    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    	
    	case MENU_CALL:
        	
    		if(txtState.getText().toString().toUpperCase().equals("BUSY")){
    			showDialog(DIALOG_QUESTION_SMS);
    		}else{
    			Intent i = new Intent(Intent.ACTION_DIAL);
            	startActivity(i);
    		}
    		break;
    		
    	case MENU_SMS:
        	
    		Intent i = new Intent(this,SMS.class);
        	startActivity(i);
        	
    		break;
    	
        }
        return super.onOptionsItemSelected(item);
    }
    
    @Override
    protected Dialog onCreateDialog(int id) {
        
    	switch (id) {
	    	case DIALOG_QUESTION_SMS:
	    		
	    		LayoutInflater factory = LayoutInflater.from(this);
	    		final View textEntryView = factory.inflate(R.layout.sms_warning, null);
	    		
	    		
		        return new AlertDialog.Builder(this)	        
		        .setTitle("Warning!")
		        .setView(textEntryView)
		        .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		            	//launch activity
		            	Intent i = new Intent(UserCard.this,SMS.class);
		            	startActivity(i);
		                /* User clicked OK so do some stuff */
		            }
		        })
		        .setNegativeButton("No", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		            	
		                /* User clicked cancel so do some stuff */
		            }
		        })
		        .create();
		    
    	}
    	
    	return null;
    }
    
}
