/*
 * Copyright (c) 2009 GSyC/LibreSoft 
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *    	 
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *  		 Raúl Román <rroman@gsyc.es>
 *  
 */

package com.LibreGeoSocialApp;

import java.util.List;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SubMenu;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.widget.FrameLayout;

import com.LibreGeoSocialApp.Overlay.MapOverlay;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;



public class ViewMap extends MapActivity {

	   private MapView mapView;
	   
	   private GeoPoint point;
	   
	   private String mUrl;

	   private static final int MENU_MAP = Menu.FIRST + 1;
	   private static final int MENU_MAP_SATELITE = Menu.FIRST + 2;
	   private static final int MENU_MAP_NORMAL = Menu.FIRST + 3;
	   private static final int MENU_GOTOPLACE = Menu.FIRST + 4;
	   private static final int MENU_GOTOURL = Menu.FIRST + 5;
	   
	   @Override
	   public void onCreate(Bundle icicle) {
		   
		   super.onCreate(icicle);
		   
		   FrameLayout frame = new FrameLayout(this);
		   
		   mapView = new MapView(this, "0Tp7WsIDiyqp_gJLEiO55RXtkz7VsxOjQvL44SA");
		   MapController mapControl = mapView.getController();
		   
		  
	       frame.addView(mapView, 
	                new FrameLayout.LayoutParams(LayoutParams.FILL_PARENT, LayoutParams.FILL_PARENT));

	       
		   setContentView(frame);		  
		   
		   /* Obtain data */
		   Intent i = getIntent();

		   Double latitude  = i.getDoubleExtra("latitude", Integer.MIN_VALUE);
		   Double longitude = i.getDoubleExtra("longitude", Integer.MIN_VALUE);
		   mUrl 			= i.getStringExtra("url");

		   point = new GeoPoint((int) (latitude * 1000000), (int) (longitude * 1000000));
		   mapControl.setZoom(20);
		   mapControl.setCenter(point);
		   
		   /* Config overlay */
		   MapOverlay myMapOver = new MapOverlay();
		   myMapOver.setDrawable(getResources().getDrawable(R.drawable.map_pin));
		   myMapOver.setGeoPoint(point);
		   
		   
	       final List<Overlay> overlays = mapView.getOverlays();
	       overlays.add(myMapOver);
	       
	       /* Settings map */
	       mapView.setClickable(true);
	       mapView.setEnabled(true);
	       mapView.setSatellite(false);

		   addZoomControls(frame);

	   }
	   
	   @Override
	   protected boolean isRouteDisplayed() {
	      // TODO Auto-generated method stub
	      return false;
	   }

	   
	   /**
	     * Get the zoom controls and add them to the bottom of the map
	     */
	    private void addZoomControls(FrameLayout frame) {
	        View zoomControls = mapView.getZoomControls();

	        FrameLayout.LayoutParams p = 
	            new FrameLayout.LayoutParams(LayoutParams.WRAP_CONTENT,
	                LayoutParams.WRAP_CONTENT, Gravity.BOTTOM + Gravity.CENTER_HORIZONTAL);
	        frame.addView(zoomControls, p);
	    }
	    

	    
	    @Override
	    public boolean onCreateOptionsMenu(Menu menu) {
	    	
	        super.onCreateOptionsMenu(menu);
	         
	        SubMenu sub = menu.addSubMenu(0,MENU_MAP,0,"View Map");
	        
	        sub.add(0, MENU_MAP_SATELITE, 0, "Satelite View");
	        
	        sub.add(0, MENU_MAP_NORMAL, 0, "Normal View");
   
	        menu.add(0, MENU_GOTOPLACE, 0, "Go to Place")
    					.setAlphabeticShortcut('C');
	        
	        menu.add(0, MENU_GOTOURL, 0, "Go to Url")
	        		.setAlphabeticShortcut('C');
	        
	        return true;
	    }
	    
	    @Override
	    public boolean onOptionsItemSelected (MenuItem item) {
	    	
	    	switch (item.getItemId()) {
	    	
	    	case MENU_MAP_SATELITE:
	    		mapView.setSatellite(true);
	    		break;
	    		
	    	case MENU_MAP_NORMAL:
	    		mapView.setSatellite(false);
	    		break;
	    		
	    	case MENU_GOTOPLACE:
	    		
	    		/*
	    		List<Address> adresses = null;
	    		
	    		Geocoder gc = new Geocoder (this, Locale.getDefault());
	    		try {
						adresses = gc.getFromLocation(-3.867252, 40.335537, 10);
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
	    		
					Address address = adresses.get(0);
					
					for (int i=0; i< address.getMaxAddressLineIndex(); i++)
					{
						Log.e("GPS", address.getAddressLine(i) + "\n");
						Log.e("GPS", address.getLocality() + "\n");
						Log.e("GPS", address.getPostalCode() + "\n");
						Log.e("GPS", address.getCountryName() + "\n");
						Log.e("GPS", "===========================\n");
					}
	    		*/
	    		
	    		break;
	    	case MENU_GOTOURL:
	    		
	            Intent i = new Intent(Intent.ACTION_VIEW);
	            i.setData(Uri.parse(mUrl));
	            startActivity(i);

	    		break;
	        }
	        return super.onOptionsItemSelected(item);
	    }


	}