/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *
 */

package com.LibreGeoSocial.Api;

import com.LibreGeoSocialApp.Camera.Overlays.DrawResource;
import com.LibreGeoSocialApp.Utils.LocationUtils;

import android.content.Context;
import android.location.Location;

/**
 * This class provides the essential Node of social Network. All the 
 * contents must inherit of this class.
 * 
 * @author Roberto Calvo
 * @author <a href="mailto:rocapal@libresoft.es">rocapal@libresoft.es</a>
 * @author <a target="_blank" href="http://libregeosocial.morfeo-project.org/">http://libregeosocial.morfeo-project.org/</a>
 * @version 0.1
 */

public class GeoNode  {
	
	


	private Integer mId;
	private Double  mRadius;
	
	private float	mHeight;
	private DrawResource drawn;
	private boolean isloaded = false;
	
	private Location mLocation;
	
	/**
     * Constructor used to create this object. 
     * @param id Identifier of social network (can be 0)
     * @param latitude The latitude of the node position 
     * @param longitude The longitude of the node position
     * @param radius The radius/error of the node position
     */

	public GeoNode(Integer id, Double latitude, Double longitude, Double radius)
	{
		super();
		mId = id;
		mRadius = radius;	
		
		mLocation = new Location ("gps");
		mLocation.setLatitude(latitude);
		mLocation.setLongitude(longitude);
		
		mHeight = 1.75f;
		
	}
	
	/**
     * Return the Identifier asociated to social network
     */
	
	public Integer getId() {
		return mId;
	}
	
	/**
     * Return the latitude of the node
     */
	public Double getLatitude() {
		return mLocation.getLatitude();
	}
	
	/**
     * Return the longitude of the node
     */
	public Double getLongitude() {
		return mLocation.getLongitude();
	}
	
	/**
     * Return the radius of the node
     */
	public Double getRadius() {
		return mRadius;
	}
	
	/**
     * Set the latitude of the node
     * 
     * @param latitude The latitude of the position node
     */
	public void setLatitude(Double latitude){
		mLocation.setLatitude(latitude);
	}
	
	/**
     * Set the longitude of the node
     * 
     * @param longitude The latitude of the position node
     */
	public void setLongitude(Double longitude){
		mLocation.setLongitude(longitude);
	}
	
	
	public Location getLocation ()
	{
		return mLocation;
	}
	
	public void setLocation (Location loc)
	{
		mLocation = loc;
	}
	
	public void setHeight(float height){
		this.mHeight = height;
	}
	
	public float[] getDrawnPosition(){
		return drawn.getResPosition();
	}
	
	public DrawResource getDrawn(){
		return drawn;
	}
	
	public boolean isLoaded(){
		return isloaded;
	}
	
	public void setLoaded(boolean isloaded){
		this.isloaded = isloaded;
	}
	
	public void setDrawnValues(float azimuth, float elevation, float distance){
		drawn.setValues(azimuth, elevation, distance);
	}
	
	public void createDrawn(Context context, String name){
		this.drawn = new DrawResource(context, name);
	}
	
	public void bringDrawnToFront(){
		drawn.bringToFront();
	}
	
	public void drawDistance(boolean draw){
		drawn.drawDistance(draw);
	}
	
	public void invalidateDrawn(){
		drawn.invalidate();
	}
	
	public float distanceToResource(float[] source){
		float d;
		
		Location sLocation = new Location ("gps");
		sLocation.setLatitude(source[0]);
		sLocation.setLongitude(source[1]);
		
		d = sLocation.distanceTo(mLocation);
		
		return d;
	}
	
	public float azimuthToResource(float source[]){
		float azimuth, resource[] = new float[2];
		
		resource[0] = (float) mLocation.getLatitude();
		resource[1] = (float) mLocation.getLongitude();
		
		azimuth = LocationUtils.calculateResourceAzimuth(source, resource);
		
		return azimuth;
	}
	
	public float rollToResource(float distance){
		float roll;
		
		roll = LocationUtils.calculateRoll(distance, mHeight);
		
		return roll;
	}
}
