/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Raúl Román López <rroman@gsyc.es>
 *
 */

package com.LibreGeoSocialApp.Camera.Overlays;

import com.LibreGeoSocialApp.R;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.RectF;
import android.view.View;

public class DrawResource extends View {
	
	private static final float MAX_AZIMUTH_VISIBLE = 25f; 
	private static final float MAX_ELEVATION_VISIBLE = 20;
	private static final float mRADIUS = 5;
	
	private float posx, posy;
	private String name;
	private float azimuth,elevation,distance;
	private boolean draw_distance;
	
	public DrawResource(Context context, String name) {
		super(context);
		// TODO Auto-generated constructor stub
		this.name = name;
		this.draw_distance = false;
	}
	
	public void setValues (float azimuth, float elevation, float distance){
		this.azimuth = azimuth;
		this.elevation = elevation;
		this.distance = distance;
	}
	
	public void drawDistance(boolean draw){
		draw_distance = draw;
	}
	
	public float[] getResPosition(){
		float[] position = {posx, posy};
		
		return position;
	}
	
	@Override
	protected void onDraw(Canvas canvas){
		int w = canvas.getWidth();
        int h = canvas.getHeight();
        int cx = (w) / 2;
        int cy = (h) / 2;
        //float inclination;
        
        //inclination = elevation -CENTER_ELEVATION;
        if ((Math.abs(azimuth)>MAX_AZIMUTH_VISIBLE) || 
        		(Math.abs(elevation)>MAX_ELEVATION_VISIBLE))
        	return;
        
        posx = cx + (float)(Math.sin(Math.toRadians(azimuth))/Math.sin(Math.toRadians(MAX_AZIMUTH_VISIBLE))) * cx;
        posy = cy + (float)(Math.sin(Math.toRadians(elevation))/Math.sin(Math.toRadians(MAX_ELEVATION_VISIBLE))) * cy;
        
        
//		RectF oval = new RectF (posx - mRADIUS, posy - mRADIUS,
//				posx + mRADIUS, posy + mRADIUS);
		
		Paint paint = new Paint();
		paint.setARGB(250,255,0,0);
		paint.setAntiAlias(true);
		paint.setFakeBoldText(true);
		paint.setTextSize(20);


		Paint backPaint = new Paint();
		backPaint.setARGB(175,0,0,0);
		backPaint.setAntiAlias(true);

		RectF backRect = new RectF (posx + 2 + mRADIUS, posy - 5*mRADIUS,
				posx + 2 + mRADIUS + name.length()*13, posy - mRADIUS);

		//canvas.drawOval(oval,paint);
		Bitmap bitmap = BitmapFactory.decodeResource(getResources(), R.drawable.needle_white_16);
		canvas.drawBitmap(bitmap, posx - 8, posy - 16, null);
		canvas.drawRoundRect(backRect, 5, 5, backPaint);

		canvas.drawText (name, posx + 2*mRADIUS, posy - 2*mRADIUS, paint);
		
		if(draw_distance){
			String dist = Float.toString(((float)((int) (distance*1000)))/1000) + " m";
			
			RectF distRect = new RectF (posx + 2 + mRADIUS, posy + mRADIUS,
					posx + 2 + mRADIUS + dist.length()*13, posy + 5*mRADIUS);
			
			canvas.drawRoundRect(distRect, 5, 5, backPaint);
			
			canvas.drawText (dist, posx + 2*mRADIUS, posy + 4*mRADIUS, paint);
		}
		super.onDraw(canvas);
	}

}