/*
 *
 *  Copyright (C) 2008 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *
 */

package com.LibreGeoSocialApp;


import java.lang.reflect.Method;
import java.util.List;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SubMenu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.User;
import com.LibreGeoSocialApp.OMF.OMFManager;
import com.LibreGeoSocialApp.Overlay.PositionOverlay;
import com.LibreGeoSocialApp.Resources.ResourceManager;
import com.LibreGeoSocialApp.Routes.ILocationServiceListener;
import com.LibreGeoSocialApp.Routes.LocationService;
import com.LibreGeoSocialApp.Routes.RouteMap;
import com.LibreGeoSocialApp.Routes.RoutePreferences;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.winterwell.jtwitter.Twitter;
import com.winterwell.jtwitter.TwitterException;

public class Home extends MapActivity implements Runnable, OnClickListener  {
	
    private ProgressDialog pd;
    
	LocationManager mLocationManager;
	Location mLocation;
	MyLocationListener mLocationListener;
	
	static Location currentLocation = null;
	
	TextView outlat;
	TextView outlong;
	Button btUpdateLocation;
	MapView mapView = null;
	ImageView imageUser;
	TextView txtUserName;
	TextView txtState;
	
	private static final int MENU_RESOURCES = Menu.FIRST + 1;
	private static final int MENU_STATE = Menu.FIRST + 2;
	private static final int MENU_TRACES = Menu.FIRST + 3;
	private static final int MENU_LOCATION = Menu.FIRST + 4;
	private static final int MENU_LOCATION_GPS = Menu.FIRST + 5;
	private static final int MENU_LOCATION_WIFI = Menu.FIRST + 6;
	private static final int MENU_LOCATION_OTHERS = Menu.FIRST + 7;	
	private static final int MENU_SETTINGS = Menu.FIRST + 8;
	
	private static final int DIALOG_UPDATE_STATE = 1;
	
	private static final int REQUEST_PREFERENCES = 1;
	
	// The state of gps is saved in this variable
	private static boolean gpsState;
	
	private String LocationType;
	
	private GeoPoint geoPoint;
	
	private User user;
	
	private PositionOverlay myposOverlay = null;
	
	public static Location getLocation()
	{
		return currentLocation;
	}
		
	protected void onCreate(Bundle savedInstanceState) {
		
		super.onCreate(savedInstanceState);
 
        setContentView(R.layout.home);
        
        LocationType = LocationManager.GPS_PROVIDER;
        
        btUpdateLocation = (Button)this.findViewById(R.id.btLocation); 
        btUpdateLocation.setOnClickListener(this);
        
        imageUser = (ImageView)this.findViewById(R.id.imgUser);
        imageUser.setImageResource(R.drawable.user_48);

        txtUserName = (TextView) this.findViewById(R.id.lbName);
        txtState = (TextView) this.findViewById(R.id.lbState);
        
        //writeSignalGPS();
                
        user = LibreGeoSocial.getInstance().getInfoUser(true, null);
        
        txtUserName.setText (user.getName());
        txtState.setText(user.getState());
        
        // Obtain gps state, and active gps
        gpsState = getGPSStatus();        
        setGPSStatus(true);
        
        
        /* Start location service */
		startService(new Intent(this,LocationService.class));		
		
		LocationService.setProvider (LocationManager.GPS_PROVIDER);
		LocationService.setLocationUpdateListener ( new ILocationServiceListener () {
			
			public void updateCurrentLocation(Location loc) {
			                
                currentLocation = loc;
                showMap();

			}

		});
        
		refreshOMFdata();
    }
    
	protected void onResume(){
		super.onResume();
		
		user = LibreGeoSocial.getInstance().getInfoUser(true, null);
		
		if (user.getLatitude()==0 || user.getLongitude()==0){
			Toast.makeText(getBaseContext(), 
                    "No location found", 
                    Toast.LENGTH_LONG).show();
			return;
		}
			
		Location loc = new Location("MANUAL_PROVIDER");
		loc.setLatitude(user.getLatitude());
		loc.setLongitude(user.getLongitude());
		
		setCurrentLocation(loc);
		showMap();
	}
	
	
    @Override
    public void onDestroy() 
	{
    	SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
		
		// location service status
		boolean active = sharedPreferences.getBoolean (RoutePreferences.KEY_LOCATION_SERVICE, false);
		
		if (!active)
		{		
	    	// Return the original value of GPS
	    	setGPSStatus(gpsState); 
	    	
	    	// Stop Location Service
	    	stopService(new Intent(this,LocationService.class));
		}
    	
    	super.onDestroy();
	}
	
    private void refreshOMFdata(){
    	SharedPreferences sharedPreferences = 
	    	PreferenceManager.getDefaultSharedPreferences(getBaseContext());
	    
    	OMFManager.setPhone(sharedPreferences.getString(
    			LGS_Preferences.KEY_MOVISTAR_USER, ""));
    	OMFManager.setMovPwd(sharedPreferences.getString(
    			LGS_Preferences.KEY_MOVISTAR_PASSWORD, ""));
    	OMFManager.setMovLocalizamePwd(sharedPreferences.getString(
    			LGS_Preferences.KEY_MOVISTAR_LOCALIZAME_PWD, ""));
    }
    
    private void setCurrentLocation(Location loc) {
    	currentLocation = loc;	
    	LibreGeoSocial.getInstance().setUserPosition(loc.getLatitude(), loc.getLongitude());
    }
    
	public void onClick(View arg0) {
		// TODO Auto-generated method stub
		
		writeSignalGPS(); 
		
	}
	
	
	private boolean getGPSStatus()
	{
	    String allowedLocationProviders =
	        Settings.System.getString(getContentResolver(),
	        Settings.System.LOCATION_PROVIDERS_ALLOWED);
	 
	    if (allowedLocationProviders == null) {
	        allowedLocationProviders = "";
	    }
	 
	    return allowedLocationProviders.contains(LocationManager.GPS_PROVIDER);
	}   
	 
	private void setGPSStatus (boolean pNewGPSStatus)
	{
	    String allowedLocationProviders =
	        Settings.System.getString(getContentResolver(),
	        Settings.System.LOCATION_PROVIDERS_ALLOWED);
	 
	    if (allowedLocationProviders == null) {
	        allowedLocationProviders = "";
	    }      
	 
	    boolean networkProviderStatus =
	        allowedLocationProviders.contains(LocationManager.NETWORK_PROVIDER);
	 
	    allowedLocationProviders = "";
	    if (networkProviderStatus == true) {
	        allowedLocationProviders += LocationManager.NETWORK_PROVIDER;
	    }
	    if (pNewGPSStatus == true) {
	        allowedLocationProviders += "," + LocationManager.GPS_PROVIDER;
	    }   
	 
	    Settings.System.putString(getContentResolver(),
	        Settings.System.LOCATION_PROVIDERS_ALLOWED, allowedLocationProviders);     
	 
	    try
	    {
	        Method m =
	        	mLocationManager.getClass().getMethod("updateProviders", new Class[] {});
	        m.setAccessible(true);
	        m.invoke(mLocationManager, new Object[]{});
	    }
	    catch(Exception e)
	    {
	        Log.e("Error", e.getClass().getName());
	    }
	    
	    return;
	}

    
    private void writeSignalGPS() {
    	
    	DialogInterface.OnCancelListener dialogCancel = new DialogInterface.OnCancelListener() {

            public void onCancel(DialogInterface dialog) {

            }
          
        };
    	
		pd = ProgressDialog.show(this, this.getResources().getString(R.string.search), 
				this.getResources().getString(R.string.search_signal_gps), true, true, dialogCancel);
		
		Thread thread = new Thread(this);
		thread.start();

    }
    
    private void showMap()
    {
    	
    	if (mapView == null)
    		mapView = (MapView) findViewById(R.id.myMapView);
    	 
    	//mapView = new MapView(this, "0Tp7WsIDiyqp_gJLEiO55RXtkz7VsxOjQvL44SA");
        MapController mapControl = mapView.getController();
        
        geoPoint = new GeoPoint(  (int) (currentLocation.getLatitude() * 1000000), 
        						  (int) (currentLocation.getLongitude() * 1000000));
        mapControl.setZoom(20);
        mapControl.animateTo(geoPoint);
        
        if (myposOverlay == null)
        	myposOverlay = new PositionOverlay();        
        else
        	mapView.removeAllViews();
        
	    final List<Overlay> overlays = mapView.getOverlays();	 
	    overlays.clear();
	    overlays.add( myposOverlay );
	    
    	myposOverlay.setUser(user);
        
    	
	    mapView.setClickable(true);
        
    }
    
    @Override
    protected Dialog onCreateDialog(int id) {
        
    	switch (id) {
	    	case DIALOG_UPDATE_STATE:
	    		
	    		LayoutInflater factory = LayoutInflater.from(this);
	    		final View textEntryView = factory.inflate(R.layout.state, null);
	    		
	    		final EditText edit = (EditText) textEntryView.findViewById (R.id.txtstate);
	    		edit.setText("");
	    		
		        return new AlertDialog.Builder(this)	        
		        .setTitle("Update state")
		        .setView(textEntryView)
		        .setPositiveButton("update", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		
		            	if ( LibreGeoSocial.getInstance().setStatus(edit.getText().toString()))
		            	{		            		
		            		txtState.setText(edit.getText().toString());
		            		
		                    // User text   
		            	    SharedPreferences sharedPreferences = 
		            	    	PreferenceManager.getDefaultSharedPreferences(getBaseContext());

		            		// Update state in twitter
		            		Boolean syncTwitter = 
		            			sharedPreferences.getBoolean(LGS_Preferences.KEY_TWITTER_ACTIVE, false);
		            		if (syncTwitter)
		            		{
		            	        
		            	        Twitter twitter = new Twitter(
		            	        		sharedPreferences.getString(LGS_Preferences.KEY_TWITTER_USER, ""),
		            	        		sharedPreferences.getString(LGS_Preferences.KEY_TWITTER_PASSWORD, ""));
		            	        try {
		            	        	twitter.updateStatus(edit.getText().toString());
		            			} catch (TwitterException e) {		            				
		            				Log.e("Home",e.getMessage());
		            			}
		            		}
		            	}
		            	else
		            	{
		            		Toast.makeText(getBaseContext(), 
			     					   "Error while the status was setted: " + 
			     					   LibreGeoSocial.getInstance().getLastMsgServer(), 
			     					   Toast.LENGTH_LONG).show();
		            	}
		            	
		                /* User clicked OK so do some stuff */
		            }
		        })
		        .setNegativeButton("cancel", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		
		                /* User clicked cancel so do some stuff */
		            }
		        })
		        .create();
		    
    	}
    	
    	return null;
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	

        
        menu.add(0, MENU_TRACES, 0 ,"Routes")
		.setIcon(R.drawable.ruta_negro_48)
        .setAlphabeticShortcut('C');
        
        SubMenu sub = menu.addSubMenu (0, MENU_LOCATION, 0 , "Location")
        			  .setIcon(R.drawable.mundo);
        
        sub.add(0,MENU_LOCATION_GPS,0, "GPS");
        sub.add(0,MENU_LOCATION_WIFI,0, "WIFI");
        sub.add(0,MENU_LOCATION_OTHERS,0,"OTHERS");
        
        super.onCreateOptionsMenu(menu);
        menu.add(0, MENU_RESOURCES, 0, "Around me")
        		.setIcon(R.drawable.register)
                .setAlphabeticShortcut('A');
        
        menu.add(0, MENU_STATE, 0 , "Update State")
        		.setIcon(R.drawable.refresh_48)
                .setAlphabeticShortcut('U');
        
        menu.add(0, MENU_SETTINGS, 0 ,"Settings")
			.setIcon(R.drawable.spanner_48)
	        .setAlphabeticShortcut('S');
        
        return true;
    }

    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    	
    	case MENU_RESOURCES:
        	
    		//ResourceManager.getInstance().reset();
    		Intent i = new Intent(this,ResourceTab.class);
        	startActivity(i); 
        	
    		break;

    	case MENU_STATE:
    		
    		showDialog(DIALOG_UPDATE_STATE);
    		
    		break;
    		
    	case MENU_TRACES:
    		Intent i2 = new Intent(this,RouteMap.class);
        	startActivity(i2); 

        	break;
        	
    	case MENU_LOCATION_GPS:
    
    		LocationType = LocationManager.GPS_PROVIDER;
    		
    		break;
    		
    	case MENU_LOCATION_WIFI:
    
    		LocationType = LocationManager.NETWORK_PROVIDER;
    		
    		break;
    		
    	case MENU_LOCATION_OTHERS:
    		
    		Intent i1 = new Intent(this,LocationWays.class);
        	startActivity(i1);
    		
    		break;
    		
    	case MENU_SETTINGS:
    		
    		Intent i3 = new Intent(this,LGS_Preferences.class);
        	startActivityForResult(i3, REQUEST_PREFERENCES);
    		
    		break;
    	
    		
        }
        return super.onOptionsItemSelected(item);
    }
    
	protected void onActivityResult (int requestCode, int resultCode, Intent data) {	 
		
		switch (requestCode) { 
		
	    	case REQUEST_PREFERENCES:
	    		
	    		
	    		// Stop Location Service
	        	stopService(new Intent(this,LocationService.class));
	    		
	    		/* Start location service */
	    		startService(new Intent(this,LocationService.class));	
	    		
	    		break;
	    		
		}
		
	}	
    
	@Override
	protected boolean isRouteDisplayed() {
		// TODO Auto-generated method stub
		return false;
	} 
	
	public void run() {
    	
		mLocationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
		
		if (mLocationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
			
			Looper.prepare();
			
			mLocationListener = new MyLocationListener();
			
			mLocationManager.requestLocationUpdates(
					LocationType, 0, 0, mLocationListener);
			Looper.loop(); 
			Looper.myLooper().quit(); 
			
		} else {
			
			
            Toast.makeText(getBaseContext(), 
                    getString(R.string.gps_signal_not_found), 
                    Toast.LENGTH_LONG).show();
            
		}
	}
	
	
    
	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			pd.dismiss();
			mLocationManager.removeUpdates(mLocationListener);
	    	if (currentLocation!=null) {
	    		showMap();	    		
	    	}
		}
	};
	
	
    private class MyLocationListener implements LocationListener 
    {

        public void onLocationChanged(Location loc) {
            if (loc != null) {
                Toast.makeText(getBaseContext(), 
                    getResources().getString(R.string.gps_signal_found), 
                    Toast.LENGTH_LONG).show();
                setCurrentLocation(loc);
                handler.sendEmptyMessage(0);
            }
        }


        public void onProviderDisabled(String provider) {
            // TODO Auto-generated method stub
        }


        public void onProviderEnabled(String provider) {
            // TODO Auto-generated method stub
        }


        public void onStatusChanged(String provider, int status, 
            Bundle extras) {
            // TODO Auto-generated method stub
        }
    }



    
}

