/*
 *
 *  Copyright (C) 2008 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *
 */


package com.LibreGeoSocialApp;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.LibreGeoSocial.Api.LibreGeoSocial;

public class LibreGeoSocialApp extends Activity implements OnClickListener {
	
	private static final String TAG 		= "LibreGeoSocialApp";
	
    private static final int MENU_REGISTER = Menu.FIRST + 1;
    private static final int MENU_ABOUT = Menu.FIRST + 2;
    
    public static String KEY_USERNAME  = "APP_username";
    public static String KEY_PASSWORD  = "APP_password";

	private static final int DIALOG_ABOUT = 1;
	
    ImageView fBook;
	Button btLogin;
	Button btRegister; 
	
	TextView txtUser;
	TextView txtPasswd;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
    	
        super.onCreate(savedInstanceState);
        setContentView(R.layout.login);
        
        txtUser = (TextView) this.findViewById(R.id.txtLogin);
        txtPasswd = (TextView) this.findViewById(R.id.txtPass);
        
        btLogin = (Button)this.findViewById(R.id.btLogin); 
        btLogin.setOnClickListener(this);
        
        btRegister = (Button)this.findViewById(R.id.btRegister); 
        btRegister.setOnClickListener(this);

        fBook = (ImageView)this.findViewById(R.id.imgFacebook); 
        fBook.setImageResource(R.drawable.facebook);
        
        loadConfig();
        
        LibreGeoSocial.getInstance().setUrl(getString(R.string.urlServer));
        LibreGeoSocial.getInstance().setFormat("JSON");
        
        
    }
    
    private void loadConfig()
    {
    	
    	SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
		
		Editor editor = sharedPreferences.edit();
		
		// Username 
		String username = sharedPreferences.getString(KEY_USERNAME, null);
		
		if (username != null)			
			txtUser.setText(username);	
		
		
		// Password 
		String password = sharedPreferences.getString(KEY_PASSWORD, null);
		
		if (password != null)			
			txtPasswd.setText(password);	
    }
    
    private void saveConfig ()
    {
    	
    	SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
		
		Editor editor = sharedPreferences.edit();
		
		editor.putString(KEY_USERNAME, txtUser.getText().toString());
		editor.putString(KEY_PASSWORD, txtPasswd.getText().toString());
		
		editor.commit();
    }
    
    
    public void onClick(View v) { 
        
        if ( btLogin.isPressed() )
        {
	    	boolean login = LibreGeoSocial.getInstance().login( txtUser.getText().toString(), 
	    														txtPasswd.getText().toString());
	    	    	
	
	    	//FacebookHandler myFB = new FacebookHandler( txtUser.getText().toString(), 
	    	//											txtPasswd.getText().toString());
	    	
	    	
	    	if (login)
	    	{
	    		saveConfig();
	    		Intent i = new Intent(this,GeoSocialTab.class);
	    		startActivity(i); 
	    		
				this.finish();

	    	}
	    	else
	    	{
	    	
	    		Toast.makeText(getBaseContext(), 
	    					   "Error: " + LibreGeoSocial.getInstance().getLastMsgServer(), 
	    					   Toast.LENGTH_LONG).show();
	    		
	    	}
        }
        else if (btRegister.isPressed())
        {        	
          
            
    		Intent i = new Intent(this,Register.class);
        	startActivity(i);  
        }
    	
    }
    	
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	
        super.onCreateOptionsMenu(menu);
        menu.add(0, MENU_REGISTER, 0, "Register")
        		.setIcon(R.drawable.register)
                .setAlphabeticShortcut('A');
        
        menu.add(0, MENU_ABOUT, 0, "About")
			    .setIcon(R.drawable.icon)
		        .setAlphabeticShortcut('B');        
        
        return true;
    }
    
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    	
    	case MENU_REGISTER:
    		
    		Intent i = new Intent(this,Register.class);
        	startActivity(i); 
        	
    		break;
    		
        case MENU_ABOUT:
        	
        	showDialog(DIALOG_ABOUT);
        	
            break;
        }
        return super.onOptionsItemSelected(item);
    }
    
    
    @Override
    protected Dialog onCreateDialog(int id) {
        
    	switch (id) {  
		    	
        	case DIALOG_ABOUT:
        		
        		LayoutInflater factory = LayoutInflater.from(this);
	    		final View textEntryView = factory.inflate(R.layout.custom_dialog, null);
	    		
	    		TextView text = (TextView) textEntryView.findViewById (R.id.dialog_text);
	    		ImageView image = (ImageView) textEntryView.findViewById(R.id.dialog_image);

	    		text.setText("LibreGeoSocial v0.2.0\n" +
	    				     "Copyright (C) GSyC/LibreSoft \n\n" +
	    				     "This software is under GPL v3.0 license!\n\n"+	    				     
	    				     "Authors: \n"+
	    				     "  The developers of LibreSoft.");

	    		
		        return new AlertDialog.Builder(this)	        
		        .setTitle("About")		        
		        .setView(textEntryView) 
		        .setIcon(R.drawable.logo)
		        .setPositiveButton("Ok", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		            	
		            }
		        })
			    .create();
		        
    	}
    	return null;
    }



}

