/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *
 */

package com.LibreGeoSocialApp;

import java.util.ArrayList;
import java.util.Collections;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.ContextMenu;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.ContextMenu.ContextMenuInfo;
import android.widget.BaseAdapter;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.AdapterContextMenuInfo;

import com.LibreGeoSocial.Api.GeoNode;
import com.LibreGeoSocial.Api.GeoNodePositionComparator;
import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.Note;
import com.LibreGeoSocial.Api.Photo;
import com.LibreGeoSocialApp.Utils.LocationUtils;

public class NoteList extends ListActivity {

	static private ArrayList<GeoNode> mList;
	
	NoteAdapter mAdapter; 
	
	// Menu
	private static final int MENU_ADD_NOTE = Menu.FIRST + 1;
	private static final int MENU_LIST_NOTES = Menu.FIRST + 2;
		
	// Context menu
	private static final int MENU_DELETE_NOTE = Menu.FIRST + 1;
	private static final int MENU_PRIVACY_NOTE = Menu.FIRST + 2;
	
	private static final int REQUEST_PREFERENCES = 1;
	
	
	private static final int DIALOG_ADD_NOTE = 0;
	
	private EditText title;
	private EditText body;
	
	private static Boolean near_notes;
	
	public static Note note;
	
	public void onCreate(Bundle savedInstanceState) {
		
		super.onCreate(savedInstanceState);		  		
		
		mList = LibreGeoSocial.getInstance().getNotes(0.0,null);
		
		GeoNodePositionComparator comparator = new GeoNodePositionComparator();
		Collections.sort(mList, comparator);
		
  		// Adapter for ListView
        mAdapter = new NoteAdapter(this);
        setListAdapter(mAdapter);
        
        if ( mList.size()==0 )
        {
        	
    		Toast.makeText(getBaseContext(), 
    					   "There aren't notes in this moment!!", 
    					   Toast.LENGTH_LONG).show();
        	
        }
        
        near_notes = true;
        
        registerForContextMenu(this.getListView());

		
	}
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View view,  ContextMenuInfo menuInfo)
	{ 

		menu.add (0, MENU_DELETE_NOTE, 0, "Delete Note");
		menu.add (0, MENU_PRIVACY_NOTE, 0, "Change Privacy");         
		
	}
	
	
	@Override
	public boolean onContextItemSelected(MenuItem item) 
	{
		
		
		AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
		note = (Note) mList.get(info.position);
		
		switch (item.getItemId()) {
		
			case MENU_DELETE_NOTE:
				
				LibreGeoSocial.getInstance().deleteNote(note.getId());
				mList.remove(info.position);
				setListAdapter(mAdapter);
				
				Toast.makeText(getBaseContext(), 
  					   "Note deleted correctly!", Toast.LENGTH_LONG).show();
				
				
				break;
				
			case MENU_PRIVACY_NOTE:
				
				Intent i = new Intent(this,LGS_Preferences.class);				
				startActivityForResult(i,REQUEST_PREFERENCES);
				
				break;
		
		}
		
		return true;
	}
	
	protected void onActivityResult (int requestCode, int resultCode, Intent data) {	 
		
		switch (requestCode) { 
		
	    	case REQUEST_PREFERENCES:
	    		
	    		// User text   
        	    SharedPreferences sharedPreferences = 
        	    	PreferenceManager.getDefaultSharedPreferences(getBaseContext());

        		// Update state in twitter
        		String notePrivacy = 
        			sharedPreferences.getString (LGS_Preferences.KEY_APP_NOTE_PRIVACY, "friends");
        		
				if ( LibreGeoSocial.getInstance().setPrivacyNote(note.getId(), notePrivacy ))
					
	        		Toast.makeText(getBaseContext(), 
	  					   "Upload note privacy correctly!!", 
	  					   Toast.LENGTH_LONG).show();
				
				else
					
					Toast.makeText(getBaseContext(), 
		  					   "There is a problem with the privacy!", 
		  					   Toast.LENGTH_LONG).show();
	    		
	    		break;
	    		
		}
		
	}	
	
	
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	
        super.onCreateOptionsMenu(menu);
        menu.add(0, MENU_ADD_NOTE, 0, "Add note")
        		.setIcon(R.drawable.comment_add_48)
                .setAlphabeticShortcut('A');
        
        menu.add(0, MENU_LIST_NOTES, 0, "Near notes")
			.setIcon(R.drawable.comment_38)
	        .setAlphabeticShortcut('B');
		        
        return true;
    }
	
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    	
    		case MENU_ADD_NOTE:
    			
    			showDialog(DIALOG_ADD_NOTE);
    			
    			break;
    			
    		case MENU_LIST_NOTES:
    			
    			if (near_notes)
    			{
    				mList = LibreGeoSocial.getInstance().getNotes(2.0,null);
    				near_notes = false;
    				item.setTitle("All notes");
    				item.setIcon(R.drawable.mundo);
    			}
    			else
    			{
    				mList = LibreGeoSocial.getInstance().getNotes(0.0,null);
    				near_notes = true;
    				item.setTitle("Near notes");
    				item.setIcon(R.drawable.comment_38);
    			}
    			
    			
    			GeoNodePositionComparator comparator = new GeoNodePositionComparator();
    			Collections.sort(mList, comparator);
    			mAdapter.notifyDataSetChanged();
    			
    			
    			
    			break;
    	}
    	
    	return super.onOptionsItemSelected(item);
    }
    
	
	@Override
	protected void onResume(){
        super.onResume();

    }
    @Override
    protected void onPause(){
       super.onPause();

    }
	
    protected void onListItemClick(ListView l, View v, int position, long id) {
        
    	Note note = (Note) mList.get(position);
		
    	Intent i = new Intent(this, NoteCard.class);
    	
    	i.putExtra("title", note.getTitle() );
    	i.putExtra("body", note.getBody() );
    	i.putExtra("date", note.getSince() );
    	i.putExtra("latitude", note.getLatitude() );
    	i.putExtra("longitude", note.getLongitude() );
    	
    	startActivity(i);
        	

    }
    
    @Override
    protected Dialog onCreateDialog(int id) {       
    	
    	switch (id) {
    	
	    	case DIALOG_ADD_NOTE:
	    		
	    		LayoutInflater factory = LayoutInflater.from(this);
	    		final View textEntryView = factory.inflate(R.layout.note, null);
	    		
	    		title = (EditText) textEntryView.findViewById (R.id.txtTitle);
	    		body = (EditText) textEntryView.findViewById (R.id.txtBody);
	    		
		        return new AlertDialog.Builder(this)	        
		        .setTitle("Add note")
		        .setView(textEntryView)
		        .setPositiveButton("add", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		            	
		                /* User clicked OK so do some stuff */ 
		            	
		            	Note note = new Note (null, 
		            						  title.getText().toString(), 
		            						  body.getText().toString(),
		            						  Home.currentLocation.getLatitude(),
		            						  Home.currentLocation.getLongitude(),
		            						  null);
		            	
		            	
		            	if ( LibreGeoSocial.getInstance().createNote(note) )
		            	{
		            		Toast.makeText(getBaseContext(), 
			     					   "Note added correctly!",
			     					   Toast.LENGTH_LONG).show();
		            		
		            		if (near_notes)
		            			mList = LibreGeoSocial.getInstance().getNotes(2.0,null);
		            		else
		            			mList = LibreGeoSocial.getInstance().getNotes(0.0,null);
		            		
		            		mAdapter.notifyDataSetChanged();
		            		
		            	}
		            	else
		            	{
		            		Toast.makeText(getBaseContext(), 
			     					   "Error while the status was setted: " + 
			     					   LibreGeoSocial.getInstance().getLastMsgServer(), 
			     					   Toast.LENGTH_LONG).show();
		            	}
		            }
		        })
		        .setNegativeButton("cancel", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		
		                /* User clicked cancel so do some stuff */
		            }
		        })
		        .create();

        	
    	}
		return null;
    
	}
    

	public static class NoteAdapter extends BaseAdapter {

		private Context mContext;
		
		public NoteAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return mList.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            // Make up a new view
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.photolist, null);
	        } else {
	            // Use convertView if it is available
	            view = convertView;
	        }
			
	        
	        Note note = (Note) mList.get(position);	       	       

	        ImageView img = (ImageView) view.findViewById(R.id.image);
	        img.setImageResource(R.drawable.paper_pencil_48);
			
			TextView t = (TextView) view.findViewById(R.id.title);
			t.setText(note.getTitle());
			

			// Calculate distance
			float dist = LocationUtils.calculateDistance (Home.getLocation(), 
														  note.getLocation());
			
			String distString = LocationUtils.displayDistance(dist, LocationUtils.AUTO);
			
			t = (TextView) view.findViewById(R.id.owner);
			t.setText("Dist: " + distString + "\n" + note.getBody());
	        
			return view;
	        
		}

	}

	
}
