/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *
 */

package com.LibreGeoSocialApp.Overlay;

import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.RectF;
import android.location.Location;

import com.LibreGeoSocial.Api.User;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.Projection;

public class PositionOverlay extends Overlay
{
	private final int mRadius = 5;
	
	private User mUser;
	
	private boolean mPerimeter = false;
	private Location mLocationPerimeter;
	
	public void setUser (User user)
	{
		mUser = user;
		
	}
	
	public void withPerimeter(Location location)
	{
		mPerimeter = true;
		mLocationPerimeter = location;
	}
	
	@Override
	public void draw(Canvas canvas, MapView mapView, boolean shadow)
	{
	
        GeoPoint geoPoint = new GeoPoint(  (int) (mUser.getLatitude() * 1000000), 
				  						   (int) (mUser.getLongitude() * 1000000));
        
		Point point= new Point();
		Projection projection = mapView.getProjection();
	        
		projection.toPixels(geoPoint, point);
	        
		RectF oval = new RectF (point.x - mRadius, point.y - mRadius,
								point.x + mRadius, point.y + mRadius);
	    
		Paint paint = new Paint();
		paint.setARGB(250,255,0,0);
		paint.setAntiAlias(true);
		paint.setFakeBoldText(true);
		
	        
		Paint backPaint = new Paint();
		backPaint.setARGB(175,0,0,0);
		backPaint.setAntiAlias(true);
	        
		String text = mUser.getName();
		RectF backRect = new RectF (point.x + 2 + mRadius, point.y - 3*mRadius,
									point.x + 60, point.y + mRadius);
	        
		canvas.drawOval(oval,paint);
		
		if (mPerimeter)
		{
			
	        GeoPoint geoPointPerimeter = new GeoPoint(  (int) (mLocationPerimeter.getLatitude() * 1000000), 
					   						            (int) (mLocationPerimeter.getLongitude() * 1000000));
	        
			Point pointPerimeter = new Point();
			Projection projectionPerimeter = mapView.getProjection();
		        
			projectionPerimeter.toPixels(geoPointPerimeter, pointPerimeter);
			
			Paint borderPaint = new Paint();
		    borderPaint.setStyle(Paint.Style.STROKE);
		    borderPaint.setAntiAlias(true);
		    borderPaint.setStrokeWidth(3);
		    borderPaint.setARGB(250,255,0,0);
		    
			canvas.drawCircle(pointPerimeter.x,pointPerimeter.y,100,borderPaint);
			
		}
		
		
		canvas.drawRoundRect(backRect, 5, 5, backPaint);
		
		canvas.drawText (text, point.x + 2*mRadius, point.y, paint);
	}
	
}
