/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *
 */


package com.LibreGeoSocialApp;

import java.util.ArrayList;
import java.util.Collections;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.ContextMenu;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.ContextMenu.ContextMenuInfo;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.AdapterContextMenuInfo;

import com.LibreGeoSocial.Api.GeoNode;
import com.LibreGeoSocial.Api.GeoNodePositionComparator;
import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.Photo;
import com.LibreGeoSocialApp.Utils.LocationUtils;



public class PhotoList_LGS extends ListActivity {

	
	private static final int MENU_PHOTOS = Menu.FIRST + 1;
	private static final int MENU_MY_PHOTOS = Menu.FIRST + 2;
	private static final int MENU_UPLOAD_PHOTOS = Menu.FIRST + 3;
	private static final int MENU_PREFERENCE_PHOTOS = Menu.FIRST + 4;

	// Context menu
	private static final int MENU_DELETE_PHOTO = Menu.FIRST + 1;
	private static final int MENU_PRIVACY_PHOTO = Menu.FIRST + 2;
	
	private static final int DIALOG_NO_PHOTOS = 0;
	
	private static final int REQUEST_PREFERENCES = 1;
	
	PhotoAdapter mAdapter; 
	static ArrayList<GeoNode> photoList;
	
	static Photo photo;
	
	public void onCreate(Bundle savedInstanceState) {
		
		super.onCreate(savedInstanceState);
		    

		// Get Photos
		photoList = LibreGeoSocial.getInstance().getPhotos(0.0,null);
		
		GeoNodePositionComparator comparator = new GeoNodePositionComparator();
		Collections.sort(photoList, comparator);
		
		
  		// Adapter for ListView
        mAdapter = new PhotoAdapter(this);
        setListAdapter(mAdapter);
        
        
        if ( photoList.size()==0 )
        {
    		showDialog(DIALOG_NO_PHOTOS);
        }
        
        registerForContextMenu(this.getListView());

		
	}
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View view,  ContextMenuInfo menuInfo)
	{ 


		menu.add (0, MENU_DELETE_PHOTO, 0, "Delete Photo");
		menu.add (0, MENU_PRIVACY_PHOTO, 0, "Change Privacy");
         
		
	}
	
	@Override
	public boolean onContextItemSelected(MenuItem item) 
	{
		
		
		AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
		photo = (Photo) photoList.get(info.position);
		
		switch (item.getItemId()) {
		
			case MENU_DELETE_PHOTO:
				
				LibreGeoSocial.getInstance().deletePhoto(photo.getId());
				photoList.remove(info.position);
				updateList();
				
				Toast.makeText(getBaseContext(), 
  					   "Photo deleted correctly!", Toast.LENGTH_LONG).show();
				
				
				break;
				
			case MENU_PRIVACY_PHOTO:
				
				Intent i = new Intent(this,LGS_Preferences.class);				
				startActivityForResult(i,REQUEST_PREFERENCES);
				
				break;
		
		}
		
		return true;
	}
	
	protected void onActivityResult (int requestCode, int resultCode, Intent data) {	 
		
		switch (requestCode) { 
		
	    	case REQUEST_PREFERENCES:
	    		
	    		// User text   
        	    SharedPreferences sharedPreferences = 
        	    	PreferenceManager.getDefaultSharedPreferences(getBaseContext());

        		// Update state in twitter
        		String photoPrivacy = 
        			sharedPreferences.getString (LGS_Preferences.KEY_APP_PHOTO_PRIVACY, "friends");
        		
				if ( LibreGeoSocial.getInstance().setPrivacyPhoto(photo.getId(), photoPrivacy ))
					
	        		Toast.makeText(getBaseContext(), 
	  					   "Upload photo privacy correctly!!", 
	  					   Toast.LENGTH_LONG).show();
				
				else
					
					Toast.makeText(getBaseContext(), 
		  					   "There is a problem with the privacy!", 
		  					   Toast.LENGTH_LONG).show();
	    		
	    		break;
	    		
		}
		
	}
	    		
	    		
	
	@Override
    protected Dialog onCreateDialog(int id) {
        
    	
		switch (id) {
			
	    	case DIALOG_NO_PHOTOS: 
	    		
		        return new AlertDialog.Builder(this)	        
		        .setTitle("No Photos!")
		        .setMessage("Currently, you can't see friend's pictures . Press 'Menu' and go to 'Upload Photo' to upload your photos!")
		        .setPositiveButton("Ok", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) { 

		            	
		            }
		        })
		        .create();
			
		
    	}
    	
		return null;
    
	}
	
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	
        super.onCreateOptionsMenu(menu);
        menu.add(0, MENU_PHOTOS, 0, "Show All Photos")
        		.setIcon(R.drawable.mundo)
                .setAlphabeticShortcut('A');
                
        menu.add(0, MENU_MY_PHOTOS, 0, "Show My Photos")
				.setIcon(R.drawable.image_48)
		        .setAlphabeticShortcut('B');
        
        menu.add(0, MENU_UPLOAD_PHOTOS, 0, "Upload Photo")
				.setIcon(R.drawable.camera)
		        .setAlphabeticShortcut('C');    
        
        menu.add(0, MENU_PREFERENCE_PHOTOS, 0, "Settings")
				.setIcon(R.drawable.spanner_48)
		        .setAlphabeticShortcut('C');
		        
        return true;
    }
	
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    	
    		case MENU_PHOTOS:
    		
    			photoList = LibreGeoSocial.getInstance().getPhotos(0.0,null);
    			updateList();
    			break;
    			
    		case MENU_MY_PHOTOS:
    			
    			photoList = LibreGeoSocial.getInstance().getMyPhotos();  	
    			updateList();
    			break;
    			
    		case MENU_UPLOAD_PHOTOS:
    			
    			Intent i = new Intent(this, UploadPhoto.class);    			
    			startActivity(i); 
    	}
    	


    	
    	return super.onOptionsItemSelected(item);
    }
	
    private void updateList ()
    {
		GeoNodePositionComparator comparator = new GeoNodePositionComparator();
		Collections.sort(photoList, comparator);
    	mAdapter.notifyDataSetChanged();    	
    }
    
    protected void onListItemClick(ListView l, View v, int position, long id) {
    	
    	Photo myphoto = (Photo) photoList.get(position);

    	Intent i = new Intent(this, ViewMap.class);
    	
    	i.putExtra("latitude",myphoto.getLatitude());
    	i.putExtra("longitude",myphoto.getLongitude());
    	i.putExtra("url",myphoto.getUrl());
    	
    	startActivity(i);
    
	}
	
    
    
	public static class PhotoAdapter extends BaseAdapter {

		private Context mContext;
		
		public PhotoAdapter(Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return photoList.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            // Make up a new view
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.photolist, null);
	        } else {
	            // Use convertView if it is available
	            view = convertView;
	        }
			
	        
	        Photo myphoto = (Photo) photoList.get(position);
	        
	        ImageView img = (ImageView) view.findViewById(R.id.image);
	        img.setImageBitmap(myphoto.getBitmapImage());
	        
	        TextView t = (TextView) view.findViewById(R.id.title);
	        t.setText(myphoto.getName());
	        
	        String distString = null;
	        
			if ( myphoto.getLatitude()==-1.0 && 
					myphoto.getLongitude()==-1.0)
			{
				distString = "Unknow Location";
			}
			else
			{
				
				// Calculate distance
				float dist = LocationUtils.calculateDistance (Home.getLocation(), 
															  myphoto.getLocation());
				
				distString = LocationUtils.displayDistance(dist, LocationUtils.AUTO);
			}
			
			t = (TextView) view.findViewById(R.id.owner);
			t.setText("Dist: " + distString + "\n" + myphoto.getDescription());
			
			return view;
	        
		}

	}



}