/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *
 */


package com.LibreGeoSocialApp.Routes;


import java.util.ArrayList;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.widget.Toast;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocialApp.R;

public class LocationService extends Service {
	
	
	private static IRouteServiceListener ROUTE_UPDATE_LISTENER;
	
	private static ILocationServiceListener LOCATION_UPDATE_LISTENER;
	
	private static String TraceName;
	
	private static String LocProvider;
	
	private NotificationManager mNM;
	
	private LocationManager mLocationManager;
	private RoutePositionListener mLocationListener = null;
	
	// Location Unit (seconds:1 or minutes:60), by default: seconds.
	static Integer LocationUnit = null;
	
	// Location Periodic in seconds, by default 2 minutes.
	static Integer LocationPeriodic = null; 
	
	// Minimum distance of gps refresh
	static Integer MinimumDistance = null;
	
	// manager for routes
	private RouteManager routeManager;
	
	// 
	private static ArrayList<Location> points = new ArrayList<Location>();
	
	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}
	
   @Override
    public void onCreate() {
	   
	   // Display a notification about us starting.  We put an icon in the status bar.
       mNM = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);        
       //showNotification();
       
	   loadConfig();
	   startService();
	   
    }

   @Override
   public void onDestroy() {
       // Cancel the persistent notification.
	   mNM.cancel(1);

       if (mLocationListener != null)
    	   mLocationManager.removeUpdates(mLocationListener);
       
       // Tell the user we stopped.
       //Toast.makeText(this, "service stoped", Toast.LENGTH_SHORT).show();
   }
   
   public static void initRoute()
   {
	   points.clear();
   }
   
   public static void setProvider (String provider)
   {
	   LocProvider = provider;   
   }
   
   public static void setRouteUpdateListener (IRouteServiceListener l) 
   {
	   ROUTE_UPDATE_LISTENER = l;	
	   
	   if (ROUTE_UPDATE_LISTENER != null)
		   initRoute();
   }
   
   public static void setLocationUpdateListener (ILocationServiceListener l)
   {
	   LOCATION_UPDATE_LISTENER = l;
   }
   
   public static void setTraceName (String name) 
   {
	   TraceName = name;
	   
   }


    /**
     * Show a notification while this service is running.
     */
    private void showNotification() {
        // In this sample, we'll use the same text for the ticker and the expanded notification
        CharSequence text = "Route Service";

        // Set the icon, scrolling text and timestamp
        Notification notification = new Notification(R.drawable.ruta_negro_48, text,
                System.currentTimeMillis());

        // The PendingIntent to launch our activity if the user selects this notification
        PendingIntent contentIntent = PendingIntent.getActivity(this, 0,
                new Intent(this, RouteMap.class), 0);

        // Set the info for the views that show in the notification panel.
        notification.setLatestEventInfo(this, "Route Service",
                       text, contentIntent);

        // Send the notification.
        // We use a layout id because it is a unique number.  We use it later to cancel.
        mNM.notify(1, notification);
                
    }

    private void startService() {
    	
    				

    	
    	mLocationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
    	
    	
    	
		mLocationListener = new RoutePositionListener();
		
		
		
		mLocationManager.requestLocationUpdates( LocProvider, 
												 ((LocationPeriodic)* 1000) * LocationUnit, 
												 MinimumDistance, 
												 mLocationListener);
	}

    
    
    private class RoutePositionListener implements LocationListener 
    {

        public void onLocationChanged(Location loc) {

        	if (loc != null) {

				//currentLocation = loc;				
				//user.setLocation(loc);
        		
        		
        		if (ROUTE_UPDATE_LISTENER != null) {

        			if (routeManager == null)
        			{
        				routeManager = new RouteManager(getBaseContext());
        	    	
        				routeManager.setStorageSDCard(false);
        				routeManager.setTraceName(TraceName);
        			}
        			
            		routeManager.saveTrace(loc);
            		points.add(loc);
            		
        			ROUTE_UPDATE_LISTENER.updateRoutePoints(points);
        			
        		}

        		if (LOCATION_UPDATE_LISTENER != null)
        		{
        			LibreGeoSocial.getInstance().setUserPosition (loc.getLatitude(), 
        														  loc.getLongitude());
        			
        			LOCATION_UPDATE_LISTENER.updateCurrentLocation(loc);        			        			
        			
        		}
            }
            

        }


        public void onProviderDisabled(String provider) {
            // TODO Auto-generated method stub
        }


        public void onProviderEnabled(String provider) {
            // TODO Auto-generated method stub
        }


        public void onStatusChanged(String provider, int status, 
            Bundle extras) {
            // TODO Auto-generated method stub
        }
    }
    
    
    private void loadConfig ()
	{
		
		SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
		
		Editor editor = sharedPreferences.edit();
		
		String s_provider = sharedPreferences.getString(RoutePreferences.KEY_LOCATION_PROVIDERS, null);
			
		if (s_provider == null)
		{
			LocProvider = LocationManager.GPS_PROVIDER;
			editor.putString (RoutePreferences.KEY_LOCATION_PROVIDERS, LocProvider);
		}
		else
		{
			
			if (s_provider.equals("gps"))
				LocProvider = LocationManager.GPS_PROVIDER;
			
			else if (s_provider=="wifi")			
				LocProvider = LocationManager.NETWORK_PROVIDER;
			
		}
		// Time unit
		String s_unit = sharedPreferences.getString(RoutePreferences.KEY_LOCATION_UNITS, null);
		
		if (s_unit == null)
		{
			LocationUnit = 60;
			editor.putString(RoutePreferences.KEY_LOCATION_UNITS, "60");
		}
		else
		{			
			Integer unit = Integer.valueOf(s_unit);
				
			LocationUnit = unit;
		}

		// Time periodic
		String s_periodic = sharedPreferences.getString(RoutePreferences.KEY_LOCATION_PERIODIC, null);
		
		if (s_periodic==null)
		{
			LocationPeriodic = 2;
			editor.putString(RoutePreferences.KEY_LOCATION_PERIODIC, "2");
			
		}
		else
		{
			Integer periodic = Integer.valueOf(s_periodic);
			LocationPeriodic = periodic;
		}

		//Minimum Distance
		String s_minDistance = sharedPreferences.getString(RoutePreferences.KEY_LOCATION_DISTANCE, null);
		if (s_minDistance == null)
		{
			MinimumDistance = 10;
			editor.putString(RoutePreferences.KEY_LOCATION_DISTANCE, "20");
		
		}
		else
		{			
			Integer minDistance = Integer.valueOf(s_unit);				
			MinimumDistance = minDistance;
		}	

		editor.commit();
		
	}

}
