/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *
 */

package com.LibreGeoSocialApp;

import java.util.List;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.User;
import com.LibreGeoSocialApp.OMFActivity.SMS;
import com.LibreGeoSocialApp.Overlay.PositionOverlay;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

public class UserCard extends MapActivity {

	
	private ImageView imageUser;
	private TextView txtUserName;
	private TextView txtState;

	private MapView mapView;
	private User user;
	
	private static final int MENU_CALL = Menu.FIRST + 1;
	private static final int MENU_SMS = Menu.FIRST + 2;
	
	private static final int DIALOG_QUESTION_SMS = 1;
	
    public void onCreate(Bundle icicle) {
    	
         super.onCreate(icicle);
        
         setContentView(R.layout.user_card);
         
         imageUser = (ImageView)this.findViewById(R.id.imgUser);
         imageUser.setImageResource(R.drawable.user_48);

         txtUserName = (TextView) this.findViewById(R.id.lbName);
         txtState = (TextView) this.findViewById(R.id.lbState);         
                 
         Intent i = getIntent();
         
         Integer userId  = i.getIntExtra ("userId", Integer.MIN_VALUE);
         
         user = LibreGeoSocial.getInstance().getInfoUser(true, userId);
         
         txtUserName.setText (user.getName());
         txtState.setText(user.getState());  
         
         
         showMap();
        
    }

	@Override
	protected boolean isRouteDisplayed() {
		// TODO Auto-generated method stub
		return false;
	}
	
    private void showMap()
    {
    	
        mapView = (MapView) findViewById(R.id.myMapView);
    	 
    	//mapView = new MapView(this, "0Tp7WsIDiyqp_gJLEiO55RXtkz7VsxOjQvL44SA");
        MapController mapControl = mapView.getController();
        
        
        GeoPoint geoPoint = new GeoPoint(  (int) (user.getLatitude() * 1000000), 
        						  		   (int) (user.getLongitude() * 1000000));
        mapControl.setZoom(20);
        mapControl.setCenter(geoPoint);
        
        PositionOverlay myposOverlay = new PositionOverlay();
        myposOverlay.setUser(user);
        
	    final List<Overlay> overlays = mapView.getOverlays();	   
	    overlays.add( myposOverlay );
	    
	    mapView.setClickable(true);
        
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	
        super.onCreateOptionsMenu(menu);
        menu.add(0, MENU_CALL, 0, "Call friend")
        		.setIcon(R.drawable.phone)
                .setAlphabeticShortcut('C');
        
        SharedPreferences sharedPreferences = 
	    	PreferenceManager.getDefaultSharedPreferences(getBaseContext());
        
        if(sharedPreferences.getBoolean(LGS_Preferences.KEY_MOVISTAR_ACTIVE, false))
        	menu.add(0, MENU_SMS, 0, "Send SMS")
        		.setIcon(R.drawable.paper_pencil_48)
        		.setAlphabeticShortcut('S');
        
        return true;
    }

    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    	
    	case MENU_CALL:
    		SharedPreferences sharedPreferences = 
    	    	PreferenceManager.getDefaultSharedPreferences(getBaseContext());
    		
    		if(txtState.getText().toString().toUpperCase().equals("BUSY") && 
    				sharedPreferences.getBoolean(LGS_Preferences.KEY_MOVISTAR_ACTIVE, false)){
    			
    			showDialog(DIALOG_QUESTION_SMS);
    		}else{
    			Intent i = new Intent(Intent.ACTION_DIAL);
            	startActivity(i);
    		}
    		break;
    		
    	case MENU_SMS:
        	
    		Intent i = new Intent(this,SMS.class);
        	startActivity(i);
        	
    		break;
    	
        }
        return super.onOptionsItemSelected(item);
    }
    
    @Override
    protected Dialog onCreateDialog(int id) { 
        
    	switch (id) {
	    	case DIALOG_QUESTION_SMS:
	    		
	    		LayoutInflater factory = LayoutInflater.from(this);
	    		final View textEntryView = factory.inflate(R.layout.sms_warning, null);
	    		
	    		
		        return new AlertDialog.Builder(this)	        
		        .setTitle("Warning!")
		        .setView(textEntryView)
		        .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		            	//launch activity
		            	Intent i = new Intent(UserCard.this,SMS.class);
		            	startActivity(i);
		                /* User clicked OK so do some stuff */
		            }
		        })
		        .setNegativeButton("No", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		            	
		                /* User clicked cancel so do some stuff */
		            }
		        })
		        .create();
		    
    	}
    	
    	return null;
    }
}
