/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *
 */

package com.LibreGeoSocial.Api;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.CookieStore;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.util.Log;


public class LibreGeoSocial {

	/* TAG for log messages*/
	static private String TAG = "LibreGeoSocial";
	
	/* Singleton */
	static private LibreGeoSocial singleton = null;
	
	/* Format that server answer*/
	private String mFormat = "JSON";
	
	/* Url from server: Ex: http://193.147.51.135/social/ */
	private String mUrl = null;
	
	/* Cookies for connection */
	private CookieStore cookieStore;
	
	/* Message from last petition */
	private String lastMsg = null;
	
	/* Code from last petition */
	private Integer lastCode = 0;
	
	private User user = null;
	
	private ArrayList<User> mUserList = null;
	
	private ArrayList<User> mFriendList = null;
	
	static public LibreGeoSocial getInstance() 
	{
        if (singleton == null) {
            singleton = new LibreGeoSocial();
        }
        return singleton;
		
	}
	
	public void setFormat (String format)
	{
		
		if ( format.compareTo("JSON") != 0 && format.compareTo("XML") != 0) 
		{
			Log.e("TAG", "Unknow format\n");
		}
		
		mFormat = format;
		
	}
	
	public void setUrl (String Url)
	{
		mUrl = Url;
	}
	
	public String getLastMsgServer()
	{		
		return "[" + lastCode.toString() + "]: " + lastMsg;
	}
	
	
    /**
     * Do user register in the server.
     * 
     * @param User object with info user.
     * 
     * @return Boolean variable operation
     */
	
	public boolean registerUser (User user)
	{
		try {            
			
			DefaultHttpClient httpclient = new DefaultHttpClient();
			HttpPost httpost = new HttpPost(mUrl + "user/create/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("username", user.getUsername()));		
		    nvps.add(new BasicNameValuePair("password", user.getPassword()));
		    nvps.add(new BasicNameValuePair("first_name", user.getName()));
		    nvps.add(new BasicNameValuePair("last_name", "" ));
		    nvps.add(new BasicNameValuePair("email", user.getEmail() ));
		    nvps.add(new BasicNameValuePair("latitude", user.getLatitude().toString() ));
		    nvps.add(new BasicNameValuePair("longitude", user.getLongitude().toString() ));
		    nvps.add(new BasicNameValuePair("radius", user.getRadius().toString() ));
			
		    
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 
    
		    HttpResponse response = httpclient.execute(httpost);
		    
		    HttpEntity entity = response.getEntity(); 
		    
            String str = convertStreamToString(entity.getContent());
            
            
            return processMessage(str);
            
		} catch (IOException e) {
	        // TODO Auto-generated catch block
	        Log.e(TAG,e.getMessage());
	        return false;
		}
	
	}
	
	
    /**
     * Do login with server. If login is ok, the session cookie is saved
     * 
     * @param username The username
     * @param password The password
     * 
     * @return Boolean
     */
	
	public boolean login (String username, String password) 
	{
        try {            
			
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "user/login/?format=JSON"); 
						
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("username", username));
		    nvps.add(new BasicNameValuePair("password", password)); 
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 
    
		    HttpResponse response = httpclient.execute(httpost);
		    
		    HttpEntity entity = response.getEntity(); 
		    
            String str = convertStreamToString(entity.getContent());
            
            if ( processMessage(str) )            
            	cookieStore = httpclient.getCookieStore();            
            else
            	return false;
            
		    /*
		    List<Cookie> cookies = cookieStore.getCookies();  
		    
			for (int i = 0; i < cookies.size(); i++) {
				Log.d("httpPost", "- " + cookies.get(i).toString());
		    } 
			*/
		    
		} catch (IOException e) {
		        // TODO Auto-generated catch block
		        Log.e(TAG,e.getMessage());
		        return false;
		}

		
		
		return true;		
		
	}

    /**
     * Return all users in the server
     * 
     * @param reaload If is false, data obtains of cache.
     * 
     * @return The user list in array list.
     */
	
	public ArrayList<User> getUserList (boolean reload)
	{
	
		if (reload == false && mUserList != null)
			return mUserList;
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpGet httpget = new HttpGet(mUrl + "user/list/?format=JSON"); 
			
			httpclient.setCookieStore(cookieStore);
			HttpResponse response = httpclient.execute(httpget);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
	        
	        return parseList(str,mUserList);
			
        
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
		
	}
	
	public ArrayList<User>  getFriendsList2 (boolean reload)
	{
		
		return getFriendsList2(reload);
		
		/*
		if (reload == false && user!=null)
		{
			return user.getFriends();
		}
		
		this.getInfoUser(true);
		return user.getFriends();
		*/
		
	}
	
    /**
     * Return all friends of login user.
     * 
     * @param reaload If is false, data obtains of cache.
     * 
     * @return The friend list in array list.
     */
	
	public ArrayList<User> getFriendsList (boolean reload)
	{
		if (reload == false && mFriendList != null)
			return mFriendList;
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "user/" + user.getId().toString() + "/friends/?format=JSON"); 
		    
			httpclient.setCookieStore(cookieStore);
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
        
	        
	        return parseList(str, mFriendList);
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
		

	}
	
    /**
     * Return the info user
     * 
     * @param reaload If is false, data obtains of cache.
     * @param userId The identifier of user 
     * 
     * @return The user info
     */
	
	public User getInfoUser (boolean reload, Integer userId)
	{
		
		if ( reload == false && user != null)
			return user;
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpget = null;
			
			if (userId==null)
				httpget = new HttpPost(mUrl + "user/my_data/?format=JSON");
			else
				httpget = new HttpPost(mUrl + "user/" + userId.toString() + "/data/?format=JSON");
			
			httpclient.setCookieStore(cookieStore);
			HttpResponse response = httpclient.execute(httpget);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());	        
	        
	        Log.e(TAG, str);
	        
            JSONObject json;
            String username, name, email, status;
            Double latitude, longitude, radius;
            Integer id;
            
            try {
				json = new JSONObject(str);
				lastCode = json.getInt("code");
				if (lastCode==200)
				{
					id = json.getInt("id");
					username = json.getString("username");
					name = json.getString("name");
					email = json.getString("email");
					status = json.getString("status");
										
					latitude = json.getDouble("latitude");
					longitude = json.getDouble("longitude");
					radius = json.getDouble("radius");					
					
					if (userId==null)
					{
						user = new User(id,name,username,email,status,latitude,longitude,radius,null);
						return user;
						
					}
					else
						return new User(id,name,username,email,status,latitude,longitude,radius,null);
					
				}
				else
				{
					lastMsg = json.getString("description");
					return null;					
				}
				
			} catch (JSONException e) {	
				lastCode = 500;
				lastMsg = "Error in json parsed: " + e.getMessage();
				Log.e(TAG, e.getMessage());
				return null;
			}
			
        
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
		
	}
	
    /**
     * Set the status of user.
     * 
     * @param status Text of the status
     * 
     * @return Boolean
     */
	
	public boolean setStatus (String status)
	{
	
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "user/" + user.getId().toString() + "/set_status/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("status", status )); 
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 		    
			httpclient.setCookieStore(cookieStore);
			
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
	        
            return processMessage(str);

		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
		
	}
	
    /**
     * Set friend of the user
     * 
     * @param idFriend Identifier of friend
     * 
     * @return Boolean variable
     */
	
	public boolean setFriend (Integer idFriend)
	{
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "user/relation/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("userid1", user.getId().toString() )); 
		    nvps.add(new BasicNameValuePair("userid2", idFriend.toString() ));
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 		    
			httpclient.setCookieStore(cookieStore);
			
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
	        
            return processMessage(str);

		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
		
		
	}
	
    /**
     * Delete a relation of friend
     * 
     * @param idFriend Identifier of friend
     * 
     * @return Boolean variable
     */
	public boolean deleteFriend (Integer idFriend)
	{
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "user/relation/delete/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("userid1", user.getId().toString() )); 
		    nvps.add(new BasicNameValuePair("userid2", idFriend.toString() ));
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 		    
			httpclient.setCookieStore(cookieStore);
			
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());
	        
	        Log.e(TAG, str);
	        
            return processMessage(str);

		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
		
		
	}
	
    /**
     * Set an user geo-position 
     * 
     * @param latitude latitude coordenate
     * @param longitude longitude coordenate
     * 
     * @return Boolean variable
     */
	public boolean setUserPosition (Double latitude, Double longitude)
	{

		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "user/" + user.getId().toString() + "/position/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    nvps.add(new BasicNameValuePair("latitude", latitude.toString()));
		    nvps.add(new BasicNameValuePair("longitude", longitude.toString()));
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 
		    
			httpclient.setCookieStore(cookieStore);
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());	        
	        
	        Log.e(TAG, str);
	        
	        return processMessage(str);
	                
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
		
	}
	
    /**
     * Create and upload note to server 
     * 
     * @param note It's a note objetc.     
     * 
     * @return Boolean variable
     */	
	public boolean createNote (Note note)
	{
		
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "note/create/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
			nvps.add(new BasicNameValuePair("title", note.getTitle()));
			nvps.add(new BasicNameValuePair("text", note.getBody()));
		    nvps.add(new BasicNameValuePair("latitude", note.getLatitude().toString()));
		    nvps.add(new BasicNameValuePair("longitude", note.getLongitude().toString() ));
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8)); 
		    
			httpclient.setCookieStore(cookieStore);
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());	        
	        
	        Log.e(TAG, str);
	        
	        return processMessage(str);
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return false;
		}
	}
	
    /**
     * Get all notes geo-located     
     * 
     * @return Return all notes in Array List
     */
	public ArrayList<Note> getNoteList ()
	{
	
		try
		{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			
			HttpPost httpost = new HttpPost(mUrl + "note/list/?format=JSON"); 
			
			List <NameValuePair> nvps = new ArrayList <NameValuePair>();
			
		    httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));
		    
			httpclient.setCookieStore(cookieStore);
			HttpResponse response = httpclient.execute(httpost);
	
		    HttpEntity entity = response.getEntity(); 
		    
	        String str = convertStreamToString(entity.getContent());	        
	        
	        Log.e(TAG, str);
	        
	        JSONObject json;
	        
	        try {
				json = new JSONObject(str);
				lastCode = json.getInt("code");
								
				if (lastCode != 200)
					return null;
				
				ArrayList<Note> list = new ArrayList<Note>();
				JSONArray array = json.getJSONArray("notes");
				String title, text, since;
				Double latitude, longitude;				
				
				Integer id = null;
				for (int i = 0; i < array.length(); i++) 
				{
					JSONObject obj = array.getJSONObject(i);
										
					id = obj.getInt("id");
					title = obj.getString("title");				
					text = obj.getString("text");
					
					latitude = obj.getDouble("latitude");
					longitude = obj.getDouble("longitude");
					
					since = obj.getString("since");
					
					Note note = new Note (id,title,text,latitude,longitude, since);
															
					list.add(note);
				}
				
				return list;
				
			} catch (JSONException e) {	
				lastCode = 500;
				lastMsg = "Error in json parsed: " + e.getMessage();
				Log.e(TAG, e.toString());
				return null;
			}	  
	        
		}
		
		catch (IOException e) {
	        Log.e(TAG,e.getMessage());
	        return null;
		}
	}
	
	
    private String convertStreamToString (InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is), 8*1024);
        StringBuilder sb = new StringBuilder();
 
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        
        return sb.toString();

    }
	
	
	private boolean processMessage (String str)
	{
		
        try {
        	JSONObject json;
			json = new JSONObject(str);
			lastCode = json.getInt("code");
			lastMsg  = json.getString("description");
			
			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		
		return true;
		
	}
	
	private ArrayList<User> parseList (String str, ArrayList<User> list)
	{
		 
        JSONObject json;
        
        try {
			json = new JSONObject(str);
			lastCode = json.getInt("code");
							
			if (lastCode != 200)
				return null;
			
			list = new ArrayList<User>();
			JSONArray array = json.getJSONArray("users");
			String username, name, status;
			Double latitude, longitude;
			
			Integer id = null;
			for (int i = 0; i < array.length(); i++) 
			{ 
				JSONObject obj = array.getJSONObject(i);
				username = obj.getString("username");
				id = obj.getInt("id");
				name = obj.getString("name");				
				status = obj.getString("status");
				
				latitude = obj.getDouble("latitude");
				longitude = obj.getDouble("longitude");
				
				
				if (id==user.getId())
					continue;
				
				User user = new User(id,name,username,null,status,latitude,longitude,null,null);
				
				list.add(user);
			}
			
			return list;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return null;
		}
	}
	
}
