/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *
 */

package com.LibreGeoSocialApp;

import java.util.ArrayList;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.User;

public class FriendList extends ListActivity {

	
	FriendsAdapter mAdapter; 
	
	private static final int MENU_PEOPLE = Menu.FIRST + 1;
	private static final int MENU_FRIENDS = Menu.FIRST + 2;
	
	private static final int DIALOG_ADD_FRIEND = 0;
	private static final int DIALOG_REMOVE_FRIEND = 1;
	
	static private ArrayList<User> mList;
	
	/* if friends = true  => Friend List 
	 * if friends = false => World People List
 	 */
	private boolean friends = true;
	private User  userDialog = null;
	
	public void onCreate(Bundle savedInstanceState) {
		
		super.onCreate(savedInstanceState);		  		
		
		mList = LibreGeoSocial.getInstance().getFriendsList(true);
		
  		// Adapter for ListView
        mAdapter = new FriendsAdapter(this);
        setListAdapter(mAdapter);
        
        if ( mList.size()==0 )
        {
        	
    		Toast.makeText(getBaseContext(), 
    					   "You have not friends!!", 
    					   Toast.LENGTH_LONG).show();
        	
        }
	}
	
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	
        super.onCreateOptionsMenu(menu);
        menu.add(0, MENU_PEOPLE, 0, "Show All People")
        		.setIcon(R.drawable.mundo)
                .setAlphabeticShortcut('A');
                
        menu.add(0, MENU_FRIENDS, 0, "Show Friends")
				.setIcon(R.drawable.users_two_38)
		        .setAlphabeticShortcut('A');
		        
        return true;
    }
	
    protected void onListItemClick(ListView l, View v, int position, long id) {
    
    	userDialog = mList.get(position); 
    	
    	if (friends)
    	{
    		//showDialog(DIALOG_REMOVE_FRIEND);
    		
        	Intent i = new Intent(this, UserCard.class);
        	
        	i.putExtra("userId", userDialog.getId() );
        	startActivity(i);
        	
    	}
    	else
    	{
    		showDialog(DIALOG_ADD_FRIEND);
    	}
    }
    
    @Override
    protected Dialog onCreateDialog(int id) {
        
    	switch (id) {
        	case DIALOG_ADD_FRIEND:

		    	return new AlertDialog.Builder(this)
		        .setIcon(R.drawable.user_add_48)
		        .setTitle("Add Friend Petition")
		        .setMessage("Do you want to add '" + this.userDialog.getUsername() +"' as a friend?")
		        .setPositiveButton("OK", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		
		                /* User clicked OK so do some stuff */
		            	if ( LibreGeoSocial.getInstance().setFriend(userDialog.getId() ) )
		            	{
		            		Toast.makeText(getBaseContext(), 
		     					   "Friend added correctly!", 
		     					   Toast.LENGTH_LONG).show();
		            	}
		            	else
		            	{
		            		Toast.makeText(getBaseContext(), 
			     					   "Error while the friend was added!", 
			     					   Toast.LENGTH_LONG).show();
		            	}
		            }
		        })
		        .setNegativeButton("CANCEL", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		
		                /* User clicked Cancel so do some stuff */
		            }
		        })
		        .create();

		    
        	case DIALOG_REMOVE_FRIEND:
        		
		    	return new AlertDialog.Builder(this)
		        .setIcon(R.drawable.user_delete_48)
		        .setTitle("Remove Friend Petition")
		        .setMessage("Do you want to remove '" + this.userDialog.getUsername() +"' as a friend?")
		        .setPositiveButton("OK", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		
		                /* User clicked OK so do some stuff */
		            	if ( LibreGeoSocial.getInstance().deleteFriend(userDialog.getId() ) )
		            	{
		            		mAdapter.notifyDataSetChanged();
		            		
		            		Toast.makeText(getBaseContext(), 
		     					   "Friend deleted correctly!", 
		     					   Toast.LENGTH_LONG).show();
		            		
		            		
		            	}
		            	else
		            	{
		            		Toast.makeText(getBaseContext(), 
			     					   "Error while the friend was deleted!", 
			     					   Toast.LENGTH_LONG).show();
		            	}
		            }
		        })
		        .setNegativeButton("CANCEL", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		
		                /* User clicked Cancel so do some stuff */
		            }
		        })
		        .create();
		    
    	}
		return null;
    
	}
    
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    	
    		case MENU_PEOPLE:
    		
    			mList = LibreGeoSocial.getInstance().getUserList(false);
    			mAdapter.notifyDataSetChanged();
    			friends = false;
    			break;
    			
    		case MENU_FRIENDS:
    			
    			mList = LibreGeoSocial.getInstance().getFriendsList(true);
    			mAdapter.notifyDataSetChanged();
    			friends = true;
    			break;
    	}
    	
    	return super.onOptionsItemSelected(item);
    }
	public static class FriendsAdapter extends BaseAdapter {

		private Context mContext;
		
		public FriendsAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return mList.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            // Make up a new view
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.photolist, null);
	        } else {
	            // Use convertView if it is available
	            view = convertView;
	        }
			
	        
	        User user = mList.get(position);	       	       

	        ImageView img = (ImageView) view.findViewById(R.id.image);	        
	        img.setImageResource(R.drawable.user_48);
			
			TextView t = (TextView) view.findViewById(R.id.title);
			t.setText( user.getName() + " (" + user.getUsername() +")");
			
			t = (TextView) view.findViewById(R.id.owner);
			t.setText(user.getState());
	        
			return view;
	        
		}

	}

	
}
