/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Roberto Calvo Palomino <rocapal@gsyc.es>
 *
 */

package com.LibreGeoSocialApp;

import java.util.ArrayList;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.Note;

public class NoteList extends ListActivity {

	static private ArrayList<Note> mList;
	
	NoteAdapter mAdapter; 
	
	private static final int MENU_ADD_NOTE = Menu.FIRST + 1;
	
	private static final int DIALOG_ADD_NOTE = 0;
	
	public void onCreate(Bundle savedInstanceState) {
		
		super.onCreate(savedInstanceState);		  		
		
		mList = LibreGeoSocial.getInstance().getNoteList();
		
  		// Adapter for ListView
        mAdapter = new NoteAdapter(this);
        setListAdapter(mAdapter);
        
        if ( mList.size()==0 )
        {
        	
    		Toast.makeText(getBaseContext(), 
    					   "There aren't notes in this moment!!", 
    					   Toast.LENGTH_LONG).show();
        	
        }
	}
	
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	
        super.onCreateOptionsMenu(menu);
        menu.add(0, MENU_ADD_NOTE, 0, "Add note")
        		.setIcon(R.drawable.comment_add_48)
                .setAlphabeticShortcut('A');
		        
        return true;
    }
	
    protected void onListItemClick(ListView l, View v, int position, long id) {
        
    	Note note = mList.get(position);
		
    	Intent i = new Intent(this, NoteCard.class);
    	
    	i.putExtra("title", note.getTitle() );
    	i.putExtra("body", note.getBody() );
    	i.putExtra("date", note.getSince() );
    	i.putExtra("latitude", note.getLatitude() );
    	i.putExtra("longitude", note.getLongitude() );
    	
    	startActivity(i);
        	

    }
    
    @Override
    protected Dialog onCreateDialog(int id) {       
    	
    	switch (id) {
    	
	    	case DIALOG_ADD_NOTE:
	    		
	    		LayoutInflater factory = LayoutInflater.from(this);
	    		final View textEntryView = factory.inflate(R.layout.note, null);
	    		
	    		final EditText title = (EditText) textEntryView.findViewById (R.id.txtTitle);
	    		final EditText body = (EditText) textEntryView.findViewById (R.id.txtBody);
	    		
		        return new AlertDialog.Builder(this)	        
		        .setTitle("Add note")
		        .setView(textEntryView)
		        .setPositiveButton("add", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		            	
		                /* User clicked OK so do some stuff */
		            	
		            	Note note = new Note (null, 
		            						  title.getText().toString(), 
		            						  body.getText().toString(),
		            						  Home.currentLocation.getLatitude(),
		            						  Home.currentLocation.getLongitude(),
		            						  null);
		            	
		            	
		            	if ( LibreGeoSocial.getInstance().createNote(note) )
		            	{
		            		Toast.makeText(getBaseContext(), 
			     					   "Note added correctly!",
			     					   Toast.LENGTH_LONG).show();
		            		
		            		mList = LibreGeoSocial.getInstance().getNoteList();
		            		mAdapter.notifyDataSetChanged();
		            		
		            	}
		            	else
		            	{
		            		Toast.makeText(getBaseContext(), 
			     					   "Error while the status was setted: " + 
			     					   LibreGeoSocial.getInstance().getLastMsgServer(), 
			     					   Toast.LENGTH_LONG).show();
		            	}
		            }
		        })
		        .setNegativeButton("cancel", new DialogInterface.OnClickListener() {
		            public void onClick(DialogInterface dialog, int whichButton) {
		
		                /* User clicked cancel so do some stuff */
		            }
		        })
		        .create();

        	
    	}
		return null;
    
	}
    
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    	
    		case MENU_ADD_NOTE:
    			
    			showDialog(DIALOG_ADD_NOTE);
    			
    			break;
    	}
    	
    	return super.onOptionsItemSelected(item);
    }
    
	public static class NoteAdapter extends BaseAdapter {

		private Context mContext;
		
		public NoteAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return mList.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            // Make up a new view
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.photolist, null);
	        } else {
	            // Use convertView if it is available
	            view = convertView;
	        }
			
	        
	        Note note = mList.get(position);	       	       

	        ImageView img = (ImageView) view.findViewById(R.id.image);	        
	        img.setImageResource(R.drawable.paper_pencil_48);
			
			TextView t = (TextView) view.findViewById(R.id.title);
			t.setText(note.getTitle());
			
			t = (TextView) view.findViewById(R.id.owner);
			t.setText(note.getBody());
	        
			return view;
	        
		}

	}

	
}
