/*
 *
 *  Copyright (C) 2010 GSyC/LibreSoft, Universidad Rey Juan Carlos.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Raúl Román López <rroman@gsyc.es>
 *
 */

package com.LibreGeoSocialApp.AR.Overlays;

import com.LibreGeoSocialApp.R;
import com.LibreGeoSocialApp.AR.ARUtils;

import android.content.Context;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.RadialGradient;
import android.graphics.RectF;
import android.graphics.Shader.TileMode;
import android.view.View;


public class DrawFocus extends View{

	public static final int CENTER = 0;
	public static final int SIDE = 1;
	
	private static final float mRADIUS = 55;
	private static final float fRADIUS = 5;
	private boolean isCompass = true;
	private int position = CENTER;
	
	private Context mContext;
	
	public DrawFocus(Context context) {
		super(context);
		mContext = context;
		// TODO Auto-generated constructor stub
	}
	
	public DrawFocus(Context context, boolean isCompass) {
		super(context);
		mContext = context;
		// TODO Auto-generated constructor stub
		
		this.isCompass = isCompass;
	}
	
	public void setCenter(int position){
		this.position = position;
	}
	
	@Override
	protected void onDraw(Canvas canvas){
		
		Paint paint = new Paint();
		paint.setStyle(Paint.Style.FILL);
		paint.setColor(Color.WHITE);
		paint.setFakeBoldText(true);
		
		int w = canvas.getWidth();
        int h = canvas.getHeight();
        float cx;
        float cy;
        
        switch (position){
        
        case CENTER:
            cx = ((float)w) / 2 ;
            cy = ((float)h) / 2 - 12.5f ;
            break;
            
        case SIDE:
        	cx = ((float)w) / 4;
            cy = ((float)h) / 2 ;
        	break;
        	
        default:
        	cx = 0;
        	cy = 0;
        }
		
		if (isCompass) {
//			Paint paint_radar = new Paint();
//			paint_radar.setARGB(127, 127, 255, 127);
//			paint_radar.setAntiAlias(true);
//			RectF oval = new RectF(cx + 160 - mRADIUS, cy + 75 - mRADIUS, cx
//					+ 160 + mRADIUS, cy + 75 + mRADIUS);
//			canvas.drawOval(oval, paint_radar);
//			canvas.drawText("N", cx + 156, cy + 75 - mRADIUS + 10, paint);
//			canvas.drawText("E", cx + 160 + mRADIUS - 8, cy + 79, paint);
//			canvas.drawText("S", cx + 156, cy + 75 + mRADIUS - 2, paint);
//			canvas.drawText("W", cx + 160 - mRADIUS + 2, cy + 79, paint);
			
			float compass_center_x = ARUtils.transformPixInDip(mContext, 180);
			float compass_center_y = ARUtils.transformPixInDip(mContext, 87.5f);
			
			canvas.drawBitmap(BitmapFactory.decodeResource(getResources(), R.drawable.compass_cool), 
					cx + compass_center_x - mRADIUS, cy + compass_center_y - mRADIUS, null);
			
			Paint centerPaint = new Paint();
			centerPaint.setStyle(Paint.Style.FILL);
			centerPaint.setShader(new RadialGradient(cx + compass_center_x, cy + compass_center_y, 5, Color.RED, Color.BLACK, TileMode.MIRROR)); 
			
	        RectF center = new RectF(cx + compass_center_x-5, cy + compass_center_y-5, cx + compass_center_x+5, cy + compass_center_y+5);
	        
			canvas.drawOval(center, centerPaint);
		}

		paint.setStyle(Paint.Style.STROKE);
		canvas.drawCircle(cx, cy, fRADIUS, paint);
		
		super.onDraw(canvas);
	}
}