/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

import java.util.Timer;
import java.util.TimerTask;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.view.Gravity;
import android.widget.Toast;

import com.LibreGeoSocialApp.Home;

/**
 * Servicio activado en las pruebas de geolocalización. Periódicamente (configurable)
 * analiza si el grupo ha alcanzado el lugar indicado, y en caso afirmativo, nofitica
 * al resto de actividades para que se salte al siguiente reto.
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class DistanceControllerService extends Service{
	/* TAG for log messages*/
	static private String TAG = "DistanceControllerService";
	
	/* Retardo de la inicialización del timer. */
	private static int DELAY = 0; // Delay for 0 seconds.
	/* Período del timer en el que se comprueba si se ha alcanzado la ubicación dada. */
	private static int PERIOD = 5000; // Repeat every 5000 msec.
	
	/* La distantia actual (en metros) que separa al equipo del lugar a encontrar. */
	public static float distanceToPlace = -1;
	private float previousDistanceToPlace = -1;
	
	private Timer timer;

	private TimerTask timerTask = new TimerTask() {
		@Override
		public void run() {
	    	double latitude = Home.currentLocation.getLatitude();
	    	double longitude = Home.currentLocation.getLongitude();
	    	float results[] = new float[10];
	    	Location.distanceBetween(latitude, longitude, Proof.lastTargetLatitude, Proof.lastTargetLongitude, results);
			
	    	distanceToPlace = results[0];
	    	
	    	if (previousDistanceToPlace != distanceToPlace){
	    		handler.sendEmptyMessage(0);
	    		previousDistanceToPlace = distanceToPlace;
	    	}
			
	    	if (distanceToPlace <= Proof.lastMinDistance){
	    		Proof.targetPlaceFound = true;
	    		Proof.distanceDifference = results[0];
	    		// Mando un broadcast para que cualquier pantalla quede enterada de que
	    		// se ha alcanzado el lugar marcado en el reto, y por tanto se debe
	    		// pasar a la siguiente prueba:
	    		Intent i = new Intent(Intent.ACTION_VIEW);
	            getApplicationContext().sendOrderedBroadcast(i, null);
	            DistanceControllerService.this.stopSelf();
	    	}
		}
	};
	
	private Handler handler = new Handler(){
		@Override
		public void handleMessage(Message msg){
			Context context = getApplicationContext();
			String text = "Distance to Place: " + distanceToPlace + " meter/s";
			int duration = Toast.LENGTH_SHORT;
			Toast toast = Toast.makeText(context, text, duration);
			toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
			toast.show();
		}
	};
	
	@Override
	public void onCreate() {
		timer = new Timer("DistanceControllerService");
		timer.scheduleAtFixedRate(timerTask, DELAY, PERIOD);
	}
	
	@Override
	public void onDestroy(){        
		stopSelf();
		timerTask.cancel();
	}

	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}

}