/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocialApp.R;

/**
 * Muestra la lista de gymkhanas en las que se puede participar actualmente y almacena
 * la seleccionada por el usuario. También se comprueba si el usuario que ha seleccionado
 * esa gymkhana pertenece ya a algún equipo (por posibles desconexiones, salidas de la
 * aplicación, etc.). En caso de que no tenga equipo, se lanzará la lista de equipos
 * disponibles, y en caso de tenerlo, se almacena y se da comienzo a la gymkhana.
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class EventList extends ListActivity implements Runnable { 
	/* TAG for log messages*/
	static private String TAG = "EventList";
	
	/* Gymkhanas entre las que el usuario puede seleccionar. */
	private Event[] events;
	/* Nombres de las gymkhanas entre las que el usuario puede seleccionar. */
	private String[] eventTitles;
	
	/* Message from last petition */
	private String lastMsg = null;
	
	/* Code from last petition */
	private Integer lastCode = 0;
	
	/* Si el usuario que participa en este evento ya se inscribió en un equipo, será true. */
	private boolean lastHasTeam = false;
	private Team lastTeam = null;
	private int lastCurrentProofId = -1;
	
	EventsAdapter mAdapter;
	static private ArrayList<String> mList;
	
	public static Event event = null;
	
	private ProgressDialog progressDialog = null;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	    
		Proof.lastIsLastProof = false;
		InformativeText.exit = false;
		InformativeText.reasonToExit = null;
		event = null;
		operate();
	}
		
	protected void onListItemClick(ListView l, View v, int position, long id) {
		Event selectedEvent = events[position];
		event = Event.findById(events, selectedEvent.getIdentifier());
		operate();
	}
	
	private void operate(){
		DialogInterface.OnCancelListener dialogCancel = new DialogInterface.OnCancelListener() {

            public void onCancel(DialogInterface dialog) {

            }
          
        };
    	
        String title = "Please Wait";
        String text = "Loading...";
		
		progressDialog = ProgressDialog.show(this, title, text, true, true, dialogCancel);
        Thread thread = new Thread(this);
        thread.start();
	}
	
	@Override
	public void run(){
		try{
			if (!event.equals(null)){
				boolean hasTeam = hasTeam(event);
				if (hasTeam){
					TeamList.team = lastTeam;
					if (TeamList.team.getFirstProofId() != this.lastCurrentProofId){
						Proof.proofId = this.lastCurrentProofId;
					}
					requestTeamList();
					handler.sendEmptyMessage(1);
				}else{
					handler.sendEmptyMessage(0);
				}
			}
		}catch(NullPointerException npe){
			Log.d(TAG, npe.toString());
			try{
				DefaultHttpClient httpclient = new DefaultHttpClient();
				URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/list/?format=json");
				HttpGet httpget = new HttpGet(uri);
				LibreGeoSocial LGS = LibreGeoSocial.getInstance();
				//User user = LGS.getInfoUser(false, null);
				httpclient.setCookieStore(LGS.getCookieStore());
				HttpResponse response = httpclient.execute(httpget);
				HttpEntity entity = response.getEntity();
				String str = convertStreamToString(entity.getContent());
				processEventListResponse(str);
			}catch(URISyntaxException urisex){
			    Log.e(TAG, urisex.getMessage());
		    }catch (IOException e) {
	            // TODO Auto-generated catch block
	            Log.e(TAG,e.getMessage());
		    }
		    handler.sendEmptyMessage(0);
		}
	}
	
	/** This adapter 
	 */
	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			try{
				if (!event.equals(null)){
					Intent i = null;
					int result = msg.what;
					if (result == 0){
						i = new Intent(EventList.this, TeamList.class);
					}else if (result == 1){
						i = new Intent(EventList.this, InformativeText.class);
					}
					progressDialog.cancel();
					startActivity(i);
				    EventList.this.setResult(RESULT_OK);
			        EventList.this.finish();
				}
			}catch(NullPointerException npe){
				Log.d(TAG, npe.toString());
				if (lastMsg.equals("ok")){
			    	try{
			    		CharSequence text = "\n\n\n     Select Event/Gymkhana     \n\n\n";
			    		Context context = getApplicationContext();
			    	    int duration = Toast.LENGTH_SHORT;
			    	    Toast toast = Toast.makeText(context, text, duration);
			    	    toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
			    	    toast.show();
		        	
			    	    progressDialog.cancel();
			    		// Adapter for ListView
			    		mAdapter = new EventsAdapter(EventList.this);
			    		setListAdapter(mAdapter);
			    	}catch(Exception e){
			    		Log.e(TAG,e.toString());
			    	}
			    }
			}
		}
	};
	
	public boolean hasTeam(Event event){
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + event.getIdentifier() + "/team_member/has_team/?format=json");
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpGet httpget = new HttpGet(uri);
			HttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String str = convertStreamToString(entity.getContent());
			return processHasTeamResponse(str);
		}catch(URISyntaxException urisex){
		    Log.e(TAG, urisex.getMessage());
	    }catch (IOException e) {
            // TODO Auto-generated catch block
            Log.e(TAG,e.getMessage());
	    }
	    return false;
	}
	
	private String convertStreamToString (InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is), 8*1024);
        StringBuilder sb = new StringBuilder();
 
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }
    
    private boolean processEventListResponse (String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);

			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			
			if (json.has("events") && lastMsg.toLowerCase().equals("ok")){
				JSONArray eventsObjects = json.getJSONArray("events");
				events = new Event[eventsObjects.length()];
				eventTitles = new String[eventsObjects.length()];
				mList = new ArrayList<String>();
				for (int i = 0; i < eventsObjects.length()-1; i++){
					JSONObject JSONEvent = (JSONObject)eventsObjects.get(i);
					int eventIdentifier = JSONEvent.getInt("identifier");
					String eventTitle = JSONEvent.getString("title");
					String eventCelebration = JSONEvent.getString("celebration");
					String eventPlace = JSONEvent.getString("place");
					String eventWelcomeText = JSONEvent.getString("welcome_text");
					String eventGoodbyeText = JSONEvent.getString("goodbye_text");
					int manager_id = JSONEvent.getInt("manager_id");
					String manager_name = JSONEvent.getString("manager_name");
					Manager manager = new Manager(manager_id, manager_name);
					events[i] = new Event(eventIdentifier, eventTitle, eventCelebration, 
											eventPlace, eventWelcomeText, eventGoodbyeText, manager);
					eventTitles[i] = events[i].getTitle();
					mList.add(eventTitles[i]);
				}
			}else{
				InformativeText.exit = true;
				InformativeText.reasonToExit = "    Sorry: Actually There    \n" +
											   "        Are No Events        ";
				Intent i = new Intent(this, InformativeText.class);
			    startActivity(i);
			    this.setResult(RESULT_OK);
		        this.finish();
			}

			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;
	}
    
    private boolean processHasTeamResponse (String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);

			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			
			if (lastCode != 200)
				return false;
			
			if (json.has("has_team"))
				lastHasTeam = json.getBoolean("has_team");
			else
				lastHasTeam = false;
			
			if (lastHasTeam){
				if (json.has("team")){
					JSONObject JSONTeam = json.getJSONObject("team"); 
					String teamName = JSONTeam.getString("team_name");
					int teamIdentifier = JSONTeam.getInt("identifier");
					int teamGroupId = JSONTeam.getInt("group_id");
					int teamFirstProofId = JSONTeam.getInt("id_first_proof");
					lastCurrentProofId = JSONTeam.getInt("id_current_proof");
					int teamNumCorrectResponses = JSONTeam.getInt("num_correct_responses");
					int teamNumIncorrectResponses = JSONTeam.getInt("num_incorrect_responses");
					double teamLatitude = JSONTeam.getDouble("latitude");
					double teamLongitude = JSONTeam.getDouble("longitude");
					double teamAltitude = JSONTeam.getDouble("altitude");
					double teamRadius = JSONTeam.getDouble("radius");
					String teamSince = JSONTeam.getString("since");
					lastTeam = new Team(teamIdentifier, teamName, teamFirstProofId, teamGroupId,
								teamNumCorrectResponses, teamNumIncorrectResponses,
								teamLatitude, teamLongitude, teamAltitude, teamRadius, teamSince);
				}
			}
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return lastHasTeam;
	}
    
    public static class EventsAdapter extends BaseAdapter {

		private Context mContext;
		
		public EventsAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			return mList.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            // Make up a new view
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.event_list, null);
	        } else {
	            // Use convertView if it is available
	            view = convertView;
	        }
	        
	        String title = (String)mList.get(position);
	        
	        //ImageView img = (ImageView) view.findViewById(R.id.image);	        
			
			TextView t = (TextView) view.findViewById(R.id.title);
			t.setText( title );
			
			t = (TextView) view.findViewById(R.id.owner);
			
			return view;       
		}
	}
    
    private boolean requestTeamList(){
    	boolean result = false; 
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + Integer.toString(EventList.event.getIdentifier()) + "/team/list/?format=json");
			HttpGet httget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httget);
			HttpEntity entity = response.getEntity();
			String str = convertStreamToString(entity.getContent());
			result = processTeamListResponse(str);
		}catch(URISyntaxException urisex){
		    Log.e(TAG, urisex.getMessage());
	    }catch (IOException e) {
            // TODO Auto-generated catch block
            Log.e(TAG,e.getMessage());
	    }
	    return result;
    }
    
    private boolean processTeamListResponse (String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);

			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			
			if (json.has("teams") && lastMsg.equals("ok")){
				JSONArray teamsObjects = json.getJSONArray("teams");
				TeamList.teams = new Team[teamsObjects.length()];
				TeamList.teamNames = new ArrayList<String>(teamsObjects.length());
				mList = new ArrayList<String>();
				for (int i = 0; i < teamsObjects.length()-1; i++){
					JSONObject JSONTeam = (JSONObject)teamsObjects.get(i);
					String teamName = JSONTeam.getString("team_name");
					int teamIdentifier = JSONTeam.getInt("identifier");
					int teamGroupId = JSONTeam.getInt("group_id");
					int teamFirstProofId = JSONTeam.getInt("id_first_proof");
					int teamNumCorrectResponses = JSONTeam.getInt("num_correct_responses");
					int teamNumIncorrectResponses = JSONTeam.getInt("num_incorrect_responses");
					double teamLatitude = JSONTeam.getDouble("latitude");
					double teamLongitude = JSONTeam.getDouble("longitude");
					double teamAltitude = JSONTeam.getDouble("altitude");
					double teamRadius = JSONTeam.getDouble("radius");
					String teamSince = JSONTeam.getString("since");
					TeamList.teams[i] = new Team(teamIdentifier, teamName, teamFirstProofId, teamGroupId,
								teamNumCorrectResponses, teamNumIncorrectResponses,
								teamLatitude, teamLongitude, teamAltitude, teamRadius, teamSince);
					TeamList.teamNames.add(i, TeamList.teams[i].getGroupName());
				}
			}

			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;
	}
	
}