/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;

import com.LibreGeoSocialApp.R;
import com.LibreGeoSocialApp.Routes.LocationService;

/**
 * Presenta la pantalla de bienvenida a la aplicación para la participación en Gymkhanas.
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class Gymkhana extends Activity implements OnClickListener{
	/* TAG for log messages*/
	static private String TAG = "Gymkhana";
	
    private static final int MENU_ABOUT = Menu.FIRST + 1;
    
	static public String urlServer;
	
	private Button buttonStart;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        Intent locationService = new Intent(Gymkhana.this, LocationService.class);
        stopService(locationService);
        Intent messagingService = new Intent(this, MessagingService.class);
        this.startService(messagingService);
        MessagingService.MESSAGING_SERVICE_ENABLED = true;
        Intent gymkhanaLocationService = new Intent(this, GymkhanaLocationListener.class);
        this.startService(gymkhanaLocationService);
        
        setContentView(R.layout.gymkhana_index);        
        buttonStart = (Button)this.findViewById(R.id.buttonStart);
        buttonStart.setOnClickListener(this);
    }
    
    public void onClick(View v) {
        if ( buttonStart.isPressed() ){
        	//urlServer = LibreGeoSocial.mUrl.replaceFirst("/social/", "/");
        	urlServer = "http://" + getString(R.string.urlServer).replaceFirst("/social/", "/");
			Proof.proofId = -1;
            Intent i = new Intent(Gymkhana.this, EventList.class);
            this.startActivity(i);
            
            this.setResult(RESULT_OK);
            this.finish();
        }
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        
        menu.add(0, MENU_ABOUT, 0, "About")
			    .setIcon(R.drawable.gymkhana)
		        .setAlphabeticShortcut('A');        
        
        return true;
    }
    
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    		
        case MENU_ABOUT:
        	
        	showDialog();
        	
            break;
        }
        return super.onOptionsItemSelected(item);
    }
    
    private void showDialog(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	builder.setMessage("Gymkhana LibreGeoSocial\n\n" +
    						"Developed by Jorge Fernández González\n\n" +
    						"GSYC/LibreSoft\n\n" + 
    						"Rey Juan Carlos University")
    	       .setCancelable(false)
    	       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle("About");
    	alert.setIcon(R.drawable.gymkhana);
    	alert.show();
    }
    
    @Override
    public void onResume(){
		super.onResume();
    }

}