/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

/**
 * Representa un mensaje interno de la aplicación de la gymkhana y que es intercambiado
 * entre los equipos, manager, etc.
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class Message{
	
	/* Identificador del mensaje. */
	private int identifier;
	/* Fecha del mensaje (String para trabajar más cómodamente en ciertas operaciones). */
	private String date;
	/* El texto, el cuerpo del mensaje. */
	private String text;
	/* El equipo que envió el mensaje, si fue mandado por un equipo. */
	private Team fromTeam;
	/* El equipo (o los equipos) al que se mandó el mensaje (si va dirigido a un equipo). */
	private Team[] toTeams;
	/* El manager (de la gymkhana) que envió el mensaje, si lo mandó un manager. */
	private Manager fromManager;
	/* El manager al que va dirigido el mensaje (si va dirigido a un manager). */
	private Manager toManager;
	
	public Message(int identifier, String date, String text, Manager fromManager, 
			Team fromTeam, Manager toManager, Team[] toTeams) {
		
		super();
		this.identifier = identifier;
		this.date = date;
		this.text = text;
		this.fromManager = fromManager;
		this.fromTeam = fromTeam;
		this.toManager = toManager;
		this.toTeams = toTeams;
	}
	
	public int getIdentifier() {
		return identifier;
	}
	public void setIdentifier(int identifier) {
		this.identifier = identifier;
	}

	public String getDate() {
		return date;
	}
	public void setDate(String date) {
		this.date = date;
	}

	public String getText() {
		return text;
	}
	public void setText(String text) {
		this.text = text;
	}

	public Team getFromTeam() {
		return fromTeam;
	}
	public void setFromTeam(Team fromTeam) {
		this.fromTeam = fromTeam;
	}
	
	public Team[] getToTeams() {
		return toTeams;
	}
	public void setToTeams(Team[] toTeams) {
		this.toTeams = toTeams;
	}

	public Manager getFromManager() {
		return fromManager;
	}
	public void setFromManager(Manager fromManager) {
		this.fromManager = fromManager;
	}

	public Manager getToManager() {
		return toManager;
	}
	public void setToManager(Manager toManager) {
		this.toManager = toManager;
	}

	public static Message findById(Message messages[], int id){
   		Message message = null;
   		for (int i = 0; i < messages.length-1; i++){
   			if (messages[i].getIdentifier() == id)
   				message = messages[i];
   		}
   		return message;
   	}
	
}