/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocialApp.R;

/**
 * Muestra la lista de mensajes recibidos/enviados (según se haya seleccionado) por
 * un determinado equipo participante en la gymkhana. Cuando se seleccione uno de esos
 * mensajes, se pasará a una nueva actividad en la que se mostrará el mensaje de manera
 * detallada (cabeceras de remitentes y destinatarios, cuerpo, opción de responder, etc.).
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class MessageList extends ListActivity{
	/* TAG for log messages*/
	static private String TAG = "MessageList";
	
	/* Message from last petition */
	private String lastMsg = null;
	
	/* Code from last petition */
	private Integer lastCode = 0;
	
	MessagesAdapter mAdapter;
	static private ArrayList<String> mList;
	
	public static Message[] messages;

	private static final int MENU_INFORMATION = Menu.FIRST + 1;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	    
		try{
			messages = null;
			String inboxOrSent = this.getIntent().getStringExtra("inbox_or_sent");
			DefaultHttpClient httpclient = new DefaultHttpClient();
			URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + 
					Integer.toString(EventList.event.getIdentifier()) + "/team/" +
					Integer.toString(TeamList.team.getIdentifier()) + "/message/" + inboxOrSent + "/?format=json");
			HttpGet httget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httget);
			HttpEntity entity = response.getEntity();
			String str = convertStreamToString(entity.getContent());
			processInboxMessageList(str);
			
			// Adapter for ListView
			mAdapter = new MessagesAdapter(this);
			setListAdapter(mAdapter);
		}catch(URISyntaxException urisex){
		    Log.e(TAG, urisex.getMessage());
	    }catch (IOException e) {
            // TODO Auto-generated catch block
            Log.e(TAG,e.getMessage());
	    }
	}
		
	protected void onListItemClick(ListView l, View v, int position, long id) {
		Message selectedMessage = messages[position];
		Message message = Message.findById(messages,selectedMessage.getIdentifier());
		
		String[] messageParts = prepareMessageToShow(message, position);
		
	    Intent i = new Intent(this, MessageViewer.class);
	    i.putExtra("header", messageParts[0]);
	    i.putExtra("messageText", messageParts[1]);
	    i.putExtra("position", position);
	    startActivity(i);
	}
	
	private String convertStreamToString (InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is), 8*1024);
        StringBuilder sb = new StringBuilder();
 
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }
    
    private boolean processInboxMessageList(String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);

			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			
			if (json.has("messages") && lastMsg.equals("ok")){
				JSONArray messagesObjects = json.getJSONArray("messages");
				messages = new Message[messagesObjects.length()];
				mList = new ArrayList<String>();
				for (int i = 0; i < messagesObjects.length()-1; i++){
					JSONObject JSONMessage = (JSONObject)messagesObjects.get(i);
					int id = JSONMessage.getInt("identifier");
					String date = JSONMessage.getString("date");
					int end = date.indexOf('.');
					date = date.substring(0, end);
					String text = JSONMessage.getString("text");
					Manager fromManager = null;
					Team fromTeam = null;
					Manager toManager = null;
					Team[] toTeams = null;
					if (JSONMessage.has("from_manager")){
						JSONArray JSONFromManager = JSONMessage.getJSONArray("from_manager");
						JSONObject JSONManager = (JSONObject)JSONFromManager.get(0);
						fromManager = new Manager(JSONManager.getInt("manager_identifier"),
															JSONManager.getString("manager_name"));
					}
					if (JSONMessage.has("from_team")){
						JSONArray JSONFromTeam = JSONMessage.getJSONArray("from_team");
						JSONObject JSONTeam = (JSONObject)JSONFromTeam.get(0);
						fromTeam = new Team(JSONTeam.getInt("from_team_identifier"),
								JSONTeam.getString("from_team_name"));
					}
					if (JSONMessage.has("to_manager")){
						JSONArray JSONToManager = JSONMessage.getJSONArray("to_manager");
						JSONObject JSONManager = (JSONObject)JSONToManager.get(0);
						toManager = new Manager(JSONManager.getInt("manager_identifier"),
															JSONManager.getString("manager_name"));
					}
					if (JSONMessage.has("to_teams")){
						JSONArray JSONToTeams = JSONMessage.getJSONArray("to_teams");
						toTeams = new Team[JSONToTeams.length()];
						for (int j = 0; j < JSONToTeams.length()-1; j++){
							JSONObject JSONToTeam = (JSONObject)JSONToTeams.get(j);
							toTeams[j] = new Team(JSONToTeam.getInt("to_team_identifier"),
													JSONToTeam.getString("to_team_name"));
						}
					}
					
					messages[i] = new Message(id, date, text, 
											fromManager, fromTeam, toManager, toTeams);
				
					String textToShow = "Message " + Integer.toString(i+1) + "\n";
					textToShow = textToShow + "Date: " + date + "\n";
					textToShow = textToShow + text;
					mList.add(textToShow);
				}
			}

			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;
	}
    
    private String[] prepareMessageToShow(Message message, int position){
    	String header = null;
    	String messageText = null;
    	String[] messageParts = new String[2];
    	
        header = "Inbox Message " + Integer.toString(position+1) + "\n\n";
        header = header + "Date: " + message.getDate() + "\n";
        try{
        	if (!(message.getFromManager()).equals(null)){
        		header = header + "From Manager: " + message.getFromManager().getName();
        	}
        }catch(NullPointerException npe){
        	Log.d(TAG, npe.toString());
        }
        try{
        	if (!(message.getFromTeam()).equals(null)){
        		header = header + "From Team: " + message.getFromTeam().getGroupName();
        	}
	    }catch(NullPointerException npe){
	    	Log.d(TAG, npe.toString());
	    }

	    try{
			Team[] toTeams = message.getToTeams();
			for (int j = 0; j < toTeams.length-1; j++){
				if (j == 0) header = header + "\nTo Team: " + toTeams[j].getGroupName();
				else header = header + ", " + toTeams[j].getGroupName();
			}
	    }catch(NullPointerException npe){
	    	Log.d(TAG,npe.toString());
	    }
		try{	
			Manager toManager = message.getToManager();
			header = header + "\nTo Manager: " + toManager.getName();
		}catch(NullPointerException npe){
	    	Log.d(TAG,npe.toString());
	    }
	    
		header = header + "\n\nText:";
		messageText = message.getText();
		
		messageParts[0] = header;
		messageParts[1] = messageText;
		return messageParts;
    }
	
	public static class MessagesAdapter extends BaseAdapter {

		private Context mContext;
		
		public MessagesAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			//return events.length();
			return mList.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            // Make up a new view
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.message_list, null);
	        } else {
	            // Use convertView if it is available
	            view = convertView;
	        }
			
	        String text = (String)mList.get(position);
	        int end = text.indexOf('\n');
	        String header = text.substring(0, end);
	        text = text.substring(end+1);
	        
	        //ImageView img = (ImageView) view.findViewById(R.id.image);	        
			
			TextView t = (TextView) view.findViewById(R.id.header);
			t.setText(header);
			
			t = (TextView) view.findViewById(R.id.text);
			t.setText(text);
			
			return view;   
		}
	}
	
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);        
        menu.add(0, MENU_INFORMATION, 0 ,"Information")
        .setIcon(R.drawable.info)
        .setAlphabeticShortcut('I');
        
        return true;
    }
	
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	switch (item.getItemId()) {
    		case MENU_INFORMATION:
    			showInformationDialog();
    			break;
    	}
    	return super.onOptionsItemSelected(item);
    }
    
	private void showInformationDialog(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	String text = "En esta pantalla tienes la lista de mensajes que has recibido/enviado según "
    				+ "la opción que seleccionases anteriormente, con un breve resumen de la "
    				+ "información relativa a cada mensaje. Para visualizar el texto íntegro de "
    				+ "un determinado mensaje de la lista, tan sólo tendrás que pulsar sobre él "
    				+ "en la pantalla táctil del teléfono móvil.\n\nRecuerda que si finalmente " 
    				+ "decides no visualizar ningún mensaje en concreto, para volver hacia "
    				+ "atrás sólo tendrás que pulsar la tecla con una flecha indicando hacia "
    				+ "atrás.";
    	builder.setMessage(text)
    	       .setCancelable(false)
    	       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle("Instructions");
    	alert.setIcon(R.drawable.info);
    	alert.show();
    }
	
	// Registro un receptor de mensajes en broadcast porque es la manera de notificar
	// si se ha alcanzado el lugar objetivo de una prueba de geolocalización, por lo
	// que habrá que pasar a la siguiente prueba.
	private BroadcastReceiver receiver = new BroadcastReceiver() {
        
        @Override
        public void onReceive(Context context, Intent intent) {
        	Intent service = new Intent(MessageList.this, DistanceControllerService.class);
		    unregisterReceiver(receiver);
       	 	stopService(service);
       	 	MessageList.this.setResult(RESULT_OK);
       	 	MessageList.this.finish();
        }
    };


    protected void onResume(){
    	super.onResume();
    	if (Proof.lastProofType == 3 && !Proof.targetPlaceFound)
    		registerReceiver(receiver, new IntentFilter(Intent.ACTION_VIEW));
    }


    @Override
    protected void onPause(){
    	super.onPause();       
        //unregisterReceiver(receiver);
    }
	
}