/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.User;
import com.LibreGeoSocialApp.R;

/**
 * Visor de mensajes (muestra en su integridad, junto a las cabeceras con los datos
 * del remitente y destinatarios, un mensaje seleccionado por el usuario).
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class MessageViewer extends Activity implements OnClickListener{
	/* TAG for log messages*/
	static private String TAG = "MessageViewer";
	
	private TextView textHeader;
	private TextView textMessageHeader;
	private TextView textMessageText;
	
	/* Remitente, destinatario, etc. */
	private String header;
	/* Texto o cuerpo del mensaje. */
	private String messageText;
	/* La posición (el mensaje) seleccionada en la lista de mensajes mostrada en la 
	 * actividad que la precede (MessageList) para identificar cuál se debe mostrar
	 * en esta otra actividad actualmente en ejecución. */
	private int position;
	
	private Button buttonReplyMessage;
	
	private static final int MENU_INFORMATION = Menu.FIRST + 1;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		header = null;
		messageText = null;
		position = -1;
		
		Bundle bundle = this.getIntent().getExtras();
        header = bundle.getString("header");
       	messageText = bundle.getString("messageText");
       	position = bundle.getInt("position");
		
		setContentView(R.layout.message_viewer);
		textHeader = (TextView) this.findViewById(R.id.textHeader);
    	User user = LibreGeoSocial.getInstance().getInfoUser(false, null);
    	textHeader.setText(user.getUsername() + " - " + TeamList.team.getGroupName() + " - " + EventList.event.getTitle());
		textMessageHeader = (TextView) this.findViewById(R.id.textMessageHeader);
        textMessageHeader.setText(header);
        textMessageText = (TextView) this.findViewById(R.id.textMessageText);
        textMessageText.setText(messageText);
        buttonReplyMessage = (Button) this.findViewById(R.id.buttonReply);
        buttonReplyMessage.setOnClickListener(this);
	}
	
	@Override
	public void onClick(View v) {
		// TODO Auto-generated method stub
		if (buttonReplyMessage.isPressed()){
			this.setResult(RESULT_OK);
			this.finish();
    		Intent i = new Intent(this, NewMessage.class);
    		i.putExtra("position", position);
    	    startActivity(i);
		}
	}
	
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);        
        menu.add(0, MENU_INFORMATION, 0 ,"Information")
        .setIcon(R.drawable.info)
        .setAlphabeticShortcut('I');
        
        return true;
    }
	
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	switch (item.getItemId()) {
    		case MENU_INFORMATION:
    			showInformationDialog();
    			break;
    	}
    	return super.onOptionsItemSelected(item);
    }
    
	private void showInformationDialog(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	String text = "En esta pantalla tienes toda la información relativa a un "
    				+ "mensaje que seleccionaste para visualizar, ya fuera en la lista de "
    				+ "mensajes recibidos o en la lista de mensajes enviados. Además, tienes "
    				+ "siempre la posibilidad de responder al mensaje pulsando en el botón "
    				+ "dedicado a ello y que te llevará a la pantalla de edición de mensajes."
    				+ "\n\nRecuerda que cuando termines de ver el mensaje y si no deseas "
    				+ "responderlo, para volver hacia atrás sólo tendrás que pulsar la tecla "
    				+ "con una flecha indicando hacia atrás.";
    	builder.setMessage(text)
    	       .setCancelable(false)
    	       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle("Instructions");
    	alert.setIcon(R.drawable.info);
    	alert.show();
    }
	
	// Registro un receptor de mensajes en broadcast porque es la manera de notificar
	// si se ha alcanzado el lugar objetivo de una prueba de geolocalización, por lo
	// que habrá que pasar a la siguiente prueba.
	private BroadcastReceiver receiver = new BroadcastReceiver() {
        
        @Override
        public void onReceive(Context context, Intent intent) {
        	Intent service = new Intent(MessageViewer.this, DistanceControllerService.class);
		    unregisterReceiver(receiver);
       	 	stopService(service);
       	 	MessageViewer.this.setResult(RESULT_OK);
       	 	MessageViewer.this.finish();
        }
    };


    protected void onResume(){
    	super.onResume();
    	if (Proof.lastProofType == 3 && !Proof.targetPlaceFound)
    		registerReceiver(receiver, new IntentFilter(Intent.ACTION_VIEW));
    }


    @Override
    protected void onPause(){
    	super.onPause();       
        //unregisterReceiver(receiver);
    }

}