/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SubMenu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.User;
import com.LibreGeoSocialApp.R;
import com.LibreGeoSocialApp.Routes.LocationService;

/**
 * Presentación del sistema de mensajería interno de la propia aplicación para gymkhanas.
 * Se da opción a escribir un nuevo mensaje y a consultar los mensajes ya sean enviados o
 * recibidos. También se puede configurar la posibilidad de activar un servicio en
 * segundo plano que compruebe periódicamente en segundo plano la recepción de nuevos
 * mensajes, lanzando una notificación en tal caso.
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class Messaging extends Activity implements Runnable, OnClickListener{
	/* TAG for log messages*/
	static private String TAG = "Messaging";
	
	/* Message from last petition */
	private String lastMsg = null;
	
	/* Code from last petition */
	private Integer lastCode = 0;
	
	private int lastNumMessages = 0;
	
	private Button buttonNewMessage;
	private Button buttonInbox;
	private Button buttonSentMessages;
	private TextView textHeader;
	
	private String numMessagesType;
	
	private static final int MENU_SETTINGS = Menu.FIRST + 1;
	private static final int MENU_ENABLE = Menu.FIRST + 2;
	private static final int MENU_DISABLE = Menu.FIRST + 3;
	private static final int MENU_CANCEL = Menu.FIRST + 4;
	private static final int MENU_INFORMATION = Menu.FIRST + 5;
	
    private ProgressDialog progressDialog = null;
    
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.messaging);
        textHeader = (TextView) this.findViewById(R.id.textHeader);
    	User user = LibreGeoSocial.getInstance().getInfoUser(false, null);
    	textHeader.setText(user.getUsername() + " - " + TeamList.team.getGroupName() + " - " + EventList.event.getTitle());
        buttonNewMessage = (Button)this.findViewById(R.id.buttonNewMessage);
    	buttonNewMessage.setOnClickListener(this);
    	buttonInbox = (Button)this.findViewById(R.id.buttonInbox);
    	buttonInbox.setOnClickListener(this);
    	buttonSentMessages = (Button)this.findViewById(R.id.buttonSentMessages);
    	buttonSentMessages.setOnClickListener(this);	
    }

	@Override
	public void onClick(View v) {
		// TODO Auto-generated method stub
		numMessagesType = null;

		if (buttonNewMessage.isPressed()){
			Intent i = new Intent(this,NewMessage.class);
        	this.startActivity(i);
		}else{
			if (buttonInbox.isPressed()){
				numMessagesType = "num_inbox";
			}else if (buttonSentMessages.isPressed()){
				numMessagesType = "num_sent";
			}
			loadMessages();
		}
	}
	
	private void loadMessages(){
    	DialogInterface.OnCancelListener dialogCancel = new DialogInterface.OnCancelListener() {

            public void onCancel(DialogInterface dialog) {

            }
          
        };
    	
        String title = "Please Wait";
        String text = "Loading Messages...";
		
		progressDialog = ProgressDialog.show(this, title, text, true, true, dialogCancel);
        Thread thread = new Thread(this);
        thread.start();
    }
    
	@Override
	public void run() {
		// TODO Auto-generated method stub
    	try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + 
					Integer.toString(EventList.event.getIdentifier()) + "/team/" +
					Integer.toString(TeamList.team.getIdentifier()) + "/message/" + 
					numMessagesType + "/?format=json");
			HttpGet httget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httget);
			HttpEntity entity = response.getEntity();
			String str = convertStreamToString(entity.getContent());
			processResponse(str);
		}catch(URISyntaxException urisex){
		    Log.e(TAG, urisex.getMessage());
	    }catch (IOException e) {
            // TODO Auto-generated catch block
            Log.e(TAG,e.getMessage());
	    }
	    handler.sendEmptyMessage(0);
	}
	
	/** This adapter 
	 */
	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
	    	String text = null;
			if (lastNumMessages > 0){
				Intent i = new Intent(Messaging.this, MessageList.class);
				if (numMessagesType.equals("num_inbox")){
					i.putExtra("inbox_or_sent", "list_inbox");
		    		text = "No Messages In Your Inbox.";
		    	}else{
		    		i.putExtra("inbox_or_sent", "list_sent");
		    		text = "No Sent Messages.";
		    	}
		    	progressDialog.cancel();
				Messaging.this.startActivity(i);
		    }else{
		    	if (numMessagesType.equals("num_inbox")){
		    		text = "No Messages In Your Inbox.";
		    	}else{
		    		text = "No Sent Messages.";
		    	}
		    	
		    	AlertDialog.Builder builder = new AlertDialog.Builder(Messaging.this);
		    	builder.setMessage(text)
		    	       .setCancelable(false)
		    	       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
		    	           public void onClick(DialogInterface dialog, int id) {
		    	        	   dialog.cancel();
		    	           }
		    	       });
		    	AlertDialog alert = builder.create();
		    	alert.setTitle("Information");
		    	alert.setIcon(R.drawable.info);
		    	progressDialog.cancel();
		    	alert.show();
		    }
		}
	};
	
	private String convertStreamToString (InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is), 8*1024);
        StringBuilder sb = new StringBuilder();
 
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }
	
	private boolean processResponse(String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);

			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			
			if (json.has("num_messages"))
				lastNumMessages = json.getInt("num_messages");
			else
				lastNumMessages = -1;

			if (lastCode != 200)
				return false;

		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;
	}
	
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        
        SubMenu sub = menu.addSubMenu (0, MENU_SETTINGS, 0 , "Settings")
		.setIcon(R.drawable.spanner_48);
        //.setAlphabeticShortcut('S');
        
        sub.add(0, MENU_ENABLE, 0, "Enable Auto-Check (Default)");
        sub.add(0, MENU_DISABLE, 0, "Disable Auto-Check");
        sub.add(0, MENU_CANCEL, 0, "Cancel");
        
        menu.add(0, MENU_INFORMATION, 0 ,"Information")
        .setIcon(R.drawable.info)
        .setAlphabeticShortcut('I');
        
        return true;
    }
	
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	switch (item.getItemId()) {
    		case MENU_ENABLE:
    			AlertDialog.Builder builder = new AlertDialog.Builder(this);
		    	builder.setMessage("It Implies a High Battery Consumption. Are You " +
		    			"Sure to Activate This Function?")
		    	       .setCancelable(false)
		    	       .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
		    	           public void onClick(DialogInterface dialog, int id) {
		    	        	   if (!MessagingService.MESSAGING_SERVICE_ENABLED){
		    	        		   Intent messagingService = new Intent(Messaging.this, MessagingService.class);
		    	        		   startService(messagingService);
			    	        	   MessagingService.MESSAGING_SERVICE_ENABLED = true;
		    	        	   }
		    	           }
		    	       })
		    	       .setNegativeButton("No", new DialogInterface.OnClickListener() {
		    	           public void onClick(DialogInterface dialog, int id) {
		    	        	   dialog.cancel();
		    	           }
		    	       });
		    	AlertDialog alert = builder.create();
		    	alert.setTitle("Warning");
		    	alert.setIcon(R.drawable.warning);
		    	alert.show();
        		break;
    		case MENU_DISABLE:
    			AlertDialog.Builder builder2 = new AlertDialog.Builder(this);
		    	builder2.setMessage("Are You Sure You Want to Disable This Function?")
		    	       .setCancelable(false)
		    	       .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
		    	           public void onClick(DialogInterface dialog, int id) {
		    	        	   if (MessagingService.MESSAGING_SERVICE_ENABLED){
		    	        		   Intent messagingService = new Intent(Messaging.this, MessagingService.class);
			    	        	   stopService(messagingService);
			    	        	   MessagingService.MESSAGING_SERVICE_ENABLED = false;
		    	        	   }
		    	           }
		    	       })
		    	       .setNegativeButton("No", new DialogInterface.OnClickListener() {
		    	           public void onClick(DialogInterface dialog, int id) {
		    	        	   dialog.cancel();
		    	           }
		    	       });
		    	AlertDialog alert2 = builder2.create();
		    	alert2.setTitle("Warning");
		    	alert2.setIcon(R.drawable.warning);
		    	alert2.show();
    			break;
    		case MENU_CANCEL:
    			break;
    		case MENU_INFORMATION:
    			showInformationDialog();
    			break;
    	}
    	return super.onOptionsItemSelected(item);
    }
    
    private void showInformationDialog(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	String text = "En esta pantalla podrás seleccionar si deseas enviar un nuevo "
    				+ "mensaje, ver la lista de mensajes recibidos, o la de mensajes "
    				+ "enviados.\n\nAdemás, en la opción del menú destinada a la configuración "
    				+ "de este servicio de mensajería, el usuario podrá activar una "
    				+ "comprobación automática y periódica de la lista de mensajes entrantes, "
    				+ "de modo que el usuario será notificado ante la recepción de mensajes "
    				+ "nuevos en todo momento.\n\n"
    				+ "En la barra de estado (parte superior de la pantalla) te aparecerán las "
    				+ "notificaciones del móvil como por ejemplo la recepción de un nuevo mensaje "
    				+ "de la aplicación para gymkhanas. Para desplegar esas notificaciones sólo "
    				+ "deberás arrastrar hacia abajo la barra de estado y pulsar sobre la "
    				+ "notificación que te interese o borrarlas todas.";
    	builder.setMessage(text)
    	       .setCancelable(false)
    	       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle("Instructions");
    	alert.setIcon(R.drawable.info);
    	alert.show();
    }
    
	// Registro un receptor de mensajes en broadcast porque es la manera de notificar
	// si se ha alcanzado el lugar objetivo de una prueba de geolocalización, por lo
	// que habrá que pasar a la siguiente prueba.
    private BroadcastReceiver receiver = new BroadcastReceiver() {
        
        @Override
        public void onReceive(Context context, Intent intent) {
        	Log.d(TAG," soy Messaging y me ha llegado el mensaje en broadcast");
        	Intent service = new Intent(Messaging.this, DistanceControllerService.class);
       	 	stopService(service);
		    unregisterReceiver(receiver);
		    Competition.FINISH_SW = true;
       	 	Messaging.this.setResult(RESULT_OK);
       	 	Messaging.this.finish();
        }
    };


    protected void onResume(){
    	super.onResume();
    	if (Proof.lastProofType == 3 && !Proof.targetPlaceFound)
    		registerReceiver(receiver, new IntentFilter(Intent.ACTION_VIEW));
    }


    @Override
    protected void onPause(){
    	super.onPause();       
        //unregisterReceiver(receiver);
    }
    
    public boolean onKeyDown(int keyCode, KeyEvent event){
    	// Si tiene actividad padre, quiere decir que se ha arrancado como un
    	// Tab de Competition. Si no tiene padre, ha sido arrancado desde la barra
    	// de estado-notificaciones, y por tanto al dar al botón de la flecha hay
    	// que matar esta actividad para regresar a la pantalla principal de la Gymkhana.
    	try{
    		Activity parentActivity = super.getParent();
    		Log.d(TAG, parentActivity.toString()); // Necesario para que salte la NPE.
    	}catch(NullPointerException npe){
    		Messaging.this.setResult(RESULT_OK);
       	 	Messaging.this.finish();
    		return false;
    	}
		
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            // TECLA ATRAS PULSADA
        	if (!Competition.FINISH_SW){
        		Log.d(TAG, "onDestroy de Competition - 2");
        		AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	    	builder.setMessage("Are you sure you want to exit?")
    		           .setCancelable(false)
    		           .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
    		               public void onClick(DialogInterface dialog, int id) {
			            	   	Intent messagingService = new Intent(Messaging.this, MessagingService.class);
			            	   	Messaging.this.stopService(messagingService);
			       	        	MessagingService.MESSAGING_SERVICE_ENABLED = false;
			       	        	Intent gymkhanaLocationService = new Intent(Messaging.this, GymkhanaLocationListener.class);
			       	        	Messaging.this.stopService(gymkhanaLocationService);
			       	        	Intent locationService = new Intent(Messaging.this, LocationService.class);
			       	        	startService(locationService);
			       	        	Messaging.this.setResult(RESULT_OK);
			       	        	Messaging.this.finish();
    		               }
    		           })
    		           .setNegativeButton("No", new DialogInterface.OnClickListener() {
    		               public void onClick(DialogInterface dialog, int id) {
    		                    dialog.cancel();
    		               }
    		           });
    		    AlertDialog alert = builder.create();
    		    alert.setTitle("Warning");
    		    alert.setIcon(R.drawable.warning);
    		    alert.show();
        	}
        	return true;
        }else{
        	return false;
        }
    }

}