/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Timer;
import java.util.TimerTask;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.view.Gravity;
import android.widget.Toast;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocialApp.R;

/**
 * En caso de estar activado, periódicamente y en segundo plano se comprueba si se ha
 * recibido algún mensaje nuevo. En caso de haberlo recibido, se notifica al usuario de
 * ello por medio de la barra de estado y de una vibración.
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class MessagingService extends Service {
	/* TAG for log messages*/
	static private String TAG = "MessagingService";
	
	private Timer timer;
	
	/* Message from last petition */
	private String lastMsg = null;
	
	/* Code from last petition */
	private Integer lastCode = 0;
	
	private Integer lastNumInboxMessages = 0;
	private int numInboxMessages = 0;
	
	/* Retardo de la inicialización del timer. */
	private static int DELAY = 0; // Delay for 0 seconds
	/* Período del timer en el que se chequea la entrada de nuevos mensajes. */
	private static int PERIOD = 60000; // Se chequea la entrada de mensajes cada minuto.
	
	public static boolean MESSAGING_SERVICE_ENABLED = false;
	
	private TimerTask timerTask = new TimerTask() {
		@Override
		public void run() {
			try{
				DefaultHttpClient httpclient = new DefaultHttpClient();
				URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + 
						Integer.toString(EventList.event.getIdentifier()) + "/team/" +
						Integer.toString(TeamList.team.getIdentifier()) + "/message/num_inbox/?format=json");
				HttpGet httget = new HttpGet(uri);
				LibreGeoSocial LGS = LibreGeoSocial.getInstance();
				httpclient.setCookieStore(LGS.getCookieStore());
				HttpResponse response = httpclient.execute(httget);
				HttpEntity entity = response.getEntity();
				String str = convertStreamToString(entity.getContent());
				processResponse(str);
			}catch(URISyntaxException urisex){
			    Log.e(TAG, urisex.getMessage());
		    }catch (IOException e) {
	            // TODO Auto-generated catch block
	            Log.e(TAG,e.getMessage());
		    }
			
	    	if (lastNumInboxMessages > numInboxMessages){
	    		mHandler.sendEmptyMessage(0);
	    	}
	    	numInboxMessages = lastNumInboxMessages;
		}
	};

	private String convertStreamToString (InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is), 8*1024);
        StringBuilder sb = new StringBuilder();
 
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }
	
	private boolean processResponse(String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);

			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			
			if (json.has("num_messages"))
				lastNumInboxMessages = json.getInt("num_messages");
			else
				lastNumInboxMessages = -1;

			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;
	}
	
	Handler mHandler = new Handler() {
		public void handleMessage(Message msg) {
			obtainMessage(0);

			//Process incoming messages here
			
			NotificationManager mNotificationManager = 
				(NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
			int icon = R.drawable.message;
			CharSequence tickerText = "New Inbox Message";
			long when = (1000000 * System.currentTimeMillis());
			Notification notification = new Notification(icon, tickerText, when);
				
			Context context = getApplicationContext();
			CharSequence contentTitle = "New Inbox Message";
			CharSequence contentText = "New Message Received";
			Intent notificationIntent = new Intent(context, Messaging.class);
			PendingIntent contentIntent = PendingIntent.getActivity(context, 0, notificationIntent, 0); 

			notification.setLatestEventInfo(context, contentTitle, contentText, contentIntent);
			notification.flags = Notification.FLAG_AUTO_CANCEL;
			//notification.defaults |= Notification.DEFAULT_SOUND;
			notification.defaults |= Notification.DEFAULT_VIBRATE;
			//notification.sound = "file:///sdcard/audiofile.mp3"
			notification.vibrate = new long[] { 500, 200 }; //The notification will vibrate for 200ms after 200ms of delay
            
			mNotificationManager.notify(1, notification);
			
			String text = "\n\n\n    NEW INBOX MESSAGE\nSee Status Bar Notification\n\n\n";
		    int duration = Toast.LENGTH_LONG;
		    Toast toast = Toast.makeText(context, text, duration);
		    toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
		    toast.show();
    	}
    };
	
	@Override
	public void onCreate() {
		timer = new Timer("MessagingService");
		timer.scheduleAtFixedRate(timerTask, DELAY, PERIOD);
	}
	
	@Override
	public void onDestroy(){
		stopSelf();
		timerTask.cancel();
	}

	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}
	
}