/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.Vibrator;
import android.util.Log;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.AdapterView.OnItemSelectedListener;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.User;
import com.LibreGeoSocialApp.Home;
import com.LibreGeoSocialApp.R;

/**
 * Pantalla para la creación y envío de un nuevo mensaje. Además de escribir el texto
 * a enviar, se puede elegir entre enviar el mensaje a todo el mundo (todos los equipos
 * y manager de la gymkhana), solamente al manager, o solamente a un determinado equipo.
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class NewMessage extends Activity implements Runnable, OnClickListener/*, OnItemSelectedListener*/{
	/* TAG for log messages*/
	static private String TAG = "NewMessage";
	
	private static String TO_ALL = "All (Teams & Manager)";
	private static String TO_MANAGER = "Manager - Organizer";
	
	private Button buttonSend;
	private TextView textHeader;
	private TextView textFrom;
	private TextView textTo;
    private EditText editMessageText;
    
    private String lastMsg;
    private int lastCode;
    
    /* El destino del mensaje (todos los equipos y manager; sólo manager; sólo un equipo). */
    private String itemSelected = null;
    
    private int position = -1;
    private boolean result = false;
    
    private static final int MENU_INFORMATION = Menu.FIRST + 1;
    
    private ProgressDialog progressDialog = null;
    
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        String text = null;
        Bundle bundle = null;
        com.LibreGeoSocialApp.Gymkhana.Message message = null;
        try{
        	bundle = this.getIntent().getExtras();
        	position = bundle.getInt("position");
        }catch(NullPointerException npe){
        	Log.d(TAG,npe.toString());
        }
        
        setContentView(R.layout.new_message);
        textHeader = (TextView) this.findViewById(R.id.textHeader);
    	User user = LibreGeoSocial.getInstance().getInfoUser(false, null);
    	textHeader.setText(user.getUsername() + " - " + TeamList.team.getGroupName() + " - " + EventList.event.getTitle());
        textFrom = (TextView) this.findViewById(R.id.textFrom);
        textFrom.setText("From Team: " + TeamList.team.getGroupName());
        textTo = (TextView) this.findViewById(R.id.textTo);
        //textTo.setText("To Manager: " + EventList.event.getManager().getName());
        textTo.setText("To: ");
        
        editMessageText = (EditText)this.findViewById(R.id.editMessageText);
        if (position >= 0){
        	message = MessageList.messages[position];
        	try{
        		if ( !(message.getFromManager()).equals(null) ){
        			text = "\n\n\nManager " + message.getFromManager().getName();
        		}
        	}catch(NullPointerException npe){
        		Log.d(TAG,npe.toString());
        	}
        	try{
        		if ( !(message.getFromTeam()).equals(null) ){
        			text = "\n\n\nTeam " + message.getFromTeam().getGroupName();
        		}
        	}catch(NullPointerException npe){
        		Log.d(TAG,npe.toString());
        	}
        	text = text + " wrote (" + message.getDate() + "):\n\n";
			text = text + message.getText();
        }
        editMessageText.setText(text);
		
		Spinner spinner = (Spinner) findViewById(R.id.spinnerTo);
        ArrayList<String> arrayList = new ArrayList<String>(TeamList.teamNames.size()+2);
        arrayList.add(0, TO_ALL);
        arrayList.add(1, TO_MANAGER);
        for (int j = 0; j < TeamList.teamNames.size(); j++){
        	arrayList.add(j+2, TeamList.teamNames.get(j));
        }
        ArrayAdapter<String> spinnerArrayAdapter = new ArrayAdapter<String>(this, android.R.layout.preference_category, arrayList);
        spinner.setAdapter(spinnerArrayAdapter);
        spinner.setOnItemSelectedListener(new MyOnItemSelectedListener());
        
        buttonSend = (Button)this.findViewById(R.id.buttonNewMessage);
    	buttonSend.setOnClickListener(this);
    }
    
	@Override
	public void onClick(View v) {
		// TODO Auto-generated method stub
		if (buttonSend.isPressed()){
			if (editMessageText.getText().toString().trim().equals("")){
	    	    Context context = getApplicationContext();
	    	    String text = "\n\n\nBe Careful! Your Message is Empty.\n\n\n";
			    int duration = Toast.LENGTH_SHORT;
			    Toast toast = Toast.makeText(context, text, duration);
			    toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
			    toast.show();
	    	}
			else{
				operate();
			}
		}
	}
	
	private void operate(){
		DialogInterface.OnCancelListener dialogCancel = new DialogInterface.OnCancelListener() {

            public void onCancel(DialogInterface dialog) {

            }
          
        };
    	
        String title = "Please Wait";
        String text = "Sending Message...";
		
		progressDialog = ProgressDialog.show(this, title, text, true, true, dialogCancel);
        Thread thread = new Thread(this);
        thread.start();
	}
	
	@Override
	public void run() {
		result = sendMessage();
		handler.sendEmptyMessage(0);
	}
	
	/** This adapter 
	 */
	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			String text = null;
			if (result) text = "\n\n\nMessage Has Been Correctly Sent\n\n\n";
			else text = "\n\n\nThe Message Not Could Be Sent.\nAn Error Has Occurred.\n\n\n";
			int duration = Toast.LENGTH_SHORT;
			Context context = getApplicationContext();
		    Toast toast = Toast.makeText(context, text, duration);
		    toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
		    progressDialog.cancel();
		    Vibrator vibrator =	(Vibrator)getSystemService(Context.VIBRATOR_SERVICE);
			vibrator.vibrate(200);
		    toast.show();
		    if (result){
		    	NewMessage.this.setResult(RESULT_OK);
		    	NewMessage.this.finish();
		    }
		}
	};
	
	private boolean sendMessage(){
		result = false;
		String text = editMessageText.getText().toString().trim();
		try{
        	URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + Integer.toString(EventList.event.getIdentifier()) + "/message/create/");
        	DefaultHttpClient httpclient = new DefaultHttpClient();
            HttpPost httpost = new HttpPost(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
            
            List <NameValuePair> nvps = new ArrayList <NameValuePair>();
            nvps.add(new BasicNameValuePair("latitude", new Double(Home.currentLocation.getLatitude()).toString()));
        	nvps.add(new BasicNameValuePair("longitude", new Double(Home.currentLocation.getLongitude()).toString()));
        	nvps.add(new BasicNameValuePair("altitude", new Double(Home.currentLocation.getAltitude()).toString()));
            nvps.add(new BasicNameValuePair("format", "json"));
            nvps.add(new BasicNameValuePair("from_team_id", Integer.toString(TeamList.team.getIdentifier())));
            if (itemSelected.equals(TO_ALL)){
    			nvps.add(new BasicNameValuePair("to", "all"));
    		}else if (itemSelected.equals(TO_MANAGER)){
    			nvps.add(new BasicNameValuePair("to", "manager"));
    		}else{
    			Team team = Team.findByName(TeamList.teams, itemSelected);
    			nvps.add(new BasicNameValuePair("to_team_id", new Integer(team.getIdentifier()).toString()));
    			Log.d(TAG, "to_team_id = " + new Integer(team.getIdentifier()).toString());
    		}
            nvps.add(new BasicNameValuePair("text", text));
            
	        httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));
            HttpResponse response = httpclient.execute(httpost);
            HttpEntity entity = response.getEntity();
            String str = convertStreamToString(entity.getContent());
            result = processResponse(str);
    	}catch(URISyntaxException urisex){
    		result = false;
    		Log.d(TAG, urisex.getMessage());
        }catch (IOException e) {
            // TODO Auto-generated catch block
        	result = false;
            Log.e(TAG,e.getMessage());
        }
        return result;
	}
	
	private String convertStreamToString (InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is), 8*1024);
        StringBuilder sb = new StringBuilder();
 
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }
	
	private boolean processResponse(String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			 
			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;
	}
	
	@Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);        
        menu.add(0, MENU_INFORMATION, 0 ,"Information")
        .setIcon(R.drawable.info)
        .setAlphabeticShortcut('I');
        
        return true;
    }
	
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	switch (item.getItemId()) {
    		case MENU_INFORMATION:
    			showInformationDialog();
    			break;
    	}
    	return super.onOptionsItemSelected(item);
    }
    
	private void showInformationDialog(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	String text = "En esta pantalla podrás escribir un nuevo mensaje y enviarlo al "
    				+ "manager de la gymkhana para que resuelva cualquier duda que puedas "
    				+ "tener.\n\nRecuerda que si finalmente decides no enviar el mensaje, "
    				+ "para volver hacia atrás sólo tendrás que pulsar la tecla con una "
    				+ "flecha indicando hacia atrás.";
    	builder.setMessage(text)
    	       .setCancelable(false)
    	       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle("Instructions");
    	alert.setIcon(R.drawable.info);
    	alert.show();
    }
	
	// Registro un receptor de mensajes en broadcast porque es la manera de notificar
	// si se ha alcanzado el lugar objetivo de una prueba de geolocalización, por lo
	// que habrá que pasar a la siguiente prueba.
	private BroadcastReceiver receiver = new BroadcastReceiver() {
        
        @Override
        public void onReceive(Context context, Intent intent) {
        	Intent service = new Intent(NewMessage.this, DistanceControllerService.class);
		    unregisterReceiver(receiver);
       	 	stopService(service);
       	 	NewMessage.this.setResult(RESULT_OK);
       	 	NewMessage.this.finish();
        }
    };


    protected void onResume(){
    	super.onResume();
    	if (Proof.lastProofType == 3 && !Proof.targetPlaceFound)
    		registerReceiver(receiver, new IntentFilter(Intent.ACTION_VIEW));
    }


    @Override
    protected void onPause(){
    	super.onPause();       
        //unregisterReceiver(receiver);
    }

    private class MyOnItemSelectedListener implements OnItemSelectedListener{
    	@Override
    	public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) {
    		itemSelected = parent.getItemAtPosition(pos).toString(); 
    	}

    	@Override
    	public void onNothingSelected(AdapterView parent) {
    		// Do nothing.
    	}
    }
	
}