/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.Vibrator;
import android.provider.MediaStore;
import android.util.Log;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SubMenu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.Photo;
import com.LibreGeoSocial.Api.User;
import com.LibreGeoSocial.Api.Layers.LibreGeoSocialLayer;
import com.LibreGeoSocialApp.Home;
import com.LibreGeoSocialApp.NodeList;
import com.LibreGeoSocialApp.R;
import com.LibreGeoSocialApp.Routes.LocationService;
import com.LibreGeoSocialApp.Utils.BitmapUtils;
import com.LibreGeoSocialApp.Utils.ClientHttpRequest;

/**
 * Se trata del corazón de la aplicación, donde se presentarán al usuario los retos
 * a superar, donde podrá enviar sus respuestas, saltar a otra prueba o solicitar una
 * pista como ayuda a superar la actual, etc.
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class Proof extends Activity implements Runnable, OnClickListener {
	/* TAG for log messages*/
	static private String TAG = "Proof";
	
	/* Identificador del reto que el equipo está afrontando. */
    public static int proofId = -1;
    /* Enunciado del reto que el equipo está afrontando. */
    public static String challenge = null;
    /* Máxima puntuación del reto que el equipo está afrontando. */
    private int lastMaxScore = -1;
    
    /* Message from last petition */
	private String lastMsg = null;
	
	/* Code from last petition */
	private Integer lastCode = 0;
	
	/* Último reto que se recibió desde el servidor. */
	private String lastChallenge = null;
	/* Último número de prueba recibido desde el servidor. */
	private int lastProofNumber = -1;
	/* Último tipo de prueba que se recibió desde el servidor (1-textual; 2-fotográfica;
	 * 3-geolocalización; ...). */
	public static int lastProofType = -1;
	/* Si su valor es true, indica que se puede saltar de la actual prueba a la siguiente. */
	private boolean lastCanJump = false;
	/* Si su valor es true, indica que hay que usar la realidad aumentada para superar la prueba textual
	 * buscando una etiqueta en la que haya escrita una adivinanza, etc.). */
	private boolean lastAugmentedReality = false;
	
	/* En pruebas de geolocalización, la latitude del lugar objetivo. */
	public static double lastTargetLatitude = 0.0;
	/* En pruebas de geolocalización, la longitude del lugar objetivo. */
	public static double lastTargetLongitude = 0.0;
	/* En pruebas de geolocalización, el radio de distancia en metros que se da como válido. */
	public static double lastMinDistance = 0.0;
	/* En pruebas de geolocalización, si el manager permite o no mostrar el lugar en el mapa. */
	public static boolean lastMarkPlace = false;
	/* En pruebas de geolocalización, si el lugar ha sido alcanzado por el equipo. */
	public static boolean targetPlaceFound = false;
	/* En pruebas de geolocalización, la distancia que separa al equipo del lugar objetivo. */
	public static float distanceDifference = -1;
	
	private boolean lastSolution = false;
	private String lastAction = null;
	public static boolean lastIsLastProof = false;
    
	//private Timer mTimer;
	
	private TextView textHeader;
    private TextView textStatementChallenge;
    private Button buttonSendResponse;
    private Button buttonActivateCamera;
    private Button buttonSetPhoto;
    private Button buttonAugmentedReality;
    private EditText responseToChallenge;
    private TextView textNote;
    
    /* Vector de pruebas que el equipo se ha saltado y a las que puede acceder
     * en cualquier momento. */
    private Vector<JumpedProof> jumpedProofs;
    
    private static final int MENU_INFORMATION = Menu.FIRST + 1;
    private static final int MENU_FINISH = Menu.FIRST + 2;
    private static final int MENU_GIVE_CLUE = Menu.FIRST + 3;
    private static final int MENU_SHOW_CLUES = Menu.FIRST + 4;
    private static final int MENU_JUMP = Menu.FIRST + 5;
    private static final int MENU_JUMPED_PROOFS = Menu.FIRST + 6;
    private static final int MENU_CANCEL_RETRY = Menu.FIRST + 7;
    
    private static final int REQUEST_CAMERA = 1;
	private static final int REQUEST_SELECT_PHOTO = 2;
	static ImageView imgPhoto;
	static String strImagePath = null; 
    
	/* Número de pistas totales que tiene la prueba actual. */
    private int lastNumClues = -1; 
    /* Pistas del reto que el equipo ha comprado. */
    private Vector<Clue> clues = null;
    
    private int progressDialogType = -1;
    private int auxProofId = -1;
    
    private static ProgressDialog progressDialog;
    
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
    	if (proofId == -1)
    		proofId = TeamList.team.getFirstProofId();
    	
    	try{
	   		progressDialog.cancel();
		}catch(NullPointerException npe){
		   	Log.d(TAG, npe.toString());
		}
    	progressDialogType = 0;
	    operate();
    }
    
    private void prepareChallenge(){
    	if (lastProofType == 2){ // Prueba fotográfica
        	setContentView(R.layout.proof_type_2);
        	buttonActivateCamera = (Button)this.findViewById(R.id.buttonActivateCamera);
        	buttonActivateCamera.setOnClickListener(this);
        	buttonSetPhoto = (Button)this.findViewById(R.id.buttonSetPhoto);
        	buttonSetPhoto.setOnClickListener(this);
        	imgPhoto = (ImageView) this.findViewById(R.id.imgPhoto);
        	imgPhoto.setImageResource(R.drawable.camera_38);
        }
        else if (lastProofType == 1){ // Prueba textual
        	if (lastAugmentedReality){
        		setContentView(R.layout.proof_type_1_bis);
        		buttonAugmentedReality = (Button)this.findViewById(R.id.buttonAugmentedReality);
        		buttonAugmentedReality.setOnClickListener(Proof.this);
        	}
        	else
        		setContentView(R.layout.proof_type_1);
        }else if (lastProofType == 3){ // Prueba de geolocalización
        	targetPlaceFound = false;
        	if (lastProofType == 3 && !targetPlaceFound){
        		registerReceiver(receiver, new IntentFilter(Intent.ACTION_VIEW));
        		Log.d(TAG, "proof ha registrado su receiver");
        	}
        	setContentView(R.layout.proof_type_3);
        	if (Proof.lastMarkPlace){
        		textNote = (TextView)this.findViewById(R.id.textNote);
        		textNote.setText("Consult Location-Tab. You will find a place marker.");
        	}
        }
        
    	textHeader = (TextView) this.findViewById(R.id.textHeader);
    	User user = LibreGeoSocial.getInstance().getInfoUser(false, null);
    	textHeader.setText(user.getUsername() + " - " + TeamList.team.getGroupName() + " - " + EventList.event.getTitle());
        textStatementChallenge = (TextView) this.findViewById(R.id.textStatementChallenge);
        textStatementChallenge.setText(lastChallenge);
        TextView textChallenge = (TextView) this.findViewById(R.id.textChallenge);
        textChallenge.setText("Max Score: " + lastMaxScore + " points\n\nChallenge " + lastProofNumber + ":");
        if (lastProofType == 1){
        	responseToChallenge = (EditText)this.findViewById(R.id.editResponse);
        }
        if (lastProofType != 3){
        	buttonSendResponse = (Button)this.findViewById(R.id.buttonSendResponse);
        	buttonSendResponse.setOnClickListener(this);
        }
        if (lastProofType == 3 && !targetPlaceFound){
        	Intent service = new Intent(Proof.this, DistanceControllerService.class);
        	startService(service);
    	}
        
        try{
	   		progressDialog.cancel();
		}catch(NullPointerException npe){
		   	Log.d(TAG, npe.toString());
		}
		
    }
    
    public void onClick(View v) {
    	try{
	    	if (buttonAugmentedReality.isPressed()){
	    		// Arrancamos el módulo AR.
	    		Intent IntentLGS = new Intent(this, new NodeList().getClass() );
	    		LibreGeoSocialLayer layer1 = new LibreGeoSocialLayer("lgs",5.0,"");
	    		IntentLGS.putExtra ("layer",layer1);
	    		startActivity(IntentLGS);
	    	}
    	}catch(NullPointerException npe){
	    	Log.d(TAG,npe.toString());
	    }
    	
    	try{
	    	if (buttonSendResponse.isPressed()){
	    		try{
		    		if (lastProofType == 2 && strImagePath.equals(null)){
		    			Context context = getApplicationContext();
			    	    String text = "\n\n\nBe Careful! Select a Photo To Send and Then Push the Button\n\n\n";
					    int duration = Toast.LENGTH_LONG;
					    Toast toast = Toast.makeText(context, text, duration);
					    toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
					    toast.show();
		    		}
		    		else if (lastProofType == 1 && responseToChallenge.getText().toString().trim().equals("")){
			    	    Context context = getApplicationContext();
			    	    String text = "\n\n\nBe Careful! Write Your Response and Then Push the Button\n\n\n";
					    int duration = Toast.LENGTH_LONG;
					    Toast toast = Toast.makeText(context, text, duration);
					    toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
					    toast.show();
			    	}else{
			    		progressDialogType = 1;
			    	    operate();
			    	}
	    		}catch(NullPointerException npe){
	    			Context context = getApplicationContext();
		    	    String text = "\n\n\nBe Careful! Select a Photo To Send and Then Push the Button\n\n\n";
				    int duration = Toast.LENGTH_LONG;
				    Toast toast = Toast.makeText(context, text, duration);
				    toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
				    toast.show();
	    		}
	    	}
    	}catch(NullPointerException npe){
	    	Log.d(TAG,npe.toString());
	    }
    	
    	try{
	    	if (buttonSetPhoto.isPressed()){
	    		this.doSelectImage();
		    }
    	}catch(NullPointerException npe){
	    	Log.d(TAG,npe.toString());
	    }
    	
    	try{
	    	if (buttonActivateCamera.isPressed()){
	    		this.doCaptureImage();
	    	}
    	}catch(NullPointerException npe){
	    	Log.d(TAG,npe.toString());
	    }
    }
    
    private void operate() {
     	
     	DialogInterface.OnCancelListener dialogCancel = new DialogInterface.OnCancelListener() {

             public void onCancel(DialogInterface dialog) {

             }
           
         };
     	
         String title = "Please Wait";
         String text = null;
		 switch(progressDialogType){
		 	case 0:
		 		text = "Loading Challenge...";
		 		break;
			case 1:
				text = "Sending Response...";
				break;
			case 2:
				text = "Going to Following Unresolved Challenge...";
				break;
			case 3:
				text = "Going to Unresolved Selected Challenge...";
				break;
		 }
		 
		 progressDialog = ProgressDialog.show(this, title, text, true, true, dialogCancel);
		 
         Thread thread = new Thread(this);
         thread.start();
     }
    
    @Override
    public void run() {
    	// TODO Auto-generated method stub
		switch(progressDialogType){
			case 0:
		   		this.getChallenge();
		   		this.getJumpedProofList();
		   		auxProofId = -1;
		   		clues = null;
		   		if (lastNumClues > 0){
		   			clues = new Vector<Clue>(lastNumClues);
		   			getAcquiredClues();
		   		}
		   		break;
			case 1:
				sendResponse();
				break;
			case 2:
				String str = notifyJump();
    			executeJump(str);
				break;
			case 3:
				notifyJump();
				break;
		}
    	handler.sendEmptyMessage(0);
    }
    
    /** This adapter 
	 */
	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {

			if (progressDialogType == 0){
		   		progressDialogType = -1;
				prepareChallenge();
			}
			else if (progressDialogType == 1){
				goNextProof();
			}else if (progressDialogType == 2){
		    
		        String text = null;
		        Context context = getApplicationContext();
		        int duration = Toast.LENGTH_LONG;
		        if (auxProofId == proofId){
		        	text = "\n\n\n     No More Challenges     \n\n\n";
		        	Toast toast = Toast.makeText(context, text, duration);
		            toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
		            progressDialog.cancel();
		        	toast.show();
		        }else{
		        	proofId = auxProofId;
		        	
		        	Intent i = new Intent(Proof.this,Competition.class);
		            Proof.this.startActivity(i);
		            Competition.FINISH_SW = true;
		            Proof.this.setResult(RESULT_OK);
		    		Proof.this.finish();
		        }
			}else if (progressDialogType == 3){
    			Intent i = new Intent(Proof.this, Competition.class);
    	        Proof.this.startActivity(i);
    	        Competition.FINISH_SW = true;
    	        Proof.this.setResult(RESULT_OK);
    			Proof.this.finish();
			}
		}
	};
	
	private void goNextProof(){
	    CharSequence text = null;
		if (lastSolution){
		    if (lastIsLastProof){
				// Correct. Y como es la última prueba: GAME OVER!
				text = "\n\n\n     GAME OVER!     \n\n\n";
		    }else{
		    	// Correct. Next Proof!
		    	text = "\n\n\n     NEXT CHALLENGE!     \n\n\n";
		    }
		}else{
			if (lastAction.equals("Continue")){
				if (lastIsLastProof){
					// Incorrect. Y como es la última prueba: GAME OVER!
					text = "\n\n\n     GAME OVER!     \n\n\n";
				}else{
					// Incorrect. Next Proof!
					text = "\n\n\n     NEXT CHALLENGE!     \n\n\n";						
				}
			}else{
				// Incorrect. You Need a Valid Response to Continue
				text = "\n\n\n   YOU NEED A VALID\nRESPONSE TO CONTINUE\n\n\n";
			}
        }
		Context context = getApplicationContext();
	    int duration = Toast.LENGTH_LONG;
	    Toast toast = Toast.makeText(context, text, duration);
	    toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
	    progressDialog.cancel();
	    Vibrator vibrator =	(Vibrator)getSystemService(Context.VIBRATOR_SERVICE);
		vibrator.vibrate(250);
	    toast.show();
		
		strImagePath = null;
	    if (lastAction.equals("Continue") && !lastIsLastProof && proofId != -1){
	    	Intent i = new Intent(Proof.this, Competition.class);
	        Proof.this.startActivity(i);
	        Competition.FINISH_SW = true;
	        Proof.this.setResult(RESULT_OK);
			Proof.this.finish();
	    }else if (lastAction.equals("Continue") && lastIsLastProof && proofId == -1){
	    	Intent i = new Intent(Proof.this,InformativeText.class);
	    	Proof.this.startActivity(i);
	    	Competition.FINISH_SW = true;
	    	Proof.this.setResult(RESULT_OK);
			Proof.this.finish();
	    }
	}
    
    private void getChallenge(){
        DefaultHttpClient httpclient = new DefaultHttpClient();
        try{
            URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + Integer.toString(EventList.event.getIdentifier()) + "/proof/" + Integer.toString(proofId) + "/get_challenge/?format=json");
            HttpGet httget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
            HttpResponse response = httpclient.execute(httget);
	        HttpEntity entity = response.getEntity();
            String str = convertStreamToString(entity.getContent());
            processRequestChallenge(str);
        }catch(URISyntaxException urisex){
        	Log.e(TAG, urisex.getMessage());
        }catch (IOException e) {
            // TODO Auto-generated catch block
            Log.e(TAG,e.getMessage());
        }
    }
    
    private void sendResponse(){
    	try{
        	URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + Integer.toString(EventList.event.getIdentifier()) + "/proof/" + Integer.toString(proofId) + "/respond_challenge/");
        	DefaultHttpClient httpclient = new DefaultHttpClient();
            HttpPost httpost = new HttpPost(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
            List <NameValuePair> nvps = new ArrayList <NameValuePair>();
            
            if (lastProofType == 2){ // Prueba fotográfica
            	Photo myPhoto = new Photo(0,
            			Home.currentLocation.getLatitude(),
            			Home.currentLocation.getLongitude(),
            			Home.currentLocation.getAltitude(),
            			0.0,
            			null,
            			null,
            			strImagePath,
            			null,
            			null);
            	uploadPhoto(myPhoto);
            }else if (lastProofType == 3){ // Prueba de geolocalización
    		    nvps.add(new BasicNameValuePair("latitude", new Double(Home.currentLocation.getLatitude()).toString()));
    		    nvps.add(new BasicNameValuePair("longitude", new Double(Home.currentLocation.getLongitude()).toString()));
    		    nvps.add(new BasicNameValuePair("altitude", new Double(Home.currentLocation.getAltitude()).toString()));
    		    nvps.add(new BasicNameValuePair("distance_difference", new Float(Proof.distanceDifference).toString()));
            	nvps.add(new BasicNameValuePair("success", "true"));
	            nvps.add(new BasicNameValuePair("format", "json"));
	    	    nvps.add(new BasicNameValuePair("team_id", Integer.toString(TeamList.team.getIdentifier())));
		        httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));
	            HttpResponse response = httpclient.execute(httpost);
	            HttpEntity entity = response.getEntity();
	            String str = convertStreamToString(entity.getContent());
	            processResponseToChallenge(str);
	            Proof.lastProofType = -1;
	            Proof.lastMarkPlace = false;
	            Proof.targetPlaceFound = false;
	            goNextProof();
            }else if (lastProofType == 1){ // Prueba textual
            	String myResponse = responseToChallenge.getText().toString().trim();
            	nvps.add(new BasicNameValuePair("response", myResponse));
            	nvps.add(new BasicNameValuePair("latitude", new Double(Home.currentLocation.getLatitude()).toString()));
    		    nvps.add(new BasicNameValuePair("longitude", new Double(Home.currentLocation.getLongitude()).toString()));
    		    nvps.add(new BasicNameValuePair("altitude", new Double(Home.currentLocation.getAltitude()).toString()));
	            nvps.add(new BasicNameValuePair("format", "json"));
	    	    nvps.add(new BasicNameValuePair("team_id", Integer.toString(TeamList.team.getIdentifier())));
		        httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));
	            HttpResponse response = httpclient.execute(httpost);
	            HttpEntity entity = response.getEntity();
	            String str = convertStreamToString(entity.getContent());
	            processResponseToChallenge(str);
            }
        }catch(URISyntaxException urisex){
    		Log.d(TAG, urisex.getMessage());
        }catch (IOException e) {
            // TODO Auto-generated catch block
            Log.e(TAG,e.getMessage());
        }
    }
    
    private String convertStreamToString (InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is), 8*1024);
        StringBuilder sb = new StringBuilder();
 
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }
    
    private boolean processRequestChallenge (String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			
			if (json.has("challenge"))
				lastChallenge = json.getString("challenge");
			else
				lastChallenge = null;
			
			if (json.has("max_score"))
				lastMaxScore = json.getInt("max_score");
			else
				lastMaxScore = 0;
			
			if (json.has("number"))
				lastProofNumber = json.getInt("number");
			else
				lastProofNumber = 0;
			
			if (json.has("proof_type"))
				lastProofType = json.getInt("proof_type");
			else
				lastProofType = -1;
			
			if (lastProofType == 3){
				if (json.has("target_latitude") && json.has("target_longitude")){
					lastTargetLatitude = json.getDouble("target_latitude");
					lastTargetLongitude = json.getDouble("target_longitude");
					targetPlaceFound = false;
					if (json.has("distance_to_target_place")){
						lastMinDistance = json.getDouble("distance_to_target_place");
					}else{
						lastMinDistance = 10;
					}
					if (json.has("mark_place")){
						lastMarkPlace = json.getBoolean("mark_place");
					}else{
						lastMarkPlace = false;
					}
				}else{
					lastTargetLatitude = 0.0;
					lastTargetLongitude = 0.0;
				}
			}

			if (json.has("augmented_reality"))
				lastAugmentedReality = Boolean.parseBoolean(json.getString("augmented_reality"));
			else
				lastAugmentedReality = false;
			
			if (json.has("can_jump"))
				lastCanJump = Boolean.parseBoolean(json.getString("can_jump"));
			else
				lastCanJump = false;
			
			if (json.has("num_clues"))
				lastNumClues = json.getInt("num_clues");
			else
				lastNumClues = 0;
 
			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;
	}
    
    private boolean processResponseToChallenge (String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;

			if (json.has("is_last_proof")){
				if (json.getString("is_last_proof").equals("True"))
					lastIsLastProof = true;
				else
					lastIsLastProof = false;
			}
			else
				lastIsLastProof = false;
			
			if (json.has("message"))
				lastAction = json.getString("message");
			else
				lastAction = null;
			
			if (json.has("next_proof_id"))
				proofId = json.getInt("next_proof_id");
			else
				proofId = -1;
			
			if (json.getString("solution").equals("CORRECT")){
			    lastSolution = true;
			}else{
				lastSolution = false;
			}
			
			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;		
	}
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        menu.add(0, MENU_INFORMATION, 0 ,"Information")
        .setIcon(R.drawable.info)
        .setAlphabeticShortcut('I');
        menu.add(0, MENU_FINISH, 0 ,"Finish")
        .setIcon(R.drawable.finish)
        .setAlphabeticShortcut('F');
        try{
		    if (this.lastNumClues > 0 && this.lastNumClues > clues.size()){
		    	menu.add(0, MENU_GIVE_CLUE, 0 ,"Give Me a Clue")
		    	.setIcon(R.drawable.buy_clue)
		    	.setAlphabeticShortcut('G');
		    }
        }catch(NullPointerException npe){
        	if (lastNumClues > 0){
	        	menu.add(0, MENU_GIVE_CLUE, 0 ,"Give Me a Clue")
	        	.setIcon(R.drawable.buy_clue)
	        	.setAlphabeticShortcut('G');
        	}
        }
        if (this.lastNumClues > 0){
	        menu.add(0, MENU_SHOW_CLUES, 0 ,"Show Clues")
	        .setIcon(R.drawable.clue)
	        .setAlphabeticShortcut('S');
	    }
        
        if (lastCanJump){
        	menu.add(0, MENU_JUMP, 0 ,"Skip Challenge")
        	.setIcon(R.drawable.jump)
        	.setAlphabeticShortcut('J');
        }
        
        try{
        	if (!jumpedProofs.equals(null) && jumpedProofs.size() > 0){
        		SubMenu sub = menu.addSubMenu (0, MENU_JUMPED_PROOFS, 0 , "Retry Challenge")
        			.setIcon(R.drawable.jumped_proofs);
        		sub.add(0,MENU_CANCEL_RETRY,0,"Cancel");
        		for (int i = 0; i < jumpedProofs.size(); i++){
        			sub.add(0,(MENU_CANCEL_RETRY + i + 1),0,
        					"Challenge " + Integer.toString(((JumpedProof)jumpedProofs.elementAt(i)).getNumber()) + " (" +
        					Integer.toString( ((JumpedProof)jumpedProofs.elementAt(i)).getMaxScore()) + " points)");
        		}
        	}
        }catch(NullPointerException npe){
        	Log.d(TAG,npe.toString());
        }
        return true;
    }
	
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	switch (item.getItemId()) {
    		case MENU_INFORMATION:
    			showInformationDialog();
    			break;
    		case MENU_FINISH:
    			showWarningDialog();
    			break;
    		case MENU_GIVE_CLUE:
    			showConfirmationDialog();
    			break;
    		case MENU_SHOW_CLUES:
    			showCluesDialog();
    			break;
    		case MENU_JUMP:
    			progressDialogType = 2;
    			operate();
    			break;
    		case MENU_JUMPED_PROOFS:
    			break;
    		case MENU_CANCEL_RETRY:
    			break;
    		default:
    			progressDialogType = 3;
    			int position = item.getItemId() - MENU_CANCEL_RETRY - 1;
				proofId = ((JumpedProof)jumpedProofs.elementAt(position)).getIdentifier();
				operate();
				break;
    	}
    	return super.onOptionsItemSelected(item);
    }
    
    private void showConfirmationDialog(){
    	int numAcquiredClues = -1;
    	try{
    		numAcquiredClues = clues.size();
    	}catch(NullPointerException npe){
    		Log.d(TAG, npe.toString());
    	}
	    if (lastNumClues > 0 && lastNumClues > numAcquiredClues){
	    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
	    	builder.setMessage("Clue #1 costs -10% Max Score\n" +
		    					"Clue #2 costs -20% Max Score\n" +
		    					"Clue #3 costs -30% Max Score\n\n" +
		    					"Are you sure you want to buy a clue?")
		           .setCancelable(false)
		           .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
		               public void onClick(DialogInterface dialog, int id) {
		            	   buyClue();
		               }
		           })
		           .setNegativeButton("No", new DialogInterface.OnClickListener() {
		               public void onClick(DialogInterface dialog, int id) {
		                    dialog.cancel();
		               }
		           });
		    AlertDialog alert = builder.create();
		    alert.setTitle("Warning");
		    alert.setIcon(R.drawable.warning);
		    alert.show();
	    }else{
	    	Context context = getApplicationContext();
    	   	String text = "\n\n\n     No More Clues     \n\n\n";
    	   	int duration = Toast.LENGTH_LONG;
    	   	Toast toast = Toast.makeText(context, text, duration);
    	   	toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
    	   	toast.show();
	    }
    }
    
    private void showWarningDialog(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	builder.setMessage("Are you sure you want to finish?")
    	       .setCancelable(false)
    	       .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   Proof.lastIsLastProof = true;
    	        	   Intent i = new Intent(Proof.this,InformativeText.class);
    	        	   Proof.this.startActivity(i);
    	        	   Competition.FINISH_SW = true;
    	        	   Proof.this.setResult(RESULT_OK);
    	        	   Proof.this.finish();
    	           }
    	       })
    	       .setNegativeButton("No", new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	                dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle("Warning");
    	alert.setIcon(R.drawable.warning);
    	alert.show();
    }
    
    private void buyClue(){
    	DefaultHttpClient httpclient = new DefaultHttpClient();
        try{
            URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + Integer.toString(EventList.event.getIdentifier()) + "/proof/" + Integer.toString(proofId) + "/clue/buy/?format=json&team_id=" + TeamList.team.getIdentifier());
            HttpGet httget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
            HttpResponse response = httpclient.execute(httget);
	        HttpEntity entity = response.getEntity();
            String str = convertStreamToString(entity.getContent());
            processRequestClue(str);
        }catch(URISyntaxException urisex){
        	Log.e(TAG, urisex.getMessage());
        }catch (IOException e) {
            // TODO Auto-generated catch block
            Log.e(TAG,e.getMessage());
        }
    }
    
    private boolean processRequestClue (String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			
			Clue newClue = null;
			if (json.has("clue") && (lastMsg.trim().toLowerCase()).equals("ok")){
				JSONObject JSONClue = json.getJSONObject("clue");
				newClue = new Clue(JSONClue.getInt("identifier"), JSONClue.getInt("proof_id"), 
									JSONClue.getInt("number"), JSONClue.getString("help"), 
									JSONClue.getInt("cost"));
				try{
					clues.add(newClue);
				}catch(NullPointerException npe){
					Log.d(TAG, npe.toString());
					clues = new Vector<Clue>(lastNumClues);
					clues.add(newClue);
				}
			}

    	    if ((lastMsg.trim().toLowerCase()).equals("ok")){
    	    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	    	builder.setMessage("Clue #" + newClue.getNumber() + 
    								" (" + newClue.getCost() + 
    								" points):\n\n" + newClue.getHelp())
    	    	       .setCancelable(false)
    	    	       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
    	    	           public void onClick(DialogInterface dialog, int id) {
    	    	        	   dialog.cancel();
    	    	           }
    	    	       });
    	    	AlertDialog alert = builder.create();
    	    	alert.setTitle("Information");
    	    	alert.setIcon(R.drawable.info);
    	    	alert.show();
    	    }
        	else{
        		Context context = getApplicationContext();
    	    	String text = "\n\n\n" + lastMsg.trim() + "\n\n\n";
    	    	int duration = Toast.LENGTH_LONG;
    	    	Toast toast = Toast.makeText(context, text, duration);
    	    	toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
    	    	toast.show();
        	}
 
			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;
	}
    
    private void getAcquiredClues(){
    	DefaultHttpClient httpclient = new DefaultHttpClient();
        try{
            URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + Integer.toString(EventList.event.getIdentifier()) + "/proof/" + Integer.toString(proofId) + "/clue/show/?format=json&team_id=" + TeamList.team.getIdentifier());
            HttpGet httget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
            HttpResponse response = httpclient.execute(httget);
	        HttpEntity entity = response.getEntity();
            String str = convertStreamToString(entity.getContent());
        	processProofClues(str);
        }catch(URISyntaxException urisex){
        	Log.e(TAG, urisex.getMessage());
        }catch (IOException e) {
            // TODO Auto-generated catch block
            Log.e(TAG,e.getMessage());
        }
    }
    
    private void showCluesDialog(){
        String text = new String("");
        try{
        	if (!clues.equals(null)){
        		for (int i = 0; i < clues.size(); i++){
	    			Clue currentClue = (Clue)clues.elementAt(i);
	    			text = text.concat("\nClue #" + currentClue.getNumber() + " (" + 
	    					currentClue.getCost() + " points):\n" +
	    					currentClue.getHelp() + "\n");
	    		}
	    	}else{
	    		text = lastMsg.trim();
	    	}
        }catch(NullPointerException npe){
        	text = "You Have Not Got a Clue.";
        	Log.d(TAG,npe.toString());
        }
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	builder.setMessage(text)
    	       .setCancelable(false)
    	       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle("Information");
    	alert.setIcon(R.drawable.info);
    	alert.show();
    }
    
    private boolean processProofClues(String str){
    	try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			
			if (json.has("clues") && lastMsg.trim().toLowerCase().equals("ok")){
				JSONArray cluesObjects = json.getJSONArray("clues");
				for (int i = 0; i < cluesObjects.length()-1; i++){
					JSONObject JSONClue = (JSONObject)cluesObjects.get(i);
					int clueIdentifier = JSONClue.getInt("identifier");
					int clueProofIdentifier = JSONClue.getInt("proof_id");
					int clueNumber = JSONClue.getInt("number");
					String clueHelp = JSONClue.getString("help");
					int clueCost = JSONClue.getInt("cost");
					clues.add(i, new Clue(clueIdentifier, clueProofIdentifier, 
											clueNumber,	clueHelp, clueCost));
				}
			}else{
				clues = null;
			}
 
			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;	
    }
    
    private void showInformationDialog(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	String text = "En esta pantalla puedes encontrar el nuevo reto que se "
    				+ "te plantea (incluyendo la puntuación máxima que podrás obtener "
    				+ "por superarlo correctamente), y el cuadro de texto en el que "
    				+ "deberás introducir tu respuesta (en caso de tratarse de una prueba "
    				+ "con respuesta de tipo textual).\n\nPara ello utiliza el teclado "
    				+ "QWERTY del propio móvil en caso de que tu terminal cuente "
    				+ "con uno, o con el propio teclado virtual que se desplegará "
    				+ "en la pantalla táctil cuando pulses sobre dicho cuadro de "
    				+ "texto.\n\nUna vez introducida la respuesta, sólo deberás pulsar "
    				+ "sobre el botón de envío.\n\nDe manera adicional, pulsando en "
    				+ "el botón \"MENU\" del teléfono, te aparecerán en pantalla diversas "
    				+ "opciones que te podrán servir de ayuda para la superación "
    				+ "de la prueba. Por ejemplo, saltar una prueba si se permite o "
    				+ "la adquisición de pistas para la resolución del reto a cambio "
    				+ "de puntos, así como consultar en cualquier momento las pistas "
    				+ "ya compradas.";
    	builder.setMessage(text)
    	       .setCancelable(false)
    	       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle("Instructions");
    	alert.setIcon(R.drawable.info);
    	alert.show();
    }
    
    private String notifyJump(){
    	String str = null;
    	
    	DefaultHttpClient httpclient = new DefaultHttpClient();
        try{
            URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + Integer.toString(EventList.event.getIdentifier()) + "/proof/" + Integer.toString(proofId) + "/jump_proof/?format=json&team_id=" + TeamList.team.getIdentifier());
            HttpGet httget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
            HttpResponse response = httpclient.execute(httget);
	        HttpEntity entity = response.getEntity();
            str = convertStreamToString(entity.getContent());
        }catch(URISyntaxException urisex){
        	Log.e(TAG, urisex.getMessage());
        }catch (IOException e) {
            // TODO Auto-generated catch block
            Log.e(TAG,e.getMessage());
        }
        return str;
    }
    
    private void executeJump(String str){
    	auxProofId = 0;
    	
    	try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;

			if (json.has("is_last_proof"))
				if (json.getString("is_last_proof").equals("True"))
					lastIsLastProof = true;
				else
					lastIsLastProof = false;
			else
				lastIsLastProof = false;
			
			if (json.has("message"))
				lastAction = json.getString("message");
			else
				lastAction = null;
			
			if (json.has("next_proof_id"))
				auxProofId = json.getInt("next_proof_id");
			else
				auxProofId = -1;
			
        }catch(JSONException e){
        	lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
        }
    }
    
    private boolean processJumpedProofListResponse(String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);

			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			
			if (json.has("jumped_proofs") && lastMsg.equals("ok")){
				JSONArray jumpedProofsObjects = json.getJSONArray("jumped_proofs");
				jumpedProofs = new Vector<JumpedProof>();
				for (int i = 0; i < jumpedProofsObjects.length()-1; i++){
					JSONObject JSONJumpedProof = (JSONObject)jumpedProofsObjects.get(i);
					int proofIdentifier = JSONJumpedProof.getInt("proof_id");
					// Si se trata de la propia prueba en la que me encuentro, la quito.
					if (proofIdentifier != proofId){
						int maxScore = JSONJumpedProof.getInt("max_score");
						int number = JSONJumpedProof.getInt("number");
						jumpedProofs.add(new JumpedProof(proofIdentifier,number,maxScore/*,challenge*/));
					}
				}
			}

			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;
	}
    
    private void getJumpedProofList(){
    	try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + Integer.toString(EventList.event.getIdentifier()) + "/team/" + TeamList.team.getIdentifier() + "/jumped_proof/list/?format=json");
			HttpGet httget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httget);
			HttpEntity entity = response.getEntity();
			String str = convertStreamToString(entity.getContent());
			processJumpedProofListResponse(str);
		}catch(URISyntaxException urisex){
		    Log.e(TAG, urisex.getMessage());
	    }catch (IOException e) {
            // TODO Auto-generated catch block
            Log.e(TAG,e.getMessage());
	    }
    }
    
    private class JumpedProof{
    	private int identifier;
    	private int number;
    	private int maxScore;
    	
    	public JumpedProof(int identifier, int number, int maxScore){
    		this.identifier = identifier;
    		this.number = number;
    		this.maxScore = maxScore;
    	}
    	
    	public int getIdentifier(){
    		return identifier;
    	}
    	public void setIdentifier(int identifier){
    		this.identifier = identifier;
    	}
    	public int getNumber(){
    		return number;
    	}
    	public void setNumber(int number){
    		this.number = number;
    	}
    	public int getMaxScore(){
    		return maxScore;
    	}
    	public void setMaxScore(int maxScore){
    		this.maxScore = maxScore;
    	}
    }
    
    private boolean uploadPhoto(Photo photo){
    	try{
    		URL url = new URL(Gymkhana.urlServer + "gymkhana/event/" + Integer.toString(EventList.event.getIdentifier()) + "/proof/" + Integer.toString(proofId) + "/respond_challenge/");
    		URLConnection urlConnection = url.openConnection();
    		ClientHttpRequest myclient = new ClientHttpRequest(urlConnection);
    		InputStream serverInput = myclient.post ( new Object[] {
    									"latitude", photo.getLatitude().toString(),
    									"longitude", photo.getLongitude().toString(),
    									"altitude", photo.getAltitude().toString(),
    									"team_id", TeamList.team.getIdentifier(),
    									"format", "json",
    									"name", new File(photo.getUrl()).getName(),
					                    "description" , new File(photo.getUrl()).getName(),
    									"filename", new File(photo.getUrl()).getName(),
    									"photo", new File(photo.getUrl()),
										});

			BufferedReader in = new BufferedReader(new InputStreamReader(serverInput));
			
			String inputLine;			
			String responseString = "";

			while ((inputLine = in.readLine()) != null) {
				Log.e(TAG,"EPPP:" + inputLine);
				responseString = responseString + inputLine;
			}
	        return processResponseToChallenge(responseString);
		}
		catch (IOException e) {
	        Log.e(TAG, e.toString());
	        return false;
		}
    }
    
    private void doCaptureImage(){
    	Intent i = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);    	
    	i.putExtra(MediaStore.EXTRA_OUTPUT, MediaStore.Images.Media.EXTERNAL_CONTENT_URI.toString()); 
    	startActivityForResult(i, REQUEST_CAMERA);
    }
    
    private void doSelectImage(){
    	Intent i = new Intent(Intent.ACTION_PICK);
    	i.setDataAndType(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, 
    			MediaStore.Images.Media.CONTENT_TYPE);
    	startActivityForResult(i,this.REQUEST_SELECT_PHOTO);
    }
    
    /*public boolean hasImageCaptureBug() {

        // list of known devices that have the bug
        ArrayList<String> devices = new ArrayList<String>();
        devices.add("android-devphone1/dream_devphone/dream");
        devices.add("generic/sdk/generic");
        devices.add("vodafone/vfpioneer/sapphire");
        devices.add("tmobile/kila/dream");
        devices.add("verizon/voles/sholes");
        devices.add("google_ion/google_ion/sapphire");

        return devices.contains(android.os.Build.BRAND + "/" + android.os.Build.PRODUCT + "/"
                + android.os.Build.DEVICE);

    }*/
    
    @Override
    protected void onActivityResult (int requestCode, int resultCode, Intent data) {  
		//super.onActivityResult(requestCode, resultCode, data);
		if (resultCode == RESULT_CANCELED) {
			Log.d(TAG,"Activity cancelled");
			return;
		}
		
		switch (requestCode) {		
		case REQUEST_CAMERA:
			Bitmap bm = (Bitmap) data.getExtras().get("data");
			//mImageView.setImageBitmap(bm); // Display image in the View
			
			Bundle b = data.getExtras();
			Log.d(TAG, new Boolean(b.containsKey(MediaStore.EXTRA_OUTPUT)).toString());
			if (!b.equals(null) && b.containsKey(MediaStore.EXTRA_OUTPUT)) { // large image?
				//showToast(this,"Large image");
				Log.d(TAG,"Large image");
			    // Shouldn't have to do this ... but
				MediaStore.Images.Media.insertImage(getContentResolver(), bm, null, null);
			} else {
				//showToast(this,"Small image");
				Log.d(TAG,"Small image");
				MediaStore.Images.Media.insertImage(getContentResolver(), bm, null, null);
			}
			break;
    		
	    	case REQUEST_SELECT_PHOTO:
	    		
	    		if( resultCode != 0 ) {
	    			
	    			Log.d(TAG,"data.getData() = " + data.getData());
		    		Cursor c = managedQuery(data.getData(),null,null,null,null) ;
		    		
		    		if( c.moveToFirst() ) {
		    		
		    			String media_path = c.getString(1) ;
		    			
		    			Log.d(TAG,"onActivityResult("+requestCode+","+resultCode+","+media_path+")") ;
		    					    			
		    			strImagePath = media_path;
		    			
		    			Bitmap photo = BitmapUtils.loadBitMapFromFile(strImagePath);
		    			
		    			imgPhoto.setImageBitmap(photo);
		    		}		    		
	    		} 
	    		
	    		break;
	
            default:
                break; 
		}
	}
    
	// Registro un receptor de mensajes en broadcast porque es la manera de notificar
	// si se ha alcanzado el lugar objetivo de una prueba de geolocalización, por lo
	// que habrá que pasar a la siguiente prueba.
    private BroadcastReceiver receiver = new BroadcastReceiver() {
        
        @Override
        public void onReceive(Context context, Intent intent) {
        	Log.d(TAG," soy Proof y me ha llegado el mensaje en broadcast");
        	if (targetPlaceFound && lastProofType == 3){
		        Intent service = new Intent(Proof.this, DistanceControllerService.class);
		        unregisterReceiver(receiver);
		        stopService(service);
		        sendResponse();
	        }
        }
    };


    protected void onResume(){
    	super.onResume();
    	
    	if (lastProofType == 3 && !targetPlaceFound){
    		registerReceiver(receiver, new IntentFilter(Intent.ACTION_VIEW));
    		Log.d(TAG, "proof ha registrado su receiver");
    	}
    }


    @Override
    protected void onPause(){
        super.onPause();       
        //unregisterReceiver(receiver);
    }
    
    @Override
    protected void onDestroy(){
        super.onDestroy();
        //unregisterReceiver(receiver);
        //this.progressDialog = null;
    }
    
    public boolean onKeyDown(int keyCode, KeyEvent event){
		if (keyCode == KeyEvent.KEYCODE_BACK) {
	        // TECLA ATRAS PULSADA
	    	if (!Competition.FINISH_SW){
	    		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		    	builder.setMessage("Are you sure you want to exit?")
			           .setCancelable(false)
			           .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
			               public void onClick(DialogInterface dialog, int id) {
			            	   	Intent messagingService = new Intent(Proof.this, MessagingService.class);
			            	   	Proof.this.stopService(messagingService);
			       	        	MessagingService.MESSAGING_SERVICE_ENABLED = false;
			       	        	Intent gymkhanaLocationService = new Intent(Proof.this, GymkhanaLocationListener.class);
			       	        	Proof.this.stopService(gymkhanaLocationService);
			       	        	Intent locationService = new Intent(Proof.this, LocationService.class);
			       	        	startService(locationService);
			       	        	Proof.this.setResult(RESULT_OK);
			       	        	Proof.this.finish();
			               }
			           })
			           .setNegativeButton("No", new DialogInterface.OnClickListener() {
			               public void onClick(DialogInterface dialog, int id) {
			                    dialog.cancel();
			               }
			           });
			    AlertDialog alert = builder.create();
			    alert.setTitle("Warning");
			    alert.setIcon(R.drawable.warning);
			    alert.show();
	    	}
	    	return true;
	    }else{
	    	return false;
	    }
	}
    
}