/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

import com.LibreGeoSocial.Api.Group;

/** 
 * Representa a cada equipo participante en una gymkhana,
 * heredando de un Group de LibreGeoSocialApp.
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class Team extends Group{
	
	/* Identificador del equipo en LibreGeoSocial. */
	private int identifier;
	/* Identificador del primer reto de la gymkhana para este equipo. */
  	private int firstProofId;
  	/* Número de respuestas que el equipo ha dado correctamente. */
  	private int numCorrectResponses;
  	/* Número de respuestas que el equipo ha dado incorrectamente. */
  	private int numIncorrectResponses;
    
   	public Team(int identifier, String name, int firstProofId, 
   				int numCorrectResponses, int numIncorrectResponses,
   				int idGroup, Double latitude, Double longitude, 
   				Double altitude, Double radius, String since){
   		
   		super(idGroup, latitude, longitude, altitude, radius, since);
   		super.setGroupName(name);
   		this.identifier = identifier;
   		this.firstProofId = firstProofId;
   		this.numCorrectResponses = numCorrectResponses;
   		this.numIncorrectResponses = numIncorrectResponses;
   	}
   	
   	public Team(int identifier, String name) {
   		super(0, 0.0, 0.0, 0.0, 0.0, null);
   		super.setGroupName(name);
		this.identifier = identifier;
	}

	public int getIdentifier(){
   		return identifier;
   	}
   	
   	public int getFirstProofId(){
   		return firstProofId;
   	}
   	
   	public void setFirstProofId(int firstProofId) {
		this.firstProofId = firstProofId;
	}

	public int getNumCorrectResponses(){
   		return numCorrectResponses;
   	}
   	
   	public int getNumIncorrectResponses(){
   		return numIncorrectResponses;
   	}
   	
   	public static Team findById(Team teams[], int id){
   		Team team = null;
   		for (int i = 0; i < teams.length-1; i++){
   			if (teams[i].getIdentifier() == id)
   				team = teams[i];
   		}
   		return team;
   	}
   	
   	public static Team findByName(Team teams[], String name){
   		Team team = null;
   		for (int i = 0; i < teams.length-1; i++){
   			if ((teams[i].getGroupName()).equals(name))
   				team = teams[i];
   		}
   		return team;
   	}
   	   	
}