/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocialApp.R;

/**
 * Muestra al usuario la lista de equipos a los que se puede suscribir de entre todos
 * los disponibles en la gymkhana que seleccionó para participar en la pantalla anterior
 * (EventList).
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class TeamList extends ListActivity implements Runnable{
	/* TAG for log messages*/
	static private String TAG = "TeamList";
	
	/* Equipos disponibles para participar en la gymkhana. */
	public static Team[] teams;
	/* Nombres de los equipos disponibles para participar en la gymkhana. */
	public static ArrayList<String> teamNames;
	
	/* Message from last petition */
	private String lastMsg = null;
	
	/* Code from last petition */
	private Integer lastCode = 0;
	
	TeamsAdapter mAdapter;
	static private ArrayList<String> mList;
	
	private Integer lastTeamId = 0;
	private Integer lastFirstProofId = 0;
	
	/* Equipo en el que se está participando en la gymkhana. */
	public static Team team = null;
	
	private ProgressDialog progressDialog = null;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
	    
		try{
			DefaultHttpClient httpclient = new DefaultHttpClient();
			URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + Integer.toString(EventList.event.getIdentifier()) + "/team/list/?format=json");
			HttpGet httget = new HttpGet(uri);
			LibreGeoSocial LGS = LibreGeoSocial.getInstance();
			httpclient.setCookieStore(LGS.getCookieStore());
			HttpResponse response = httpclient.execute(httget);
			HttpEntity entity = response.getEntity();
			String str = convertStreamToString(entity.getContent());
			processTeamListResponse(str);
		}catch(URISyntaxException urisex){
		    Log.e(TAG, urisex.getMessage());
	    }catch (IOException e) {
            // TODO Auto-generated catch block
            Log.e(TAG,e.getMessage());
	    }
		if (lastMsg.equals("ok") && teams.length > 0){
			try{
				CharSequence text = "\n\n\n     Select Your Team     \n\n\n";
				Context context = getApplicationContext();
			    int duration = Toast.LENGTH_SHORT;
			    Toast toast = Toast.makeText(context, text, duration);
			    toast.setGravity(Gravity.CENTER_VERTICAL, 0, 0);
			    toast.show();
			    
				// Adapter for ListView
				mAdapter = new TeamsAdapter(this);
				setListAdapter(mAdapter);
			}catch(Exception e){
				Log.e(TAG,e.toString());
			}
		}else{
			InformativeText.exit = true;
			InformativeText.reasonToExit = "      Sorry: Actually There    \n" +
										   "Are No Teams Ready To Play ";
			Intent i = new Intent(this, InformativeText.class);
		    startActivity(i);
		    this.setResult(RESULT_OK);
	        this.finish();
		}
	}
		
	protected void onListItemClick(ListView l, View v, int position, long id) {
		Team selectedTeam = teams[position];
		team = Team.findById(teams,selectedTeam.getIdentifier());
		operate();
	}
	
	private void operate(){
		DialogInterface.OnCancelListener dialogCancel = new DialogInterface.OnCancelListener() {

            public void onCancel(DialogInterface dialog) {

            }
          
        };
    	
        String title = "Please Wait";
        String text = "Loading...";
		
		progressDialog = ProgressDialog.show(this, title, text, true, true, dialogCancel);
        Thread thread = new Thread(this);
        thread.start();
	}
	
	@Override
	public void run(){
		try {
			String params = "?teamname=" + team.getGroupName() + "&format=json";
			DefaultHttpClient httpclient = new DefaultHttpClient();
			httpclient.setCookieStore(LibreGeoSocial.getInstance().getCookieStore());
			URI uri = new URI(Gymkhana.urlServer + "gymkhana/event/" + Integer.toString(EventList.event.getIdentifier()) + "/team/join/" + params);
			HttpGet httpget = new HttpGet(uri);
			HttpResponse response = httpclient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String str = convertStreamToString(entity.getContent());
			processJoinResponse(str);
		}catch(URISyntaxException urisex){
			Log.e(TAG, urisex.getMessage());
		}catch (IOException e) {
			// TODO Auto-generated catch block
			Log.e(TAG,e.getMessage());
		}
		
		if (this.lastFirstProofId != team.getFirstProofId()){
			team.setFirstProofId(this.lastFirstProofId.intValue());
		}
		handler.sendEmptyMessage(0);
	}
	
	/** This adapter 
	 */
	private Handler handler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
		    Intent i = new Intent(TeamList.this, InformativeText.class);
		    progressDialog.cancel();
		    startActivity(i);
		    TeamList.this.setResult(RESULT_OK);
	        TeamList.this.finish();
		}
	};
	
	private String convertStreamToString (InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is), 8*1024);
        StringBuilder sb = new StringBuilder();
 
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }
    
    private boolean processTeamListResponse (String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);

			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			
			if (json.has("teams") && lastMsg.equals("ok")){
				JSONArray teamsObjects = json.getJSONArray("teams");
				teams = new Team[teamsObjects.length()];
				teamNames = new ArrayList<String>(teamsObjects.length());
				mList = new ArrayList<String>();
				for (int i = 0; i < teamsObjects.length()-1; i++){
					JSONObject JSONTeam = (JSONObject)teamsObjects.get(i);
					String teamName = JSONTeam.getString("team_name");
					int teamIdentifier = JSONTeam.getInt("identifier");
					int teamGroupId = JSONTeam.getInt("group_id");
					int teamFirstProofId = JSONTeam.getInt("id_first_proof");
					int teamNumCorrectResponses = JSONTeam.getInt("num_correct_responses");
					int teamNumIncorrectResponses = JSONTeam.getInt("num_incorrect_responses");
					double teamLatitude = JSONTeam.getDouble("latitude");
					double teamLongitude = JSONTeam.getDouble("longitude");
					double teamAltitude = JSONTeam.getDouble("altitude");
					double teamRadius = JSONTeam.getDouble("radius");
					String teamSince = JSONTeam.getString("since");
					teams[i] = new Team(teamIdentifier, teamName, teamFirstProofId, teamGroupId,
								teamNumCorrectResponses, teamNumIncorrectResponses,
								teamLatitude, teamLongitude, teamAltitude, teamRadius, teamSince);
					teamNames.add(i, teams[i].getGroupName());
					mList.add(teamNames.get(i));
				}
			}

			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;
	}
    
    private boolean processJoinResponse(String str) {
        try {
        	JSONObject json;
			json = new JSONObject(str);
			
			if (json.has("code"))
				lastCode = json.getInt("code");
			else
				lastCode = -1;
			
			if (json.has("description"))
				lastMsg = json.getString("description");
			else
				lastMsg = null;
			
			if (json.has("team_id"))
				lastTeamId = json.getInt("team_id");
			else
				lastTeamId = -1;
			
			if (json.has("first_proof_id")){
				lastFirstProofId = json.getInt("first_proof_id");
			}else{
				lastFirstProofId = -1;
			}

			if (lastCode != 200)
				return false;
			
		} catch (JSONException e) {	
			lastCode = 500;
			lastMsg = "Error in json parsed: " + e.getMessage();
			Log.e(TAG, e.toString());
			return false;
		}
		return true;
	}
    
    public static class TeamsAdapter extends BaseAdapter {

		private Context mContext;
		
		public TeamsAdapter (Context c) {
            mContext = c;
        }

		public int getCount() {
			// TODO Auto-generated method stub
			//return events.length();
			return mList.size();
		}

		public Object getItem(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public long getItemId(int position) {
			// TODO Auto-generated method stub
			return position;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			// TODO Auto-generated method stub
			View view;
			
	        if (convertView == null) {
	            // Make up a new view
	            LayoutInflater inflater = (LayoutInflater) mContext
	                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
	            view = inflater.inflate(R.layout.team_list, null);
	        } else {
	            // Use convertView if it is available
	            view = convertView;
	        }
			
	        String name = (String)mList.get(position);
	        
	        //ImageView img = (ImageView) view.findViewById(R.id.image);	        
			
			TextView t = (TextView) view.findViewById(R.id.title);
			t.setText( name );
			
			t = (TextView) view.findViewById(R.id.owner);
			
			return view;   
		}
	}
	
}