/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 *
 */

package com.LibreGeoSocialApp.Gymkhana;

import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.LinearLayout;
import android.widget.Toast;
import android.widget.ZoomControls;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.User;
import com.LibreGeoSocialApp.Home;
import com.LibreGeoSocialApp.R;
import com.LibreGeoSocialApp.Routes.LocationService;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayItem;

/**
 * Muestra al usuario en un GoogleMaps la posición del equipo, así como la localización
 * del lugar al que se debe dirigir si se encuentra en una prueba de geolocalización
 * (y si el manager permite que ésta sea mostrada).
 * 
 * @author Jorge Fernandez Gonzalez <jorge.fernandez.gonzalez@gmail.com>
 * 
 */
public class TeamLocationMap extends MapActivity {
	/* TAG for log messages*/
	static private String TAG = "TeamLocationMap";
	
	private MapView mapView;
	
	private List<Overlay> mapOverlays;
	private MapLayer itemizedOverlay;
	private MapLayer itemizedOverlay2;
	
	private Timer mTimer;
	
	private static final int MENU_INFORMATION = Menu.FIRST + 1;
	
	/* Retardo de la inicialización del timer en el que se actualiza la posición en el mapa. */
	private static int DELAY = 0; // Delay for 0 seconds.
	/* Período del timer en el que se actualiza la posición en el mapa. */
	private static int PERIOD = 5000; // Repeat every 10 seconds.
	
	private float lastDistanceToPlace = -1;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.team_location);
        
        LinearLayout linearLayout = (LinearLayout) findViewById(R.id.zoomview);
        mapView = (MapView) findViewById(R.id.mapview);
        ZoomControls mZoom = (ZoomControls) mapView.getZoomControls();
        
        linearLayout.addView(mZoom);
        
        mapView.setSatellite(true);
        
        mapOverlays = mapView.getOverlays();
        Drawable drawable = this.getResources().getDrawable(R.drawable.user_48);
        itemizedOverlay = new MapLayer(this,drawable);
        
        double lat = Home.currentLocation.getLatitude();
    	double lng = Home.currentLocation.getLongitude();
        
        GeoPoint geoPoint = new GeoPoint((int) (lat * 1E6), (int) (lng * 1E6));
        
        MapController mapController = mapView.getController();
        mapController.animateTo(geoPoint);
        mapController.setZoom(17);
        mapView.invalidate();
        
        if (Proof.lastProofType == 3){
			if (Proof.lastMarkPlace){
				if (mapOverlays.contains(itemizedOverlay2)){
					mapOverlays.remove(itemizedOverlay2);
					itemizedOverlay2.clearOverlay();
				}
				Drawable drawable2 = TeamLocationMap.this.getResources().getDrawable(R.drawable.map_pin);
		        itemizedOverlay2 = new MapLayer(TeamLocationMap.this,drawable2);
				double lat2 = Proof.lastTargetLatitude;
				double lng2 = Proof.lastTargetLongitude;
				GeoPoint geoPoint2 = new GeoPoint((int) (lat2 * 1E6), (int) (lng2 * 1E6));
				OverlayItem overlayitem2 = new OverlayItem(geoPoint2, "Place", "Place");
				itemizedOverlay2.addOverlay(overlayitem2);
				mapOverlays.add(itemizedOverlay2);
			}
		}
        
        launchTask();
    }
    
	private void launchTask(){
		mTimer = new Timer();
		mTimer.scheduleAtFixedRate(new TimerTask() {
			public void run() {
				// Si es una prueba de geolocalización y se ha movido, actualizo.
				// Si la prueba ha sido superada, no actualizo el mapa para pasar al
				// siguiente reto lo más rápido posible.
				if (Proof.lastProofType == 3 && 
					lastDistanceToPlace != DistanceControllerService.distanceToPlace &&
					DistanceControllerService.distanceToPlace > Proof.lastMinDistance){
					handlerPosition.sendEmptyMessage(0);
					
					lastDistanceToPlace = DistanceControllerService.distanceToPlace;
				}else if (Proof.lastProofType != 3){
					handlerPosition.sendEmptyMessage(0);
				}
			}
		}, DELAY, PERIOD);
	}
	
	private Handler handlerPosition = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			mapOverlays.remove(itemizedOverlay);
			//itemizedOverlay.clearOverlay();
			Drawable drawable = TeamLocationMap.this.getResources().getDrawable(R.drawable.user_48);
	        itemizedOverlay = new MapLayer(TeamLocationMap.this,drawable);
			mapOverlays = mapView.getOverlays();
			// We can modify the GUI here.
			double lat = Home.currentLocation.getLatitude();
			double lng = Home.currentLocation.getLongitude();
			GeoPoint geoPoint = new GeoPoint((int) (lat * 1E6), (int) (lng * 1E6));
    	    
			User user = LibreGeoSocial.getInstance().getInfoUser(false, null);
			OverlayItem overlayItem = new OverlayItem(geoPoint, TeamList.team.getGroupName(), 
					user.getUsername() + " - " + TeamList.team.getGroupName() + 
					" - You Are Here");
    	        
			itemizedOverlay.addOverlay(overlayItem);
			mapOverlays.add(itemizedOverlay);
			
			MapController mapController = mapView.getController();
	        mapController.animateTo(geoPoint);
		}
	};
    
    @Override
    protected boolean isRouteDisplayed() {
        return false;
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        
        menu.add(0, MENU_INFORMATION, 0 ,"Information")
        .setIcon(R.drawable.info)
        .setAlphabeticShortcut('I');
        return true;
    }
	
    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	switch (item.getItemId()) {
    		case MENU_INFORMATION:
    			showInformationDialog();
    			break;
    	}
    	//return false;
    	return super.onOptionsItemSelected(item);
    }
    
    private void showInformationDialog(){
    	AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	String text = "En esta pantalla podrás conocer la posición en la que te "
    				+ "encuentras.\n\nAdemás, con tan sólo pulsar sobre el mapa, "
    				+ "aparecerá en pantalla un control que te permitirá aumentar "
    				+ "o disminuir el zoom, al mismo tiempo que podrás mover el "
    				+ "mapa con tan sólo deslizar el dedo sobre la pantalla táctil "
    				+ "del teléfono.";
    	builder.setMessage(text)
    	       .setCancelable(false)
    	       .setPositiveButton("OK", new DialogInterface.OnClickListener() {
    	           public void onClick(DialogInterface dialog, int id) {
    	        	   dialog.cancel();
    	           }
    	       });
    	AlertDialog alert = builder.create();
    	alert.setTitle("Instructions");
    	alert.setIcon(R.drawable.info);
    	alert.show();
    }
    
	// Registro un receptor de mensajes en broadcast porque es la manera de notificar
	// si se ha alcanzado el lugar objetivo de una prueba de geolocalización, por lo
	// que habrá que pasar a la siguiente prueba.
    private BroadcastReceiver receiver = new BroadcastReceiver() {
        
        @Override
        public void onReceive(Context context, Intent intent) {
       	 	mTimer.cancel();
		    unregisterReceiver(receiver);
		    Competition.FINISH_SW = true;
       	 	TeamLocationMap.this.setResult(RESULT_OK);
       	 	TeamLocationMap.this.finish();
        }
    };

    protected void onResume(){
    	super.onResume();
    	//this.launchTask();
    	if (Proof.lastProofType == 3 && !Proof.targetPlaceFound){
    		registerReceiver(receiver, new IntentFilter(Intent.ACTION_VIEW));
    		Log.d(TAG, "teamLocationMap ha registrado su receiver");
    	}
    }

    @Override
    protected void onPause(){
    	super.onPause();
    }
    
    @Override
	public void onDestroy(){
    	super.onDestroy();
		mTimer.cancel();
	}
    
    public boolean onKeyDown(int keyCode, KeyEvent event){
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            // TECLA ATRAS PULSADA
        	if (!Competition.FINISH_SW){
        		AlertDialog.Builder builder = new AlertDialog.Builder(this);
    	    	builder.setMessage("Are you sure you want to exit?")
    		           .setCancelable(false)
    		           .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
    		               public void onClick(DialogInterface dialog, int id) {
			            	   	Intent messagingService = new Intent(TeamLocationMap.this, MessagingService.class);
			            	   	TeamLocationMap.this.stopService(messagingService);
			       	        	MessagingService.MESSAGING_SERVICE_ENABLED = false;
			       	        	Intent gymkhanaLocationService = new Intent(TeamLocationMap.this, GymkhanaLocationListener.class);
			       	        	TeamLocationMap.this.stopService(gymkhanaLocationService);
			       	        	Intent locationService = new Intent(TeamLocationMap.this, LocationService.class);
			       	        	startService(locationService);
			       	        	TeamLocationMap.this.setResult(RESULT_OK);
			       	        	TeamLocationMap.this.finish();
    		               }
    		           })
    		           .setNegativeButton("No", new DialogInterface.OnClickListener() {
    		               public void onClick(DialogInterface dialog, int id) {
    		                    dialog.cancel();
    		               }
    		           });
    		    AlertDialog alert = builder.create();
    		    alert.setTitle("Warning");
    		    alert.setIcon(R.drawable.warning);
    		    alert.show();
        	}
        	return true;
        }else{
        	return false;
        }
    }

}