/*
 *
 *  Copyright (C) 2010 GSyC/LibreSoft, Universidad Rey Juan Carlos.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Raúl Román López <rroman@gsyc.es>
 *
 */

package com.LibreGeoSocialApp.Tagging;


import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.provider.MediaStore;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.LibreGeoSocial.Api.LibreGeoSocial;
import com.LibreGeoSocial.Api.Photo;
import com.LibreGeoSocialApp.LGS_Preferences;
import com.LibreGeoSocialApp.R;
import com.LibreGeoSocialApp.Utils.BitmapUtils;




public class UploadPhotoTag extends Activity implements OnClickListener {

	static Button btTakePhoto;
	static Button btSelectPicture;
	static Button btUpload;
	static Button btPrivacy;

	static TextView txtDescription;
	static TextView txtTitle;
	static ImageView imgPhoto;
	
	static String strImagePath = null; 
	
	static float latitude;
	static float longitude;
	static float altitude;
	
	private static final int MENU_TAKE_PHOTO = Menu.FIRST +1;
	
	private static final int REQUEST_SELECT_PHOTO = 1;
	
    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.camera);
        
        latitude = getIntent().getExtras().getFloat("LATITUDE", -1);
        longitude = getIntent().getExtras().getFloat("LONGITUDE", -1);
        altitude = getIntent().getExtras().getFloat("ALTITUDE", -1);
        
        btSelectPicture = (Button)this.findViewById(R.id.btSelectPicture); 
        btSelectPicture.setOnClickListener(this);
        
        btUpload = (Button)this.findViewById(R.id.btUpload); 
        btUpload.setOnClickListener(this);
        
        btPrivacy = (Button)this.findViewById(R.id.btPrivacy); 
        btPrivacy.setOnClickListener(this);
        
        imgPhoto = (ImageView) this.findViewById(R.id.imgPhoto);
 
        
        txtDescription = (TextView)this.findViewById(R.id.txtComment);
        
        txtTitle = (TextView)this.findViewById(R.id.txtTitle);
        
    }
    
    
    public static boolean isSdPresent() 
    {
    	return android.os.Environment.getExternalStorageState().equals(android.os.Environment.MEDIA_MOUNTED);
    }


	public void onClick(View arg0) {

		
		if (btPrivacy.isPressed())
		{
			Intent i = new Intent(this,LGS_Preferences.class);
			startActivity(i);
		}    	
		else if (btSelectPicture.isPressed())
		{
			doSelectImage();			
			
		}
		else if (btUpload.isPressed())
		{
						 						
			Photo myPhoto = new Photo(0,
							(double) latitude,
							(double) longitude,
							(double) altitude,
							0.0,
							txtTitle.getText().toString(),
							txtDescription.getText().toString(),
							strImagePath,
							null,
							null);
			
			if ( LibreGeoSocial.getInstance().uploadPhoto(myPhoto) )
			{
				
				// Get the photo privacy
				
                // User text   
        	    SharedPreferences sharedPreferences = 
        	    	PreferenceManager.getDefaultSharedPreferences(getBaseContext());

        		// Update state in twitter
        		String photoPrivacy = 
        			sharedPreferences.getString (LGS_Preferences.KEY_APP_PHOTO_PRIVACY, "friends");
				
				// Get the photo id
				Integer id = LibreGeoSocial.getInstance().getLastId();
					
				if ( LibreGeoSocial.getInstance().setPrivacyNode (id, LibreGeoSocial.getInstance().PRIVACY_FRIENDS, false))
				
	        		Toast.makeText(getBaseContext(), 
	  					   "Upload photo correctly!!", 
	  					   Toast.LENGTH_LONG).show();
				
				else
					
					Toast.makeText(getBaseContext(), 
		  					   "The photo was upload correctly, but there is a problem with the privacy!", 
		  					   Toast.LENGTH_LONG).show();
					
			}
			else
			{
        		Toast.makeText(getBaseContext(), 
  					   "Upload photo error: " + 
  					   LibreGeoSocial.getInstance().getLastMsgServer(), 
  					   Toast.LENGTH_LONG).show();
			}
			finish();
		}
	}
    
   
    
	private void doSelectImage() {
		
    	Intent i = new Intent(Intent.ACTION_PICK) ;
    	i.setDataAndType(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, 
    					 MediaStore.Images.Media.CONTENT_TYPE) ;
    	
    	startActivityForResult(i,REQUEST_SELECT_PHOTO) ;
    }
	
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	
        super.onCreateOptionsMenu(menu);
        // NOT ON 1.1
        menu.add(0, MENU_TAKE_PHOTO, 0, "Take photo")
        		.setIcon(R.drawable.camera_38)
                .setAlphabeticShortcut('T'); 
        
        return true;
    }

    @Override
    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    	
    	// NOT IN 1.1
    	case MENU_TAKE_PHOTO:
    		Intent i = new Intent(MediaStore.INTENT_ACTION_STILL_IMAGE_CAMERA); // 1.5
        	startActivity(i);
    		break;    	
        }
    	
        return super.onOptionsItemSelected(item);
    }
    
	protected void onActivityResult (int requestCode, int resultCode, Intent data) { 
		
		switch (requestCode) { 
		
    		
	    	case REQUEST_SELECT_PHOTO:
	    		
	    		if( resultCode != 0 ) {
	    			
		    		Cursor c = managedQuery(data.getData(),null,null,null,null) ;
		    		
		    		if( c.moveToFirst() ) {
		    			try{
		    				String media_path = c.getString(1) ;

		    				Log.d("LibreGeoSocial","onActivityResult("+requestCode+","+resultCode+","+media_path+")") ;

		    				strImagePath = media_path;

		    				Bitmap photo = BitmapUtils.loadBitMapFromFile(strImagePath);

		    				imgPhoto.setImageBitmap(photo);
		    			}catch(Exception e){
		    				Toast.makeText(getBaseContext(), 
		    	  					   e.toString() + 
		    	  					   LibreGeoSocial.getInstance().getLastMsgServer(), 
		    	  					   Toast.LENGTH_LONG).show();
		    			}
		    		}		    		
	    		} 
	    		
	    		break;
	
            default:
                break; 
		}
	}

}
