/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Raúl Román López <rroman@gsyc.es>
 *
 */


package com.SensorTesting;

import android.app.Activity;
import android.os.Bundle;
import android.content.Context;
import android.hardware.Camera;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.Window;

public class CameraExample extends Activity {

	private CamPreview mPreview;

	protected void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
        
            // Hide the window title.
            requestWindowFeature(Window.FEATURE_NO_TITLE);
        
            // Create our Preview view and set it as the content of our activity.
            mPreview = new CamPreview(this);
        
            setContentView(mPreview);
	}

	public class CamPreview extends SurfaceView implements SurfaceHolder.Callback {
	    private SurfaceHolder mHolder;
	    private Camera mCamera;
    
	    public CamPreview(Context context) {
	      super(context);
        
	      // Install a SurfaceHolder.Callback so we get notified when the
	      // underlying surface is created and destroyed.
	      mHolder = getHolder();
	      mHolder.addCallback(this);
	      mHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
	    }

	    public void surfaceCreated(SurfaceHolder holder) {
	      // The Surface has been created, acquire the camera and tell it where
              // to draw.
 	      mCamera = Camera.open();
      	      mCamera.setPreviewDisplay(holder);
    	    }

	    public void surfaceDestroyed(SurfaceHolder holder) {
	      // Surface will be destroyed when we return, so stop the preview.
	      // Because the CameraDevice object is not a shared resource, it's very
	      // important to release it when the activity is paused.
	      mCamera.stopPreview();
	      //IMPORTANT: releasing the camera:
	      mCamera.release();
	      mCamera = null;
	    }

    
	    public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
	      // Now that the size is known, set up the camera parameters and begin
	      // the preview.
	      Camera.Parameters parameters = mCamera.getParameters();
	      parameters.setPreviewSize(w, h);
	      mCamera.setParameters(parameters);
	      mCamera.startPreview();
	    }

	}
}