/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Raúl Román López <rroman@gsyc.es>
 *
 */

package com.SensorTesting;

import java.lang.reflect.Method;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.provider.Settings;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SubMenu;
import android.widget.Toast;
import android.widget.TextView;

public class GPSExample extends Activity{
	
	private static final int MENU_TIME_INTERVAL = Menu.FIRST +1;
	private static final int MENU_TIME_INTERVAL_1000 = Menu.FIRST +2;
	private static final int MENU_TIME_INTERVAL_2000 = Menu.FIRST +3;
	private static final int MENU_TIME_INTERVAL_5000 = Menu.FIRST +4;
	private static final int MENU_TIME_INTERVAL_10000 = Menu.FIRST +5;
	private static final int MENU_TIME_INTERVAL_0 = Menu.FIRST +6;
	
	private static final int MENU_SPACE_INTERVAL = Menu.FIRST +7;
	private static final int MENU_SPACE_INTERVAL_1 = Menu.FIRST +8;
	private static final int MENU_SPACE_INTERVAL_10 = Menu.FIRST +9;
	private static final int MENU_SPACE_INTERVAL_100 = Menu.FIRST +10;
	private static final int MENU_SPACE_INTERVAL_500 = Menu.FIRST +11;
	private static final int MENU_SPACE_INTERVAL_1000 = Menu.FIRST +12;
	private static final int MENU_SPACE_INTERVAL_0 = Menu.FIRST +13;
	
	
	private static LocationManager mLocationManager;
	private static MyLocationListener mLocationListener;
	private static TextView t_latitude;
	private static TextView t_longitude;
	
	private static long time_delay;
	private static float space_delay;
	
	protected void onCreate(Bundle savedInstanceState) {
          super.onCreate(savedInstanceState);
          setContentView(R.layout.gps);
        
          t_latitude = (TextView) findViewById(R.id.gps_lat);
          t_longitude = (TextView) findViewById(R.id.gps_lng);

          time_delay = 0;
          space_delay = 0;
        
	}

	@Override
	protected void onPause(){
	  super.onPause();
	  if((mLocationManager!=null) && (mLocationListener!=null)){
		mLocationManager.removeUpdates(mLocationListener);
		setGPSStatus(false);
	  }
	}
	
	@Override
	protected void onResume(){
		super.onResume();
		
		upGPS();
	}

	private void upGPS(){
		String locationType = LocationManager.GPS_PROVIDER;
		try {
      	  mLocationManager = (LocationManager)getSystemService(
      			  Context.LOCATION_SERVICE);
      	  if (!getGPSStatus())
      		  setGPSStatus(true);

      	  if (mLocationManager.isProviderEnabled(locationType)) {

      		  mLocationListener = new MyLocationListener();

      		  mLocationManager.requestLocationUpdates(
      				  locationType, time_delay, space_delay, mLocationListener);

      	  } else {
      		  Toast.makeText(getBaseContext(), 
      				  "Unable to locate", 
      				  Toast.LENGTH_LONG).show();

      	  }
        } catch (Exception e) {
      	  // TODO Auto-generated catch block
      	  Toast.makeText(getBaseContext(), 
      			  e.toString(), 
      			  Toast.LENGTH_LONG).show();
        }
	}
	
	private boolean getGPSStatus(){
	    String allowedLocationProviders =
	        Settings.System.getString(getContentResolver(),
	        Settings.System.LOCATION_PROVIDERS_ALLOWED);
	 
	    if (allowedLocationProviders == null) {
	        allowedLocationProviders = "";
	    }
	 
	    return allowedLocationProviders.contains(LocationManager.GPS_PROVIDER);
	}

	private void setGPSStatus(boolean pNewGPSStatus){
	    String allowedLocationProviders =
	        Settings.System.getString(getContentResolver(),
	        Settings.System.LOCATION_PROVIDERS_ALLOWED);
	 
	    if (allowedLocationProviders == null) {
	        allowedLocationProviders = "";
	    }      
	 
	    boolean networkProviderStatus =
	        allowedLocationProviders.contains(LocationManager.NETWORK_PROVIDER);
	 
	    allowedLocationProviders = "";
	    if (networkProviderStatus == true) {
	        allowedLocationProviders += LocationManager.NETWORK_PROVIDER;
	    }
	    if (pNewGPSStatus == true) {
	        allowedLocationProviders += "," + LocationManager.GPS_PROVIDER;
	    }   
	 
	    Settings.System.putString(getContentResolver(),
	        Settings.System.LOCATION_PROVIDERS_ALLOWED, allowedLocationProviders);     
 
	    try
	    {
	        Method m =
	            mLocationManager.getClass().getMethod("updateProviders", 
								new Class[] {});
	        m.setAccessible(true);
	        m.invoke(mLocationManager, new Object[]{});
	    }
	    catch(Exception e){
	    }
	    return;
	}

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	SubMenu sub1 = menu.addSubMenu (0, MENU_TIME_INTERVAL, 0 , "Time delay");
    	sub1.add(0,MENU_TIME_INTERVAL_0,0, "0 sec.");
    	sub1.add(0,MENU_TIME_INTERVAL_1000,0, "1 sec.");
    	sub1.add(0,MENU_TIME_INTERVAL_2000,0,"2 sec.");
    	sub1.add(0,MENU_TIME_INTERVAL_5000,0,"5 sec.");
    	sub1.add(0,MENU_TIME_INTERVAL_10000,0,"10 sec.");
    	
    	SubMenu sub2 = menu.addSubMenu (0, MENU_SPACE_INTERVAL, 0 , "Space delay");

    	sub2.add(0,MENU_SPACE_INTERVAL_0,0, "0 m.");
    	sub2.add(0,MENU_SPACE_INTERVAL_1,0, "1 m.");
    	sub2.add(0,MENU_SPACE_INTERVAL_10,0,"10 m.");
    	sub2.add(0,MENU_SPACE_INTERVAL_100,0,"100 m.");
    	sub2.add(0,MENU_SPACE_INTERVAL_500,0,"0.5 Km.");
    	sub2.add(0,MENU_SPACE_INTERVAL_1000,0,"1 Km.");
        
        super.onCreateOptionsMenu(menu);        
        return true;
    }

    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    	
    	case MENU_TIME_INTERVAL_1000:
    		time_delay = 1000;
    		break;
    		
    	case MENU_TIME_INTERVAL_2000:
    		time_delay = 2000;
    		break;
    		
    	case MENU_TIME_INTERVAL_5000:
    		time_delay = 5000;
    		break;
    		
    	case MENU_TIME_INTERVAL_10000:
    		time_delay = 10000;
    		break;
    		
    	case MENU_TIME_INTERVAL_0:
    		time_delay = 0;
    		break;
    		
    	case MENU_SPACE_INTERVAL_0:
    		space_delay = 0;
    		break;	
    		
    	case MENU_SPACE_INTERVAL_1:
    		space_delay = 1;
    		break;
    		
    	case MENU_SPACE_INTERVAL_10:
    		space_delay = 10;
    		break;
    		
    	case MENU_SPACE_INTERVAL_100:
    		space_delay = 100;
    		break;
    		
    	case MENU_SPACE_INTERVAL_500:
    		space_delay = 500;
    		break;
    		
    	case MENU_SPACE_INTERVAL_1000:
    		space_delay = 1000;
    		break;
    	
        }
    	
    	if((mLocationManager!=null) && (mLocationListener!=null)){
    		mLocationManager.removeUpdates(mLocationListener);
    	}
    	
    	upGPS();
    	
        return super.onOptionsItemSelected(item);
    }
	
	
	private class MyLocationListener implements LocationListener{

          public void onLocationChanged(Location loc) {
            if (loc != null) {
            	t_latitude.setText(Double.toString(loc.getLatitude()));
            	t_longitude.setText(Double.toString(loc.getLongitude()));
            }
          }

          public void onProviderDisabled(String provider) {
            // TODO Auto-generated method stub
          }


          public void onProviderEnabled(String provider) {
            // TODO Auto-generated method stub
          }


          public void onStatusChanged(String provider, int status, 
	            Bundle extras) {
            // TODO Auto-generated method stub
          }
	}
}