/*
 *
 *  Copyright (C) 2009 GSyC/LibreSoft
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/. 
 *
 *  Author : Raúl Román López <rroman@gsyc.es>
 *
 */

package com.SensorTesting;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.hardware.SensorListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SubMenu;
import android.widget.TextView;
import android.widget.Toast;

public class SensorTesting extends Activity {
	
    private static final int MENU_DELAY = Menu.FIRST + 1;
    private static final int MENU_DELAY_UI = Menu.FIRST + 2;
    private static final int MENU_DELAY_GAME = Menu.FIRST + 3;
    private static final int MENU_DELAY_NORMAL = Menu.FIRST + 4;
    private static final int MENU_DELAY_FASTEST = Menu.FIRST + 5;
    private static final int MENU_SENSORS = Menu.FIRST + 6;
	private static final int MENU_SENSOR_ACCELEROMETER = Menu.FIRST + 7;
	private static final int MENU_SENSOR_MAGNETIC = Menu.FIRST + 8;
	private static final int MENU_SENSOR_ORIENTATION = Menu.FIRST + 9;
	private static final int MENU_SENSOR_ORIENTATION_RAW = Menu.FIRST + 10;
	private static final int MENU_SENSOR_CAMERA = Menu.FIRST + 11;
	private static final int MENU_SENSOR_GPS = Menu.FIRST + 12;
	
	private static TextView sensor_name;
	private static TextView x;
	private static TextView y;
	private static TextView z;
	private static TextView x_raw;
	private static TextView y_raw;
	private static TextView z_raw;
	
	private static SensorManager sm;
	private static MSensListener sensorListener;
	private static int time_delay;
	private static int sensor = -1;
    
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        sensor_name = (TextView) findViewById(R.id.sens_name);
        x = (TextView) findViewById(R.id.sens_value_x);
        y = (TextView) findViewById(R.id.sens_value_y);
        z = (TextView) findViewById(R.id.sens_value_z);
        x_raw = (TextView) findViewById(R.id.sens_value_x_raw);
        y_raw = (TextView) findViewById(R.id.sens_value_y_raw);
        z_raw = (TextView) findViewById(R.id.sens_value_z_raw);
        
        time_delay = SensorManager.SENSOR_DELAY_NORMAL;
        sm = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
        
        Toast.makeText(getBaseContext(), 
                "Choose a sensor in the menu", 
                Toast.LENGTH_LONG).show();
    }
    
    private void setSensor(){
    	if (sensorListener == null)
    		sensorListener = new MSensListener();
        if ((sm.getSensors() & sensor) == sensor){
        	sm.registerListener(sensorListener, sensor, time_delay);
        }else{
        	x.setText("Error");
        }
    }
    
    protected void onPause(){
    	super.onPause();
    	sm.unregisterListener(sensorListener);
    }
    
    protected void onResume(){
    	super.onResume();
    	if(sensor != -1)
    		setSensor();
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
    	SubMenu sub1 = menu.addSubMenu(0, MENU_DELAY, 0 ,"Set delay");
    	sub1.add(0,MENU_DELAY_NORMAL,0, "Normal");
    	sub1.add(0,MENU_DELAY_UI,0, "UI");
    	sub1.add(0,MENU_DELAY_GAME,0,"Game");
    	sub1.add(0,MENU_DELAY_FASTEST,0,"Fastest");
    	
    	SubMenu sub2 = menu.addSubMenu (0, MENU_SENSORS, 0 , "Sensors");
    	sub2.add(0,MENU_SENSOR_ACCELEROMETER,0, "Accelerometer");
    	sub2.add(0,MENU_SENSOR_MAGNETIC,0, "Magnetic Field");
    	sub2.add(0,MENU_SENSOR_ORIENTATION,0,"Orientation");
    	sub2.add(0,MENU_SENSOR_ORIENTATION_RAW,0,"Orientation RAW");
    	sub2.add(0,MENU_SENSOR_CAMERA,0,"Camera Preview");
    	sub2.add(0,MENU_SENSOR_GPS,0,"GPS");
        
        super.onCreateOptionsMenu(menu);        
        return true;
    }

    public boolean onOptionsItemSelected (MenuItem item) {
    	
    	switch (item.getItemId()) {
    	
    	case MENU_DELAY_NORMAL:
    		time_delay = SensorManager.SENSOR_DELAY_NORMAL;
    		break;
    		
    	case MENU_DELAY_UI:
    		time_delay = SensorManager.SENSOR_DELAY_UI;
    		break;
    		
    	case MENU_DELAY_GAME:
    		time_delay = SensorManager.SENSOR_DELAY_GAME;
    		break;
    		
    	case MENU_DELAY_FASTEST:
    		time_delay = SensorManager.SENSOR_DELAY_FASTEST;
    		break;
    		
    	case MENU_SENSOR_ACCELEROMETER:
    		sensor_name.setText("Accelerometer");
    		
    		sensor = SensorManager.SENSOR_ACCELEROMETER;
    		
    		break;
    		
    	case MENU_SENSOR_MAGNETIC:
    		sensor_name.setText("Magnetic Field");
    		
    		sensor = SensorManager.SENSOR_MAGNETIC_FIELD;
    		break;
    	
    	case MENU_SENSOR_ORIENTATION:
    		sensor_name.setText("Orientation");
    		
    		sensor = SensorManager.SENSOR_ORIENTATION;
    		break;
    	
    	case MENU_SENSOR_ORIENTATION_RAW:
    		sensor_name.setText("Orientation RAW");
    		
    		sensor = SensorManager.SENSOR_ORIENTATION_RAW;
    		break;
    	
    	case MENU_SENSOR_CAMERA:
    		Intent i = new Intent(this,CameraExample.class);
        	startActivity(i); 
        	return super.onOptionsItemSelected(item);
    		
    	case MENU_SENSOR_GPS:
    		Intent i1 = new Intent(this,GPSExample.class);
        	startActivity(i1);
        	return super.onOptionsItemSelected(item);
        }
    	
    	if(sensor!=-1){
    		sm.unregisterListener(sensorListener);
    		setSensor();
    	}
    	
        return super.onOptionsItemSelected(item);
    }
    
    
    private class MSensListener implements SensorListener{

    	@Override
    	public void onAccuracyChanged(int sensor, int accuracy) {
    	  // TODO Auto-generated method stub
    		
    	}

    	@Override
    	public void onSensorChanged(int sensor, float[] values) {
    	  // TODO Auto-generated method stub
    		x.setText(Float.toString(values[0]));
    		y.setText(Float.toString(values[1]));
    		z.setText(Float.toString(values[2]));
    		x_raw.setText(Float.toString(values[3]));
    		y_raw.setText(Float.toString(values[4]));
    		z_raw.setText(Float.toString(values[5]));
    		
    	}
    }
}